package com.bocloud.cmp.config;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.Butler;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.megatron.common.encrypt.DESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.framework.registry.RegistryClient;

import dm.jdbc.util.StringUtil;
import lombok.extern.slf4j.Slf4j;

@Service
@Slf4j
public class ButlerConfig {

	@Autowired
	private RegistryClient serviceClient;
	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	static Encryptor encryptor = new DESEncryptor();
	static final String DEFAULT_KEY = "BocloudCMP";

	public Butler butler(String uuid) {
		return butler(uuid, null, null);
	}

	public Butler projectButler(String uuid, String projectName) {
		return butler(uuid, projectName, null);
	}

	public Butler regionButler(String uuid, String region) {
		return butler(uuid, null, region);
	}

	public Butler butler(String uuid, String projectName, String region) {
		try {
			String butler = serviceClient.getButler();
			if (StringUtil.isEmpty(uuid) || StringUtil.isEmpty(butler)) {
				log.error("uuid or butler is null !!!");
				return null;
			}
			
			JSONObject butlerObject = JSON.parseObject(butler);
			if(butlerObject.containsKey(uuid)) {
				Butler newButler = JSON.parseObject(butlerObject.getJSONObject(uuid).toJSONString(), Butler.class);
				if(null != newButler) {
					String password = newButler.getPassword();
					if (!StringUtils.isEmpty(password)) {
		            	newButler.setPassword(encryptor.decrypt(password, DEFAULT_KEY));
		            }
					if (!StringUtils.isEmpty(newButler.getAccessKey())) {
		            	newButler.setAccessKey(encryptor.decrypt(newButler.getAccessKey(), DEFAULT_KEY));
		            }
		            if (!StringUtils.isEmpty(newButler.getSecretKey())) {
		            	newButler.setSecretKey(encryptor.decrypt(newButler.getSecretKey(), DEFAULT_KEY));
		            }
					if (StringUtils.isNotEmpty(region) && !region.equalsIgnoreCase(newButler.getRegionId())) {
						newButler.setRegionId(region);
					}
					if (!StringUtils.isEmpty(projectName)) {
						newButler.setProjectName(projectName);
						newButler.setTenant(projectName);
					}
					return newButler;
				}
			}
		} catch (Exception e) {
			log.error("获取butler异常：" + e.getMessage());
		}
		return findButler(uuid, projectName, region);
	}

	private Butler findButler(String uuid, String projectName, String region) {
		CloudVendor cloudVendor = cloudVendorRepository.query(uuid);
		if(null == cloudVendor) {
			return null;
		}
		Butler butler  = new Butler();
		JSONObject authentication = JSONObject.parseObject(cloudVendor.getAuthentication());
		String regions = authentication.getString("regions") == null ? "" : authentication.getString("regions");
		
		butler.setName(cloudVendor.getName());
		butler.setUuid(cloudVendor.getUuid());
		butler.setUsername(authentication.getString("account"));
		butler.setRegions(regions);
		butler.setUrl(authentication.getString("address"));
		butler.setCategory(cloudVendor.getType());
		butler.setDomainName(authentication.getString("domainName"));
		butler.setVersion(cloudVendor.getVersion());
		butler.setToken(authentication.getString("token"));
		butler.setProjectId(authentication.getString("projectId"));
		butler.setSubscriptionId(authentication.getString("subscriptionId"));
		butler.setClientId(authentication.getString("clientId"));
		butler.setVersion(authentication.getString("version"));
		
		String password = authentication.getString("password");
		if(!StringUtil.isEmpty(password)) {
			butler.setPassword(encryptor.decrypt(password, DEFAULT_KEY));
		}
		String accessKey = authentication.getString("accessKey");
		if(!StringUtil.isEmpty(accessKey)) {
			butler.setPassword(encryptor.decrypt(accessKey, DEFAULT_KEY));
		}
		String secretKey = authentication.getString("secretKey");
		if(!StringUtil.isEmpty(secretKey)) {
			butler.setPassword(encryptor.decrypt(secretKey, DEFAULT_KEY));
		}
		if (StringUtils.isNotEmpty(region)) {
			butler.setRegionId(region);
		}
		if (!StringUtils.isEmpty(projectName)) {
			butler.setProjectName(projectName);
			butler.setTenant(projectName);
		}
		
		return butler;
	}
	
	

}
