package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.service.CloudServerService;
import com.bocloud.cmp.service.CloudVendorService;
import com.bocloud.cmp.service.ServerService;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@Tag(name = "BSM接口")
@RestController
@RequestMapping("/v1/bsm")
public class BsmController {

    @Autowired
    private CloudVendorService cloudVendorService;
    @Autowired
    private CloudServerService cloudServerService;
    @Autowired
    @Qualifier("ServerService")
    private ServerService serverService;

    private RequestContext getRequestContext() {
        RequestContext context = new RequestContext();
        context.setTarget(1L);
        context.setCatalog(RequestContext.Catalog.Manager);
        return context;
    }

    private boolean checkToken(String token) {
        token = new AESEncryptor().decrypt(token, "");
        if (!token.startsWith("Bsm+Cmp@v587")) return false;
        String time = token.replace("Bsm+Cmp@v587:", "");
        if (System.currentTimeMillis() - Long.parseLong(time) > 30000) return false;
        return true;
    }

    @Operation(tags = {"CMC"}, summary = "获取云供应商列表")
    @GetMapping("/vendors")
    public GeneralResult<GridBean<CloudVendor>> listVendors(Pager pager, @RequestHeader(value = "bsm-token") String token) {
        if (!checkToken(token)) {
            return new GeneralResult<>(false, "TOKEN异常");
        }
        return cloudVendorService.list(pager, this.getRequestContext());
    }

    @Operation(tags = {"CMC"}, summary = "获取物理服务器列表")
    @GetMapping("/hosts")
    public GeneralResult<GridBean> listHosts(Pager pager, @RequestHeader(value = "bsm-token") String token) {
        if (!checkToken(token)) {
            return new GeneralResult<>(false, "TOKEN异常");
        }
        return serverService.list(pager);
    }

    @Operation(tags = {"CMC"}, summary = "查询云主机列表")
    @GetMapping("/vms")
    public GeneralResult<GridBean<CloudServer>> listVms(Pager pager , @RequestHeader(value = "bsm-token") String token) {
        if (!checkToken(token)) {
            return new GeneralResult<>(false, "TOKEN异常");
        }
        return cloudServerService.list(pager, this.getRequestContext());
    }
}
