package com.bocloud.cmp.controller;

import com.bocloud.cmp.domain.CloudVendorBean;
import com.bocloud.cmp.service.CloudVendorTaskService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 定时任务 cron
 *
 * @author ymh
 * @时间 2019年4月8日
 */
@RestController
@RequestMapping("/v1/timedtask")
@Tag(name = "定时任务管理")
public class CloudVendorTaskController {

    @Autowired
    private CloudVendorTaskService cloudVendorTaskService;

    /**
     * 添加定时任务
     *
     * @param id
     * @param context
     * @return
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "新增定时任务管理")
    public GeneralResult create(@RequestParam(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorTaskService.create(id, context);
    }

    /**
     * 取消平台定时任务
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "取消定时任务")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorTaskService.remove(id, context);
    }

    /**
     * 更新平台定时任务
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "更新平台同步评率")
    @PutMapping(value = "/{id}")
    public GeneralResult modify(@PathVariable(value = Common.ID) Long id, @RequestBody CloudVendorBean cloudVendorBean,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudVendorTaskService.modify(cloudVendorBean, context);
    }

}
