package com.bocloud.cmp.controller;

import com.bocloud.cmp.domain.DictionaryBean;
import com.bocloud.cmp.entity.Dictionary;
import com.bocloud.cmp.service.DictService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 字典信息控制器
 *
 * @author dmw
 */
@RestController
@RequestMapping("/v1/dictionaries")
@Tag(name = "字典信息管理")
public class DictController {
    private final DictService dictService;

    public DictController(DictService dictService) {
        this.dictService = dictService;
    }

    @Operation(tags = {"CMC", "CSC"}, summary = "查询字典树列表")
    @GetMapping
    public GeneralResult<List<Dictionary>> list(@RequestParam(value = "pid", required = false) Long pid) {
        return dictService.list(pid);
    }

    /**
     * 根据字典值获取子节点列表
     *
     * @param value
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询系统字典子信息")
    @GetMapping("/children")
    public GeneralResult<List<Dictionary>> children(@RequestParam(value = Common.VALUE, required = false) String value) {
        return dictService.listChild(value);
    }

    /**
     * 查询字典树
     *
     * @param value
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询字典树")
    @GetMapping("/children/tree")
    public GeneralResult<List<Dictionary>> childTree(@RequestParam(value = Common.VALUE, required = false) String value) {
        return dictService.childTree(value);
    }

    /**
     * 创建
     *
     * @param dictionary
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "新增字典")
    @PostMapping
    public GeneralResult<Dictionary> create(@RequestBody Dictionary dictionary) {
        return dictService.create(dictionary);
    }

    /**
     * 删除字典
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "删除字典")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id) {
        return dictService.remove(id);
    }

    /**
     * 修改字典
     *
     * @param id
     * @param bean
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改字典")
    @PutMapping("/{id}")
    public GeneralResult<Dictionary> modify(@PathVariable(value = Common.ID) Long id,
                                            @RequestBody DictionaryBean bean) {
        bean.setId(id);
        return dictService.modify(bean);
    }

}
