package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.ElasticIp;
import com.bocloud.cmp.service.ElasticIpService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/v1/elasticips")
@Tag(name = "弹性IP")
public class ElasticIpController {

    @Autowired
    private ElasticIpService elasticIpService;

    @Operation(tags = {"CMC", "CSC"}, summary = "查询弹性ip列表")
    @GetMapping
    public GeneralResult<GridBean<ElasticIp>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return elasticIpService.list(pager, context);
    }

    /**
     * 查看ip明细
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看ip明细")
    @GetMapping("/{id}")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id) {
        return elasticIpService.detail(id);
    }

    /***
     * 查询平台下未关联EIP
     *
     * @param params
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询未关联EIP")
    @GetMapping("/frees")
    public GeneralResult listFreeEips(@RequestParam(value = Common.VENDORID, required = false) Long vendorId,
                                      @RequestParam(value = Common.REGIONID, required = false) String regionId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return elasticIpService.listFrees(vendorId, regionId, context);
    }

}
