package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.FloatingIp;
import com.bocloud.cmp.service.FloatingIpService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/v1/floatips")
@Tag(name = "浮动IP")
public class FloatingIpController {

    @Autowired
    private FloatingIpService floatingIpService;

    /**
     * 查看浮动IP列表
     *
     * @param pager
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询浮动IP列表")
    @GetMapping
    public GeneralResult<GridBean<FloatingIp>> list(Pager pager,
                                                    @RequestParam(value = Common.CONDITION, required = false) String condition,
                                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return floatingIpService.list(pager, context);
    }

    /**
     * 根据condition查询浮动IP列表
     *
     * @param condition
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "根据condition查询浮动IP列表")
    @GetMapping("/condition")
    public GeneralResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition) {
        return floatingIpService.condition(condition);
    }

    /**
     * 查看浮动IP详情
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "浮动IP详情")
    @GetMapping("/{id}")
    public GeneralResult<FloatingIp> detail(@PathVariable(value = Common.ID) Long id) {
        return floatingIpService.detail(id);
    }

}
