package com.bocloud.cmp.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.LoadBalancer;
import com.bocloud.cmp.service.LoadBalancerService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/v1/lbs")
@Tag(name = "负载均衡器")
public class LoadBalancerController {

    @Autowired
    private LoadBalancerService loadBalancerService;

    /**
     * 查看负载均衡器列表
     *
     * @param pager
     * @param context
     * @return
     * @throws Exception
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看负载均衡器列表")
    @GetMapping
    public GeneralResult<GridBean<LoadBalancer>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context)
            throws Exception {
        return loadBalancerService.list(pager, context);
    }


    /**
     * 查看负载均衡器详情
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看负载均衡器详情")
    @GetMapping("/{id}")
    public GeneralResult<LoadBalancer> detail(@PathVariable(value = Common.ID) Long id) {
        return loadBalancerService.detail(id);
    }

    @Operation(summary = "负载均衡器标签")
    @PatchMapping(value = "/{id}")
    public GeneralResult labels(@PathVariable(value = Common.ID) Long id,
                                @RequestParam(value = Common.PARAMS, required = false) String params,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        JSONObject object = JSONTools.isJSONObj(params);
        if (null != object && object.containsKey("labels")) {
            String labels = object.getString("labels");
            return loadBalancerService.labels(id, labels, context);
        }
        return ResultTools.formatErrResult();

    }

    @Operation(summary = "查询可绑定负载")
    @GetMapping("/availableLbs")
    public GeneralResult listAvailableLbs(@RequestParam(value = Common.VENDORID, required = true) Long vendorId,
                                          @RequestParam(value = Common.REGIONID, required = true) String regionId,
                                          @Value(Common.REQ_CONTEXT) RequestContext context) {
        return loadBalancerService.listAvailableLbs(vendorId, regionId, context);
    }
}
