package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.ResourceProperty;
import com.bocloud.cmp.service.ResourcePropertyService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 资源属性相关接口
 *
 * @author dmw
 */
@Tag(name = "资源属性")
@RestController
@RequestMapping("/v1/resourceoproperty")
public class ResourcePropertyController {

    @Autowired
    private ResourcePropertyService resourcePropertyService;

    /**
     * 获取资源属性列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询资源属性列表")
    @GetMapping
    public GeneralResult<GridBean<ResourceProperty>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourcePropertyService.list(pager, context);
    }

    /**
     * 添加资源属性
     *
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "添加资源属性")
    @PostMapping
    public GeneralResult<ResourceProperty> create(@RequestBody ResourceProperty resourceProperty, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourcePropertyService.create(resourceProperty, context.getTarget());
    }

    /**
     * 修改资源属性
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改资源属性")
    @PutMapping("/{id}")
    public GeneralResult<ResourceProperty> modify(@PathVariable(value = Common.ID) Long id, @RequestBody ResourceProperty resourceProperty,
                                                  @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourcePropertyService.modify(resourceProperty, context.getTarget());
    }

    /**
     * 移除资源属性
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "移除资源属性")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourcePropertyService.remove(id, context.getTarget());
    }

    /**
     * 查看资源属性详细信息
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看资源属性详情")
    @GetMapping("/{id}")
    public GeneralResult<ResourceProperty> detail(@PathVariable(value = Common.ID) Long id) {
        return resourcePropertyService.detail(id);
    }


}
