package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.Room;
import com.bocloud.cmp.service.RoomService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 机房相关接口
 *
 * @author dmw
 */
@RestController
@RequestMapping("/v1/rooms")
@Tag(name = "机房")
public class RoomController {
    @Autowired
    private RoomService roomService;

    /**
     * 获取机房列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询机房列表")
    @GetMapping
    public GeneralResult<GridBean<Room>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return roomService.list(pager, context);
    }

    /**
     * 获取机房列表
     *
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "通过condition方式查询机房列表")
    @GetMapping("/condition")
    public GeneralResult<List<Room>> condition(@RequestParam(value = Common.CONDITION, required = false) String condition,
                                               @Value(Common.REQ_CONTEXT) RequestContext context) {
        return roomService.condition(condition, context);
    }

    /**
     * 添加机房
     *
     * @param room
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "添加机房")
    @PostMapping
    public GeneralResult<Room> create(@RequestBody Room room, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != room) {
            return roomService.create(room, context.getTarget());
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 修改机房
     *
     * @param room
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改机房")
    @PutMapping(value = "/{id}")
    public GeneralResult<String> modify(@RequestBody Room room, @PathVariable(value = Common.ID) Long id,
                                        @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != room) {
            room.setId(id);
            return roomService.modify(room, context.getTarget());
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 移除机房
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "移除机房")
    @DeleteMapping(value = "/{id}")
    public GeneralResult<String> remove(@PathVariable(value = Common.ID) Long id,
                                        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return roomService.remove(id, context.getTarget());
    }

    /**
     * 查看机房详细信息
     *
     * @param id 机房ID
     * @return 操作结果
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看机房详情")
    @GetMapping(value = "/{id}")
    public GeneralResult<Room> detail(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context,
                                      @RequestParam(value = Common.ACTION, required = false) String action) {
        return roomService.details(id, action);
    }

}
