package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.VendorSyncRecord;
import com.bocloud.cmp.service.VendorSyncRecordService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 云平台同步记录
 */

@RestController
@RequestMapping("/v1/vendors/records")
@Tag(name = "云平台同步管理")
public class VendorSyncRecordController {

    @Autowired
    private VendorSyncRecordService vendorSyncRecordService;


    /**
     * 列表
     *
     * @param pager
     * @return
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "获取云平台同步记录")
    public GeneralResult<GridBean<VendorSyncRecord>> list(Pager pager) {
        return vendorSyncRecordService.list(pager);
    }

    /**
     * 删除
     *
     * @param id
     * @return
     */
    @DeleteMapping("/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "删除云平台同步记录")
    public Result remove(@PathVariable(value = Common.ID) Long id) {
        return vendorSyncRecordService.remove(id);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "获取云平台同步记录")
    public GeneralResult<VendorSyncRecord> detail(@PathVariable(value = Common.ID) Long id) {
        return vendorSyncRecordService.detail(id);
    }

    /**
     * 根据云平台ID删除
     *
     * @param vendorId
     * @return
     */
    @DeleteMapping("/vendors/{vendorId}")
    @Operation(tags = {"CMC", "CSC"}, summary = "根据云平台ID删除平台同步记录")
    public Result removeByVendor(@PathVariable(value = Common.VENDORID) Long vendorId) {
        return vendorSyncRecordService.removeByVendor(vendorId);
    }

    /**
     * 根据时间段删除
     *
     * @param startTime
     * @param endTime
     * @param vendorId
     * @return
     */
    @DeleteMapping("/time")
    @Operation(tags = {"CMC", "CSC"}, summary = "根据时间段删除平台同步记录")
    public Result removeByTime(@RequestParam(value = "startTime", required = false) String startTime, @RequestParam(value = "endTime", required = false) String endTime, @RequestParam(value = "vendorId", required = false) Long vendorId) {
        return vendorSyncRecordService.removeByTime(startTime, endTime, vendorId);
    }

}
