package com.bocloud.cmp.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.Volume;
import com.bocloud.cmp.service.VolumeService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@Tag(name = "卷信息管理")
public class VolumeController {

    @Autowired
    private VolumeService volumeService;

    /**
     * 查询卷列表
     *
     * @param pager
     * @param action
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询卷信息列表")
    @GetMapping("/v1/volumes")
    public GeneralResult<GridBean<Volume>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        // 分页查询volume
        return volumeService.list(pager, context);
    }

    /**
     * 通过condition查询卷信息列表
     *
     * @param pager
     * @param action
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "通过condition查询卷信息列表")
    @GetMapping("/v1/volumes/condition")
    public GeneralResult condition(Pager pager, @RequestParam(value = "action", required = false) String action,
                                   @RequestParam(value = Common.CONDITION, required = false) String condition,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (!StringUtils.isEmpty(condition)) {
            JSONObject object = JSONObject.parseObject(condition);
            action = object.getString("condition");
        }
        if (StringUtils.isEmpty(action) || "common".equals(action)) {
            // 分页查询volume
            return volumeService.list(pager, context);
        } else if ("listByMdisk".equals(action)) {
//			// 按照Mdisk查询卷
//			Map<String, Object> paramMap = new HashMap<>(8);
//			if (pager.getParams() != null && pager.getParams().size() > 0) {
//				for (Param param : pager.getParams()) {
//					paramMap.putAll(param.getParam());
//				}
//			}
//			if (!paramMap.containsKey("mdiskId")) {
//				return new GeneralResult(false, "缺少参mdiskId");
//			}
//			long mdiskId = Long.parseLong(paramMap.get("mdiskId").toString());
//			return mdiskVolumeService.listVolumeByMdisk(pager.getPage(), pager.getRows(), mdiskId, paramMap,
//					pager.getSorter());
            return ResultTools.noImplementedResult();
        } else if ("listFlashCopyTargetVolume".equals(action)) {
//			// 按照原卷查询FlashCopy目标卷
//			Map<String, Object> paramMap = new HashMap<>(8);
//			if (pager.getParams() != null && pager.getParams().size() > 0) {
//				for (Param param : pager.getParams()) {
//					paramMap.putAll(param.getParam());
//				}
//			}
//			if (!paramMap.containsKey("volumeId")) {
//				return new GeneralResult(false, "缺少参volumeId");
//			}
//			long volumeId = Long.parseLong(paramMap.get("volumeId").toString());
//			return flashCopyService.listTargetVolumeFromDB(pager.getPage(), pager.getRows(), volumeId, paramMap,
//					pager.getSorter());
            return ResultTools.noImplementedResult();
        } else if ("listforpowervcvmcreate".equals(action)) {
            // Powervc 创建云主机的时候，查询可使用的卷
//			Map<String, Object> paramMap = new HashMap<>(8);
//			if (pager.getParams() != null && pager.getParams().size() > 0) {
//				for (Param param : pager.getParams()) {
//					paramMap.putAll(param.getParam());
//				}
//			}
//			if (!paramMap.containsKey(Common.VENDORID)) {
//				return new GeneralResult(false, "缺少参vendorId");
//			}
//			long vendorId = Long.parseLong(paramMap.get(Common.VENDORID).toString());
//			String keyword = paramMap.get("keyword") == null ? null : paramMap.get("keyword").toString();
//			return volumeService.listForPowervcVMCreate(pager.getPage(), pager.getRows(), keyword, vendorId);
            return ResultTools.noImplementedResult();
        } else if ("listforpowervcvmattach".equals(action)) {
            // Powervc 创建云主机的时候，查询可使用的卷
            Map<String, Object> paramMap = new HashMap<>(8);
            if (pager.getParams() != null && pager.getParams().size() > 0) {
                for (Param param : pager.getParams()) {
                    paramMap.putAll(param.getParam());
                }
            }
            if (!paramMap.containsKey(Common.VENDORID)) {
                return new GeneralResult(false, "缺少参vendorId");
            }
            if (!paramMap.containsKey("vmId")) {
                return new GeneralResult(false, "缺少参vmId");
            }
            long vendorId = Long.parseLong(paramMap.get(Common.VENDORID).toString());
            long vmId = Long.parseLong(paramMap.get("vmId").toString());
            String keyword = paramMap.get("keyword") == null ? null : paramMap.get("keyword").toString();
            return volumeService.listForPowervcVMAttach(pager.getPage(), pager.getRows(), keyword, vendorId, vmId);
        } else if ("listforpowervcimagecreate".equals(action)) {
            // Powervc 创建云主机的时候，查询可使用的卷
            Map<String, Object> paramMap = new HashMap<>(8);
            if (pager.getParams() != null && pager.getParams().size() > 0) {
                for (Param param : pager.getParams()) {
                    paramMap.putAll(param.getParam());
                }
            }
            if (!paramMap.containsKey(Common.VENDORID)) {
                return new GeneralResult(false, "缺少参vendorId");
            }
            long vendorId = Long.parseLong(paramMap.get(Common.VENDORID).toString());
            String keyword = paramMap.get("keyword") == null ? null : paramMap.get("keyword").toString();
            return volumeService.listForPowervcImageCreate(pager.getPage(), pager.getRows(), keyword, vendorId);
        } else if ("listByStatus".equalsIgnoreCase(action)) {
            return volumeService.listByStatus(condition, context);
        } else if ("listBackupResources".equalsIgnoreCase(action)) {
            return volumeService.listBackupResources(condition, context);
        } else if ("listbyvendor".equalsIgnoreCase(action)) {
            JSONObject object = JSONObject.parseObject(condition);
            return volumeService.listByVendor(object.getLong(Common.VENDORID));
        } else if ("listbyimage".equalsIgnoreCase(action)) {
            JSONObject object = JSONObject.parseObject(condition);
            return volumeService.listByImage(object.getLong("imageId"));
        } else if ("listInSyncedSet".equalsIgnoreCase(action)) {
            Map<String, Object> paramMap = new HashMap<>(8);
            if (pager.getParams() != null && pager.getParams().size() > 0) {
                for (Param param : pager.getParams()) {
                    paramMap.putAll(param.getParam());
                }
            }
            return volumeService.listInSyncedSet(pager.getPage(), pager.getRows(), paramMap, pager.getSorter());
        } else if ("listNotInSyncedSet".equalsIgnoreCase(action)) {
            Map<String, Object> paramMap = new HashMap<>(8);
            if (pager.getParams() != null && pager.getParams().size() > 0) {
                for (Param param : pager.getParams()) {
                    paramMap.putAll(param.getParam());
                }
            }
            return volumeService.listNotInSyncedSet(pager.getPage(), pager.getRows(), paramMap, pager.getSorter());
        } else if ("listByHostForXsky".equalsIgnoreCase(action)) {
            Map<String, Object> paramMap = new HashMap<>(8);
            if (pager.getParams() != null && pager.getParams().size() > 0) {
                for (Param param : pager.getParams()) {
                    paramMap.putAll(param.getParam());
                }
            }
            if (!paramMap.containsKey("hostId")) {
                return new GeneralResult(false, "缺少参hostId");
            }
            long hostId = Long.parseLong(paramMap.get("hostId").toString());
            return volumeService.listByHost(pager.getPage(), pager.getRows(), hostId, paramMap, pager.getSorter());
        } else if ("listByCondition".equalsIgnoreCase(action)) {
            return volumeService.listByCondition(JSONObject.parseObject(condition), context);
        } else if ("listByVmId".equalsIgnoreCase(action)) {
            return volumeService.listByVmId(JSONObject.parseObject(condition));
        } else if ("vmDisks".equalsIgnoreCase(action)) {
            return volumeService.listDataDisks(JSONObject.parseObject(condition));
        }
        return new GeneralResult<>(false, "查询类型不支持");
    }

    /**
     * 根据storage查询卷
     *
     * @param params
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "根据storage查询卷")
    @PostMapping("/volume/listbystorage")
    public GeneralResult<List<Volume>> listByStorage(@RequestParam(value = Common.PARAMS) String params) {
        JSONObject object = JSONTools.isJSONObj(params);
        if (null != object) {
            return volumeService.listByStorage(object.getLong("storageId"));
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 根据存储器模板查询卷
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "根据存储器模板查询卷")
    @GetMapping("/v1/storages/templates/{id}/volumes")
    public GeneralResult<List<Volume>> listByTemplate(@PathVariable(value = Common.ID) Long id) {
        return volumeService.listByTemplate(id);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看卷详情")
    @GetMapping("/v1/volumes/{id}")
    public GeneralResult<Volume> detail(@PathVariable(value = Common.ID) Long id) {
        return volumeService.detail(id);
    }


    /**
     * 查看主机列表
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看主机列表")
    @GetMapping("/v1/volumes/{id}/vms")
    public GeneralResult<List<CloudServer>> servers(@PathVariable(value = Common.ID) Long id,
                                                    @RequestParam(value = Common.CONDITION, required = false) String condition,
                                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (StringUtils.isEmpty(condition)) {
            return volumeService.servers(id);
        }
//		else if ("powervc".equals(condition) || "smartx".equals(condition)) {
//			return serverVolumeService.listServerByVolume(id);
//		}
        return new GeneralResult(false, "不支持的condition参数值");
    }

    /**
     * 批量删除卷
     *
     * @param params
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "批量删除卷")
    @DeleteMapping("/v1/volumes")
    public GeneralResult<String> batchRemove(@RequestBody List<Long> ids,
                                             @Value(Common.REQ_CONTEXT) RequestContext context) {
        return volumeService.batchRemove(ids, context);
    }

    /**
     * 上传镜像
     *
     * @param params
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "上传镜像")
    @PostMapping("/v1/volumes/{id}/images")
    public GeneralResult<String> uploadToImage(@PathVariable(value = Common.ID) Long id,
                                               @RequestParam(value = Common.PARAMS) String params, @Value(Common.REQ_CONTEXT) RequestContext context) {
        JSONObject object = JSONTools.isJSONObj(params);
        if (null != object) {
            return volumeService.uploadToImage(params, context);
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 汇总卷数量
     *
     * @param userId
     * @param projectId
     * @param tenantId
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "汇总卷数量")
    @PostMapping("/volume/count")
    public GeneralResult<Integer> count(@RequestParam(value = "userId") Long userId,
                                        @RequestParam(value = "projectId") Long projectId, @RequestParam(value = Common.TENANTID) Long tenantId) {
        return volumeService.count(userId, projectId, tenantId);
    }

    /**
     * 管理存储上现有受支持的卷
     *
     * @param params
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "管理存储上现有受支持的卷")
    @PostMapping("/v1/storages/{id}/volumes")
    public GeneralResult<String> onboard(@RequestParam(value = Common.PARAMS) String params,
                                         @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        JSONObject object = JSONTools.isJSONObj(params);
        if (null != object) {
            return volumeService.onboard(params, context);
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 取消管理存储上的卷
     *
     * @param params
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "取消管理存储上的卷")
    @DeleteMapping("/v1/storages/{id}/volumes")
    public GeneralResult<String> unmanage(@RequestParam(value = Common.PARAMS) String params,
                                          @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        JSONObject object = JSONTools.isJSONObj(params);
        if (null != object) {
            return volumeService.unmanage(context, object.getLong(Common.VENDORID), object.getLong("storageId"),
                    JSONObject.parseArray(object.getString("volumeIds"), Long.class));
        } else {
            return ResultTools.formatErrResult();
        }
    }

    @Operation(tags = {"CMC", "CSC"}, summary = "根据卷查询快照列表")
    @GetMapping("/v1/volumes/{id}/snapshots")
    public GeneralResult listSnapshots(Pager pager, @PathVariable(value = Common.ID) Long id) {
        return volumeService.listSnapshotById(id, pager);
    }
}
