package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.Vpc;
import com.bocloud.cmp.service.VpcService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/v1/vpcs")
@Tag(name = "VPC管理")
public class VpcController {

    @Autowired
    private VpcService vpcService;

    /**
     * 查询vpc列表
     *
     * @param pager
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "VPC列表查询")
    @GetMapping
    public GeneralResult<GridBean<Vpc>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return vpcService.list(pager, context);
    }

    /**
     * 通过condition方式查询vpc列表
     *
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "通过condition方式查询vpc列表")
    @GetMapping("/condition")
    public GeneralResult<List<Vpc>> condition(
            @RequestParam(value = Common.CONDITION, required = false) String condition,
            @Value(Common.REQ_CONTEXT) RequestContext context) {
        return vpcService.condition(condition, context);
    }


}
