package com.bocloud.cmp.entity;

import com.megatron.common.utils.DaySerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 数据库服务
 * 
 * @author weiwei
 *
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("rds")
public class Rds extends GenericEntity {

	@PK(PKStrategy.AUTO)
	private Long id;// ID
	@Column("uuid")
	private String uuid;// UID
	@Column("instance_id")
	private String instanceId;
	@Column("type")
	private String type;// 数据库类型(MYSQL,ORACLE)
	@Column("category")
	private String category;// 部署类型
	@Column("version")
	private String version;// 数据库版本
	@Column("sku_id")
	private Long skuId;// 商品规格
	@Column("network_id")
	private Long networkId;// 网络ID
	@Column("region")
	private String region;// 所属域
	@Column("tenant_id")
	private Long tenantId;// 所属租户
	@Column("inventory")
	private String inventory;
	@Column("category_id")
	private Long categoryId; // 类型ID
	@Column("spec")
	private String spec; // 配置
	@Column("address")
	private String address;
	@Column("username")
	private String username;
	@Column("password")
	private String password;
	@Column("manage_ip")
	private String manageIp;
	@Column("gmt_catalog")
	private String gmtCatalog;
	@Column("task_id")
	private Long taskId;
	@Column("vendor_id")
	private Long vendorId;
	@Column("business_id")
	private Long businessId;
	@Column("project_id")
	private Long projectId;
	@Column("pool_group_id")
	private Long poolGroupId;
	@Column("expired_time")
	@JsonSerialize(using = DaySerializer.class)
	@DateTimeFormat(pattern = "yyyy-MM-dd")
	private Date expiredTime;// 过期时间
	@Column("resource_label")
	private String resourceLabel;
	@IgnoreAll
	private List<RdsNode> nodes;
	@IgnoreAll
	private List<RdsDb> dbs;
	@IgnoreAll
	private List<RdsUser> users;
	@IgnoreAll
	private String categoryName;
	@IgnoreAll
	private Long diskSkuId;
	@IgnoreAll
	private Integer dataDisk;
	@IgnoreAll
	private String diskCategory;
	@IgnoreAll
	private List<Map<String, String>> connections;
	@IgnoreAll
	private String tenantName;// 租户名称
	@IgnoreAll
	private String businessName;// 业务
	@IgnoreAll
	private String creatorName;// 创建者名称
	@IgnoreAll
	private String ownerName;// 所属人名称
	@IgnoreAll
	private String menderName;// 维护人名称
	@IgnoreAll
	private String projectName;// 项目名称

	/**
	 * 
	 * @param uuid
	 * @param type
	 * @param version
	 * @param skuId
	 * @param networkId
	 * @param region
	 * @param tenantId
	 */
	public Rds(String uuid, String type, String version, Long skuId, Long networkId, String region, Long tenantId) {
		super();
		this.uuid = uuid;
		this.type = type;
		this.version = version;
		this.skuId = skuId;
		this.networkId = networkId;
		this.region = region;
		this.tenantId = tenantId;
	}
}
