package com.bocloud.cmp.entity;

import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

import java.util.Arrays;
import java.util.List;

/**
 * 资源实体类
 *
 * @author dongkai
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("resource")
public class Resource extends GenericEntity {

	@PK(value = PKStrategy.AUTO)
	private Long id; // ID
	@Column("category")
	private String category; // 类型（大类）
	@Column("catalog")
	private String catalog; // 类型（小类）
	@Column("target")
	private Long target; // 目标机器ID
	@Column("private_ips")
	private String privateIps; // 私有IP
	@Column("vendor_id")
	private Long vendorId;// 云供应商Id
	@Column("username")
	private String username;// 用户名
	@Column("password")
	private String password;// 密码
	@Column("inventory")
	private String inventory;//
	@Column("is_accurated")
	private Boolean accurated;
	@Column("is_authed")
	private Boolean authed;
	@Column("resource_group")
	private String resourceGroup;
	@Column("os_category")
	private String osCategory; // 操作系统类型
	@Column("tenant_id")
	private Long tenantId;// 租户ID
	@Column("group_id")
	private Long groupId; // 资源池分组
	@Column("group_ids")
	private String groupIds;
	@Column("type_id")
	private Long typeId; // 资源类型
	@Column("project_id")
	private Long projectId;
	@Column("uuid")
	private String uuid;
	@IgnoreAll
	private String resourceGroupName;
	@IgnoreAll
	private String vendorName;
	@IgnoreAll
	private String rootVolumeUuid;
	@IgnoreAll
	private String tenantName;
	@IgnoreAll
	private String ownerName; // 根据ownerId查询
	@IgnoreAll
	private String groupName;
	@IgnoreAll
	private String groupsName;
	@Column("region_id")
	private String regionId;
	@Column("charset")
	private String charset; // 字符编码
	@IgnoreAll
	private List<ResourceProtocol> protocols;
	@IgnoreAll
	private List<ResourceAccount> accounts;
	@IgnoreAll
	@Column("access_protocols")
	private String accessProtocols;
	@IgnoreAll
	@Column("access_accounts")
	private String accessAccounts;
	@IgnoreAll
	@Column("rule_ids")
	private String ruleIds;
	@IgnoreAll
	private String oldName;
	@IgnoreAll
	private ResourceType resourceType;
	@IgnoreAll
	private String type;

	public Resource(String category, String catalog, Long target, String privateIps, String osCategory, Long vendorId) {
		this.category = category;
		this.catalog = catalog;
		this.target = target;
		this.privateIps = privateIps;
		this.osCategory = osCategory;
		this.vendorId = vendorId;
	}

	public void preSave(String name, String status, Long userId, Long tenantId) {
		this.setName(name);
		this.setStatus(status);
		this.setCreatorId(userId);
		this.setMenderId(userId);
		this.setOwnerId(userId);
		this.setTenantId(tenantId);
	}

	public void preSave(String name, String status, Long userId, Long tenantId, Long projectId) {
		this.setName(name);
		this.setStatus(status);
		this.setCreatorId(userId);
		this.setMenderId(userId);
		this.setOwnerId(userId);
		this.setTenantId(tenantId);
		this.setProjectId(projectId);
	}

	/**
	 * @return 0 主机，1 网络资产，2 数据库，3 应用
	 */
	public Integer getJumpserverCatagory() {
		if (this.category == null) {
			return null;
		}
		final String[] hostCategories = new String[]{"Unix","Linux","Windows","Main Frame"};
		final String[] networkCategories = new String[]{"Network"};
		final String[] dbCategories = new String[]{"Database"};
		final String[] appCategories = new String[]{"Application"};
		if (Arrays.asList(hostCategories).contains(this.category)) {
			return 0;
		}
		if (Arrays.asList(networkCategories).contains(this.category)) {
			return 1;
		}
		if (Arrays.asList(dbCategories).contains(this.category)) {
			return 2;
		}
		if (Arrays.asList(appCategories).contains(this.category)) {
			return 3;
		}
		return null;
	}

}
