package com.bocloud.cmp.entity;

import com.alibaba.fastjson.JSONObject;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * 任务实例类
 *
 * @author weiwei
 * @version 1.0
 * @since 2016.08.16
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Table("task_instance")
public class TaskInstance extends GenericEntity {

    @PK(value = PKStrategy.AUTO)
    private Long id; // ID
    @Column("uuid")
    private String uuid; // uuid
    @Column("code")
    private String code; // 任务类型
    @Column("graph")
    private String graph;
    @Column("cron")
    private String cron; // cron表达式
    @Column("gmt_start")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtStart; // 开始时间
    @Column("gmt_finish")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtFinish; // 结束时间
    @Column("task_id")
    private Long taskId; // 任务ID
    @Column("execute_mode")
    private String executeMode; // 启动方式
    @Column("execute_host")
    private String executeHost;// 执行主机
    @Column("tenant_id")
    private Long tenantId; // 租户ID
    @Column("rollbacks")
    private Boolean rollbacks;// 超时回滚
    @Column("timeout")
    private Long timeout;// 超时时间
    @Column("is_backup")
    private Boolean isBackup; // 是否纯备份任务
    @Column("cos_id")
    private Long cosId;
    @Column("ros_id")
    private Long rosId;
    @IgnoreAll
    private Double cost; // 总耗时
    @IgnoreAll
    private String operator; // 启动人
    @IgnoreAll
    private List<JSONObject> results;
    @IgnoreAll
    private List<TaskNode> nodes;// 资源堆栈下的所有堆栈节点
    @IgnoreAll
    private String templateName;// 模板名称
    @IgnoreAll
    private String vendorName;// 平台名称
    @IgnoreAll
    private RequestContext context;
    @IgnoreAll
    private String tenantName;// 租户名称

    public TaskInstance(Long creatorId, Long ownerId, Long menderId, String name, String status, String remark,
                        String code, String graph, String cron) {
        super();
        this.setCreatorId(creatorId);
        this.setOwnerId(ownerId);
        this.setMenderId(menderId);
        this.setName(name);
        this.setStatus(status);
        this.setRemark(remark);
        this.code = code;
        this.graph = graph;
        this.cron = cron;
    }

}
