package com.bocloud.cmp.entity;

import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 资源池实体类
 * 
 * @author wangyu
 * @version 1.0
 * @since 2017年11月24日
 *
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Table("resource_pool")
public class VmResourcePool extends GenericEntity {

	@PK(value = PKStrategy.AUTO)
	private Long id;// ID
	@Column("parent_id")
	private Long parentId;// 父资源池
	@Column("cpu_reservation")
	private Long cpuReservation;// CPU预留
	@Column("cpu_expan_reservation")
	private Boolean cpuExpanReservation;// CPU可扩展预留
	@Column("cpu_limit")
	private Long cpuLimit;// CPU限制
	@Column("cpu_shares")
	private int cpuShares;// CPU份额值
	@Column("cpu_share_level")
	private String cpuShareLevel;// CPU份额
	@Column("mem_reservation")
	private Long memReservation;// 内存预留
	@Column("mem_expan_reservation")
	private Boolean memExpanReservation;// 内存可扩展预留
	@Column("mem_limit")
	private Long memLimit;// 内存限制
	@Column("mem_shares")
	private int memShares;// 内存份额值
	@Column("mem_share_level")
	private String memShareLevel;// 内存份额
	@Column("pool_val")
	private String poolVal;// 资源池value
	@Column("vdc_id")
	private Long vdcId;// 数据中心ID
	@Column("owner")
	private Long owner;// 资源池所属
	@Column("owner_category")
	private String ownerCategory;// 资源池所属类别
	@Column("vendor_id")
	private Long vendorId;// 云平台ID
	@Column("tenant_id")
	private Long tenantId;// 租户ID
	@IgnoreAll
	private String children;// 子资源池
	@IgnoreAll
	private String hostName;// 宿主机名称
	@IgnoreAll
	private String hostVal;// 宿主机value
	@IgnoreAll
	private String clusterName;// 集群名称
	@IgnoreAll
	private String clusterVal;// 集群value
	@IgnoreAll
	private String vdcName;// 数据中心名称
	@IgnoreAll
	private String parentName;// 父资源池名称
	@IgnoreAll
	private String parentPoolVal;// 父资源池value
	@IgnoreAll
	private String ownerName;// 所属名称

	// 版本
	@IgnoreAll
	private String version;

	// 服务器数量（主机数量）
	@IgnoreAll
	private Integer hostCount;

	// 云主机数量（实例数量）
	@IgnoreAll
	private Integer instanceCount;

	// 模板数量
	@IgnoreAll
	private Integer templateCount;

	/**
	 * 
	 */
	public VmResourcePool() {
		super();
	}

	/**
	 * @param parentId
	 * @param cpuReservation
	 * @param cpuExpanReservation
	 * @param cpuLimit
	 * @param cpuShares
	 * @param cpuShareLevel
	 * @param memReservation
	 * @param memExpanReservation
	 * @param memLimit
	 * @param memShares
	 * @param memShareLevel
	 * @param poolVal
	 * @param vdcId
	 * @param owner
	 * @param ownerCategory
	 * @param vendorId
	 * @param tenantId
	 * @param children
	 * @param hostName
	 * @param hostVal
	 * @param clusterName
	 * @param clusterVal
	 * @param vdcName
	 * @param parentName
	 * @param parentPoolVal
	 */
	public VmResourcePool(Long parentId, Long cpuReservation, Boolean cpuExpanReservation, Long cpuLimit, int cpuShares,
                          String cpuShareLevel, Long memReservation, Boolean memExpanReservation, Long memLimit, int memShares,
                          String memShareLevel, String poolVal, Long vdcId, Long owner, String ownerCategory, Long vendorId,
                          Long tenantId, String children, String hostName, String hostVal, String clusterName, String clusterVal,
                          String vdcName, String parentName, String parentPoolVal) {
		super();
		this.parentId = parentId;
		this.cpuReservation = cpuReservation;
		this.cpuExpanReservation = cpuExpanReservation;
		this.cpuLimit = cpuLimit;
		this.cpuShares = cpuShares;
		this.cpuShareLevel = cpuShareLevel;
		this.memReservation = memReservation;
		this.memExpanReservation = memExpanReservation;
		this.memLimit = memLimit;
		this.memShares = memShares;
		this.memShareLevel = memShareLevel;
		this.poolVal = poolVal;
		this.vdcId = vdcId;
		this.owner = owner;
		this.ownerCategory = ownerCategory;
		this.vendorId = vendorId;
		this.tenantId = tenantId;
		this.children = children;
		this.hostName = hostName;
		this.hostVal = hostVal;
		this.clusterName = clusterName;
		this.clusterVal = clusterVal;
		this.vdcName = vdcName;
		this.parentName = parentName;
		this.parentPoolVal = parentPoolVal;
	}

	public void preSave(Long creatorId, Long menderId, Long vendorId, Long tenantId) {
		this.setCreatorId(creatorId);
		this.setMenderId(menderId);
		this.setVendorId(vendorId);
		this.setTenantId(tenantId);
	}
}
