package com.bocloud.cmp.entity;

import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.List;

/**
 * 存储池
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Table("volume_pool")
public class VolumePool extends GenericEntity {

    /**
     * 自增id
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;
    /**
     * 所属平台id
     */
    @Column(value = "vendor_id")
    private Long vendorId;
    /**
     * 所属存储服务器
     */
    @Column(value = "storage_id")
    private Long storageId;
    /**
     * 所属存储服务器
     */
    @Column(value = "storage_name")
    @IgnoreAll
    private String storageName;
    /**
     * 存储池uuid
     */
    @Column(value = "uuid")
    private String uuid;
    /**
     * 池标识
     */
    @Column("identifier")
    private String identifier;
    /**
     * 警告阈值
     */
    @Column(value = "warning")
    private Double warning;
    /**
     * 虚拟使用容量（virtualCapacity）和容量（capacity）的比率
     */
    @Column(value = "overallocation")
    private Double overallocation;
    /**
     * 最小分配单元（MB）
     */
    @Column(value = "extent_size")
    private Double extentSize;
    /**
     * 容量
     */
    @Column(value = "capacity")
    private Double capacity;
    /**
     * 真实容量
     */
    @Column(value = "real_capacity")
    private Double realCapacity;
    /**
     * 使用容量
     */
    @Column(value = "used_capacity")
    private Double usedCapacity;
    /**
     * 虚拟使用容量
     */
    @Column(value = "virtual_capacity")
    private Double virtualCapacity;
    /**
     * 可用容量
     */
    @Column(value = "free_capacity")
    private Double freeCapacity;
    /**
     * 是否有压缩卷存在
     */
    @Column(value = "compression_active")
    private Boolean compressionActive;
    /**
     * 所有压缩卷虚拟容量
     */
    @Column(value = "compression_virtual_capacity")
    private Double compressionVirtualCapacity;
    /**
     * 所有压缩卷已使用容量
     */
    @Column(value = "compression_compressed_capacity")
    private Double compressionCompressedCapacity;
    /**
     * 所有压缩卷中未压缩容量
     */
    @Column(value = "compression_uncompressed_capacity")
    private Double compressionUncompressedCapacity;
    /**
     * 自动分层存储设定状态
     */
    @Column(value = "easy_tier")
    private String easytier;
    /**
     * 自动分层存储运行状态
     */
    @Column(value = "easy_tier_status")
    private String easytierStatus;
    /**
     * mdisk数量
     */
    @Column(value = "mdisk_count")
    private Integer mdiskCount;
    /**
     * 卷数量
     */
    @Column(value = "vdisk_count")
    private Integer vdiskCount;

    /**
     * mdiskId
     */
    @Column(value = "mdisk_Id")
    private Long mdiskId;

    /**
     * raidName
     */
    @Column(value = "raid")
    private String raid;

    /**
     * 池uuid
     */
    @Column(value = "pool_uuid")
    private String poolUuid;
    /**
     * smis传参用
     */
    @Column("key_path")
    private String keyPath;

    /**
     * operationalStatus
     */
    @Column(value = "operational_status")
    private String operationalStatus; // 运行状态

    /**
     * mdiskName
     */
    @Column(value = "mdisk_name")
    @IgnoreAll
    private String mdiskName;

    /**
     * raidName
     */
    @Column(value = "raid_name")
    @IgnoreAll
    private String raidName;


    /**
     * raidTactics
     */
    @Column(value = "raid_tactics")
    @IgnoreAll
    private String raidTactics;

    @Column(value = "role")
    private String role;

    @Column(value = "device_type")
    private String deviceType;

    @Column(value = "replicate_cnt")
    private Integer replicateCnt;

    @Column(value = "min_replicate_cnt")
    private Integer minReplicateCnt; // 最小副本数量

    @Column(value = "placement_group_cnt")
    private Integer placementGroupCnt; // 放置组数量

    @Column(value = "data_status")
    private String dataStatus;

    @IgnoreAll
    private Double poolMaxCapacity; // 池可用的最大容量
    @IgnoreAll
    private Double poolMinCapacity; // 池可用的最小容量
    @IgnoreAll
    private Long ecDataBolckCnt; // ec纠删码 数据块个数
    @IgnoreAll
    private Long ecCodeBolckCnt; // ec纠删码 校验块个数
    @IgnoreAll
    private List<Long> osdIds; // 存储池包含的osd列表
    @IgnoreAll
    private List<CephOsd> osds; // 存储池包含的osd
    @IgnoreAll
    private List<com.bocloud.cmp.entity.Volume> volumes; // 存储池包含的卷


}
