package com.bocloud.cmp.internal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cmp.domain.ElementModel;
import com.bocloud.cmp.domain.OrderItem;
import com.bocloud.cmp.domain.ServiceOrder;
import com.bocloud.cmp.entity.CloudService;
import com.bocloud.cmp.entity.Recycled;
import com.bocloud.cmp.entity.ResourceQuota;
import com.bocloud.cmp.entity.ServiceCatalog;
import com.bocloud.cmp.entity.ServiceCategory;
import com.bocloud.cmp.entity.ServiceSku;
import com.bocloud.cmp.entity.ServiceTenant;
import com.bocloud.cmp.entity.SkuPrice;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;

import lombok.extern.slf4j.Slf4j;

/**
 * cos服务内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月11日
 */
@Component
@Slf4j
public class CosInternalService implements InitializingBean {

	@Autowired
	private ServiceFactory serviceFactory;
	@Autowired
	private CurrentService currentService;
	private static Service service = Service.create("cos");
	private Map<String, Object> headers = new HashMap<>(4);
	
	public GeneralResult resourceReclaim(String sn) {
		Map<String, Object> params = Maps.newHashMap();
		JSONObject object = new JSONObject();
		params.put(Common.PARAMS, object.toJSONString());
		RemoteService remote = serviceFactory.build(service, "/v1/resource/reclaim/" + sn, HttpMethod.POST, headers,
				params);
		return remote.invoke();
	}

	public Integer list(Long userId, String apiKey) {
		List<Param> params = Lists.newArrayList();
		params.add(new Param(MapTools.simpleMap("status", "UNREAD"), Sign.EQ));
		params.add(new Param(MapTools.simpleMap("ownerId", userId), Sign.EQ));
		Map<String, Object> paramMap = MapTools.simpleMap("simple", true);
		paramMap.put("page", 1);
		paramMap.put("rows", Integer.MAX_VALUE);
		paramMap.put(Common.PARAMS, JSONArray.toJSONString(params));
		this.headers.put(Common.API_KEY, apiKey);
		RemoteService remote = serviceFactory.build(service, "/v1/messages", HttpMethod.GET, headers, paramMap);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return 0;
		}
		JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), JSONObject.class);
		return jsonObject.getInteger("total");
	}

	public GeneralResult updateQuotas(List<ResourceQuota> quotas, ResourceQuota.Action action, Long tenantId) {
		if(null == tenantId || 0 == tenantId) {
			return new GeneralResult<>(true, "租户为空，不调用配额接口");
		}
		Map<String, Object> paramMap = new HashMap<>();
		paramMap.put("quotas", quotas);
		paramMap.put(Common.ACTION, action.name());
		paramMap.put("tenantId", tenantId);
		RemoteService remote = serviceFactory.build(service, "/v1/quotas", HttpMethod.PATCH, headers, paramMap);
		remote.setDataFormat(PostDataFormat.RAW);
		return remote.invoke();
	}

	public List<ServiceTenant> listTenantServices(Long tenantId) {
		RemoteService remote = serviceFactory.build(service, "/v1/services/tenants/" + tenantId + "/services/items",
				HttpMethod.GET, headers, null);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return null;
		}
		String content = JSONObject.toJSONString(result.getData());
		if (StringUtils.isEmpty(content)) {
			return null;
		}
		return JSONArray.parseArray(content, ServiceTenant.class);
	}

	public SkuPrice queryPrice(Long skuId) {
		List<Param> params = Lists.newArrayList();
		Map<String, Object> simpleMap = MapTools.simpleMap("skuId", skuId);
		simpleMap.put("period", "Hour");
		params.add(new Param(simpleMap, Sign.EQ));
		Map<String, Object> paramMap = MapTools.simpleMap("simple", false);
		paramMap.put("page", 1);
		paramMap.put("rows", 1000);
		paramMap.put(Common.PARAMS, JSONArray.toJSONString(params));
		RemoteService remote = serviceFactory.build(service, "/v1/skus/prices", HttpMethod.GET, headers, paramMap);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return null;
		}
		JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), JSONObject.class);
		List<SkuPrice> prices = JSONArray.parseArray(jsonObject.getString("rows"), SkuPrice.class);
		return prices.get(0);
	}

	public ServiceCatalog getServiceCatalog(Long catalogId) {
		RemoteService remote = serviceFactory.build(service, "/v1/services/catalogs/" + catalogId, HttpMethod.GET,
				headers, null);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return null;
		}
		return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), ServiceCatalog.class);
	}

	public ServiceSku detailSku(Long skuId) {
		RemoteService remote = serviceFactory.build(service, "/v1/services/skus/" + skuId, HttpMethod.GET, headers,
				null);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return null;
		}
		return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), ServiceSku.class);
	}

	public GeneralResult<ServiceSku> querySku(String serviceCode, Integer cpu, Integer memory, Integer disk) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "getByCondition");
		object.put("cpu", cpu);
		object.put("memory", memory);
		object.put("disk", disk);
		object.put("serviceCode", serviceCode);
		RemoteService remote = serviceFactory.build(service, "/v1/services/skus", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String jsonString = JSONObject.toJSONString(result.getData());
		JSONObject parseObject = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
		if (null != parseObject) {
			List<ServiceSku> parseArray = JSONObject.parseArray(parseObject.getString("rows"), ServiceSku.class);
			if (!ListTool.isEmpty(parseArray)) {
				return new GeneralResult<>(true, parseArray.get(0), "查询成功");
			}
		}
		return new GeneralResult<>(false, "找不到合适的云主机产品规格");
	}

	public GeneralResult<ServiceCategory> getCategory(Long categoryId) {
		RemoteService remote = serviceFactory.build(service, "/v1/services/categories/" + categoryId, HttpMethod.GET,
				headers, null);
		GeneralResult result = remote.invoke();
		if (result.isFailed() || null == result.getData()) {
			return new GeneralResult<>(false, "服务类型查询失败");
		}
		return new GeneralResult<>(true,
				JSONObject.parseObject(JSONObject.toJSONString(result.getData()), ServiceCategory.class), "查询成功");
	}

	public GeneralResult recycle(String sn) {
		JSONObject object = new JSONObject();
		object.put("sn", sn);
		RemoteService remote = serviceFactory.build(service, "/v1/recycles/sn", HttpMethod.DELETE, headers, object);
		remote.setDataFormat(PostDataFormat.RAW);
		GeneralResult<?> invoke = remote.invoke();
		return invoke;
	}

	public CloudService detailService(String serviceCode) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "queryByCode");
		object.put("serviceCode", serviceCode);
		RemoteService remote = serviceFactory.build(service, "/v1/cloud/services", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return null;
		}
		return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), CloudService.class);
	}

	public GeneralResult removeByVendorId(Long vendorId) {
		Map<String, Object> params = Maps.newHashMap();
		JSONObject object = new JSONObject();
		object.put("vendorId", vendorId);
		params.put(Common.PARAMS, object.toJSONString());
		RemoteService remote = serviceFactory.build(service, "/price/removebyvendorid", HttpMethod.POST, headers,
				params);
		return remote.invoke();
	}

	public GeneralResult deleteByVendor(Long vendorId) {
		try {
			RemoteService remote = serviceFactory.build(service, "/v1/vendors/records/vendors/" + vendorId,
					HttpMethod.DELETE, headers, null);
			return remote.invoke();
		} catch (Exception e) {
			log.error("调用cos异常：" + e.getMessage());
			return new GeneralResult(false, "调用cos异常：" + e.getMessage());
		}
	}

	public GeneralResult joinRecycle(Recycled recycled) {
		Map<String, Object> paramMap = new HashMap<>();
		paramMap.put(Common.PARAMS, JSONObject.toJSONString(recycled));
		RemoteService remote = serviceFactory.build(service, "/v1/recycles", HttpMethod.POST, headers, paramMap);
		return remote.invoke();
	}

	public GeneralResult createOrder(ServiceOrder order) {
		RemoteService remote = serviceFactory.build(service, "/v1/orders", HttpMethod.POST, headers, order);
		remote.setDataFormat(PostDataFormat.RAW);
		return remote.invoke();
//		if (result.isFailed()) {
//			return result;
//		}
//		return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), ServiceOrder.class);
	}

	public OrderItem createOrderItem(OrderItem item) {
		RemoteService remote = serviceFactory.build(service, "/v1/orders/items", HttpMethod.POST, headers, item);
		remote.setDataFormat(PostDataFormat.RAW);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			log.error("创建订单项异常：" + result.getMessage());
			return null;
		}
		return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), OrderItem.class);
	}

	public GeneralResult updateStatus(Long id, String status) {
		JSONObject object = new JSONObject();
		object.put("resourceStatus", status);
		RemoteService remote = serviceFactory.build(service, "/v1/orders/" + id, HttpMethod.PATCH, headers,
				MapTools.simpleMap(Common.PARAMS, object.toJSONString()));
		remote.setDataFormat(PostDataFormat.RAW);
		return remote.invoke();
	}

	public GeneralResult updateStatus(Long id, String status, Long tenantId, Long projectId) {
		JSONObject object = new JSONObject();
		object.put("resourceStatus", status);
		if (null != tenantId && 0 != tenantId) {
			object.put("tenantId", tenantId);
		}
		if (null != projectId && 0 != projectId) {
			object.put("projectId", projectId);
		}
		RemoteService remote = serviceFactory.build(service, "/v1/orders/" + id, HttpMethod.PATCH, headers,
				MapTools.simpleMap(Common.PARAMS, object.toJSONString()));
		return remote.invoke();
	}

	public GeneralResult updateElements(String sn, List<ElementModel> elements) {
		JSONObject object = new JSONObject();
		object.put("sn", sn);
		object.put("elements", JSONObject.toJSONString(elements));
		RemoteService remote = serviceFactory.build(service, "/v1/orders/items/elements", HttpMethod.PATCH, headers,
				MapTools.simpleMap(Common.PARAMS, object.toJSONString()));
		return remote.invoke();
	}

	public GeneralResult recycleBill(String orderItemSn) {
		RemoteService remote = serviceFactory.build(service, "/v1/bills", HttpMethod.PUT, headers,
				MapTools.simpleMap("sn", orderItemSn));
		remote.setDataFormat(PostDataFormat.RAW);
		return remote.invoke();
	}

	public GeneralResult updateItemStatus(String sn, String status) {
		JSONObject object = new JSONObject();
		object.put("sn", sn);
		object.put("status", status);
		RemoteService remote = serviceFactory.build(service, "/v1/orders/items", HttpMethod.PATCH, headers, object);
		remote.setDataFormat(PostDataFormat.RAW);
		return remote.invoke();
	}

	public GeneralResult queryByResourceUuid(String resourceUuid, String resourceCategory) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "queryByResourceUuid");
		object.put("resourceUuid", resourceUuid);
		object.put("resourceCategory", resourceCategory);
		RemoteService remote = serviceFactory.build(service, "/v1/orders", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		return remote.invoke();
	}

	public GeneralResult detail(Long id) {
		RemoteService remote = serviceFactory.build(service, "/v1/orders/" + id, HttpMethod.GET, headers, null);
		return remote.invoke();
	}

	public GeneralResult getByOrderSn(String orderSn) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "getByOrderSn");
		object.put("orderSn", orderSn);
		RemoteService remote = serviceFactory.build(service, "/v1/orders", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		return remote.invoke();
	}

	public ServiceOrder getServiceOrder(String itemSn) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "getServiceOrder");
		object.put("itemSn", itemSn);
		RemoteService remote = serviceFactory.build(service, "/v1/orders", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return null;
		}
		String content = JSONObject.toJSONString(result.getData());
		if (org.apache.commons.lang3.StringUtils.isEmpty(content)) {
			return null;
		}
		return JSONObject.parseObject(content, ServiceOrder.class);
	}

	public OrderItem queryByItemSn(String itemSn) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "queryByItemSn");
		object.put("sn", itemSn);
		RemoteService remote = serviceFactory.build(service, "/v1/orders", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return null;
		}
		String content = JSONObject.toJSONString(result.getData());
		if (org.apache.commons.lang3.StringUtils.isEmpty(content)) {
			return null;
		}
		return JSONObject.parseObject(content, OrderItem.class);
	}

	public GeneralResult unsubscribeSuccessOrderItem(String sn, Boolean isProcessBill) {
		JSONObject object = new JSONObject();
		object.put("sn", sn);
		object.put("isProcessBill", isProcessBill);
		Map<String, Object> params = MapTools.simpleMap(Common.PARAMS, object.toJSONString());

		RemoteService remote = serviceFactory.build(service, "/v1/orders/items/unsubscribe", HttpMethod.POST,
			headers, params);
		return remote.invoke();
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
		this.headers.put("source", currentService.getService().getName());
	}

}
