package com.bocloud.cmp.internal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cmp.entity.Dictionary;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;

/**
 * @ClassName: SmsDictionaryService.java
 * @Description: 访问SMS字段数据
 *
 * @author: ymh
 * @date: 2021年4月7日 下午6:16:24
 */
@Component
public class SmsDictionaryService implements InitializingBean {

	@Autowired
	private ServiceFactory serviceFactory;
	@Autowired
	private CurrentService currentService;
	private static Service service = Service.create("sms");
	private Map<String, Object> headers = new HashMap<>(4);

	public GeneralResult<List<Dictionary>> dictionaries(String pvalue) {
		JSONObject params = new JSONObject();
		params.put("value", pvalue);
		RemoteService remoteService = serviceFactory.build(service, "/v1/dictionaries/children/tree", HttpMethod.GET,
				headers, params);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		List<Dictionary> dictionarys = JSONObject.parseArray(data, Dictionary.class);
		return new GeneralResult<>(true, dictionarys, "查询成功");
	}

	public GeneralResult<List<Dictionary>> childrens(String pvalue) {
		JSONObject params = new JSONObject();
		params.put("value", pvalue);
		RemoteService remoteService = serviceFactory.build(service, "/v1/dictionaries/children", HttpMethod.GET,
				headers, params);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		List<Dictionary> dictionarys = JSONObject.parseArray(data, Dictionary.class);
		return new GeneralResult<>(true, dictionarys, "查询成功");
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
		this.headers.put("source", currentService.getService().getName());
	}

}
