package com.bocloud.cmp.internal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cmp.domain.UserGroupPermissionModel;
import com.bocloud.cmp.entity.Project;
import com.bocloud.cmp.entity.UserProject;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;

/**
 * SOC项目内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年3月5日
 */
@Component
public class SmsProjectService implements InitializingBean {

	@Autowired
	private ServiceFactory serviceFactory;
	@Autowired
	private CurrentService currentService;
	private static Service service = Service.create("sms");
	private Map<String, Object> headers = new HashMap<>(4);

	public GeneralResult<Project> detail(Long id) {
		RemoteService remote = serviceFactory.build(service, "/v1/projects/" + id, HttpMethod.GET, headers, null);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONObject.parseObject(data, Project.class), "success");
	}

	public GeneralResult<Project> basic(Long id) {
		RemoteService remote = serviceFactory.build(service, "/v1/projects/" + id, HttpMethod.GET, headers, null);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONObject.parseObject(data, Project.class), "success");
	}

	public List<Project> listProjectsByIds(List<Long> ids) {
		ArrayList<Param> params = null;
		if (null != ids && !ids.isEmpty()) {
			String idString = JSONArray.toJSONString(ids);
			params = Lists.newArrayList(
					new Param(MapTools.simpleMap("id", idString.substring(1, idString.length() - 1)), Sign.IN));
		}
		Map<String, Object> paramMap = MapTools.simpleMap("simple", false);
		paramMap.put("page", 1);
		paramMap.put("rows", Integer.MAX_VALUE);
		paramMap.put(Common.PARAMS, JSONArray.toJSONString(params));
		RemoteService remoteService = serviceFactory.build(service, "/v1/projects", HttpMethod.GET, headers,
				paramMap);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new ArrayList<>();
		}
		String data = JSONObject.toJSONString(result.getData());
		return JSONObject.parseObject(data).getJSONArray("rows").toJavaList(Project.class);
	}

	public List<Project> projectsByCondition(Long target, Catalog catalog) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "projectsByCondition");
		object.put("target", target);
		object.put("catalog", catalog.name());
		RemoteService remote = serviceFactory.build(service, "/v1/projects/condition", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return Lists.newArrayList();
		}
		String data = JSONObject.toJSONString(result.getData());
		if (StringUtils.isEmpty(data)) {
			return Lists.newArrayList();
		}
		return JSONArray.parseArray(data, Project.class);
	}

	/**
	 * 查询用户管理的项目
	 *
	 * @param userId
	 * @return
	 */
	public List<Project> listUserManagerProjects(Long userId) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "listUserManagerProjects");
		object.put("userId", userId);
		RemoteService remote = serviceFactory.build(service, "/v1/projects/condition", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return Lists.newArrayList();
		}
		String data = JSONObject.toJSONString(result.getData());
		if (StringUtils.isEmpty(data)) {
			return Lists.newArrayList();
		}
		return JSONArray.parseArray(data, Project.class);
	}

	/**
	 * 查询用户有可见权限的项目
	 *
	 * @param userId
	 * @return
	 */
	public List<Project> listUserVisibleProjects(Long userId) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "listUserVisibleProjects");
		object.put("userId", userId);
		RemoteService remote = serviceFactory.build(service, "/v1/projects/condition", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return Lists.newArrayList();
		}
		String data = JSONObject.toJSONString(result.getData());
		if (StringUtils.isEmpty(data)) {
			return Lists.newArrayList();
		}
		return JSONArray.parseArray(data, Project.class);
	}

	/**
	 * 查询租户下用户所在的全局项目（管理端项目）
	 *
	 * @param tenantId
	 * @return
	 */
	public List<UserProject> listTenantUserGlobalProjects(Long tenantId) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "listTenantUserGlobalProjects");
		object.put("tenantId", tenantId);
		RemoteService remote = serviceFactory.build(service, "/v1/projects/condition", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return Lists.newArrayList();
		}
		String data = JSONObject.toJSONString(result.getData());
		if (StringUtils.isEmpty(data)) {
			return Lists.newArrayList();
		}
		return JSONArray.parseArray(data, UserProject.class);
	}

	/**
	 * 查询租户下项目
	 *
	 * @param tenantId
	 * @return
	 */
	public List<Project> listTenantProjects(Long tenantId) {
		RemoteService remote = serviceFactory.build(service, "/v1/tenants/" + tenantId + "/projects", HttpMethod.GET,
				headers, null);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return Lists.newArrayList();
		}
		String data = JSONObject.toJSONString(result.getData());
		if (StringUtils.isEmpty(data)) {
			return Lists.newArrayList();
		}
		return JSONArray.parseArray(data, Project.class);
	}

	public GeneralResult<List<Project>> listByTenant(Long tenantId) {
		RemoteService remote = serviceFactory.build(service, "/v1/tenants/" + tenantId + "/projects",
				HttpMethod.POST, headers, null);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONArray.parseArray(data, Project.class), "success");
	}

	public List<SimpleBean> list(Long businessId) {
		List<Param> params = Lists.newArrayList();
		params.add(new Param(MapTools.simpleMap("businessId", businessId), Sign.EQ));
		Map<String, Object> paramMap = MapTools.simpleMap("simple", true);
		paramMap.put(Common.PARAMS, JSONArray.toJSONString(params));
		paramMap.put("page", 1);
		paramMap.put("rows", Integer.MAX_VALUE);
		paramMap.put(Common.PARAMS, JSONArray.toJSONString(params));
		RemoteService remote = serviceFactory.build(service, "/v1/projects", HttpMethod.POST, headers, paramMap);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return Lists.newArrayList();
		}
		JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), JSONObject.class);
		return JSONArray.parseArray(jsonObject.getString("rows"), SimpleBean.class);
	}

	public List<Project> listAll() {
		List<Param> params = Lists.newArrayList();
		Map<String, Object> paramMap = MapTools.simpleMap("simple", false);
		paramMap.put(Common.PARAMS, JSONArray.toJSONString(params));
		paramMap.put("page", 1);
		paramMap.put("rows", Integer.MAX_VALUE);
		paramMap.put(Common.PARAMS, JSONArray.toJSONString(params));
		RemoteService remote = serviceFactory.build(service, "/v1/projects", HttpMethod.GET, headers, paramMap);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return Lists.newArrayList();
		}
		JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), JSONObject.class);
		return JSONArray.parseArray(jsonObject.getString("rows"), Project.class);
	}

	public List<UserGroupPermissionModel> listUserGroupPermission(Long userId, String type){
		Map<String, Object> paramMap = MapTools.simpleMap("userId", userId);
		paramMap.put("type", type);
		RemoteService remote = serviceFactory.build(service, "/v1/user/group/permission", HttpMethod.GET, headers, paramMap);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return Lists.newArrayList();
		}
		String data = JSONObject.toJSONString(result.getData());
		if (StringUtils.isEmpty(data)) {
			return Lists.newArrayList();
		}
		return JSONArray.parseArray(data, UserGroupPermissionModel.class);
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
		this.headers.put("source", currentService.getService().getName());
	}
}
