package com.bocloud.cmp.listener;

import com.alibaba.fastjson.JSONObject;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.JSONTools;
import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.stereotype.Component;

/**
 * 站内信通知监听
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年8月10日
 */
@Slf4j
@Component
public class BocloudMessageListener implements MessageListener {

    public final AmqpTemplate amqpTemplate;

    public BocloudMessageListener(AmqpTemplate amqpTemplate) {
        this.amqpTemplate = amqpTemplate;
    }

    @Override
    public void onMessage(Message message) {
        try {
            String content = new String(message.getBody());
            if (log.isDebugEnabled()) {
                log.debug("get bocloud message:{}", content);
            }
            Assert.notNull(content, "消息内容为空");
            GeneralResult<JSONObject> result = JSONTools.isJSON(content);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            OperateResult operate = JSONObject.parseObject(content, OperateResult.class);
            Assert.notNull(content, "消息格式不正确");
            RequestContext context = operate.getContext();
            System.out.println(JSONObject.toJSONString(message));
            amqpTemplate.send(message);
        } catch (Exception e) {
            log.error("Get error message:{}", e.getMessage());
        }
    }
}
