package com.bocloud.cmp.listener;

import java.util.List;

import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.ApplicationParam;
import com.bocloud.cmp.domain.OrderMessage;
import com.bocloud.cmp.enums.ResourceCategory;
import com.bocloud.cmp.service.ResourceOperationService;
import com.megatron.common.model.Result;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ListTool;

import lombok.extern.slf4j.Slf4j;

/**
 * 资源申请监听器
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月8日
 */
@Component
@Slf4j
public class ResourceApplyListener {

	@Autowired
	private ResourceOperationService resourceOperationService;
	private static final String CREATION_QUEUE = "iaas.resource.creation.queue";

	@RabbitListener(queues = CREATION_QUEUE, admin = "rabbitAdmin", autoStartup = "true")
	public void process(List<ApplicationParam> params) {
		try {
			if (ListTool.isEmpty(params)) {
				return;
			}
			for (ApplicationParam param : params) {
				JSONObject object = JSONTools.isJSONObj(JSONObject.toJSONString(param));
				log.info("接收到资源申请创建请求：" + object.toJSONString());
				// 申请类别
				JSONObject configs = object.getJSONObject("configs");
				String category = configs.getString("category");
				// 当前申请类型是本服务需要处理的
				Result processResult = resourceOperationService.process(ResourceCategory.valueOf(category), object,
						OrderMessage.Operation.Application);
				log.info("资源申请处理结果：{}", processResult.toString());
			}
		} catch (Exception e) {
			log.error("Get error message:{}", e.getMessage());
		}
	}

}
