package com.bocloud.cmp.listener;

import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.CancelationParam;
import com.bocloud.cmp.domain.OrderMessage;
import com.bocloud.cmp.enums.ResourceCategory;
import com.bocloud.cmp.service.ResourceOperationService;

import lombok.extern.slf4j.Slf4j;

/**
 * 资源退订监听器
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月8日
 */
@Component
@Slf4j
public class ResourceUnsubscribeListener {

	@Autowired
	private ResourceOperationService resourceOperationService;
	private static final String DELETION_QUEUE = "iaas.resource.deletion.queue";

	@RabbitListener(queues = DELETION_QUEUE, admin = "rabbitAdmin", autoStartup = "true")
	public void process(CancelationParam param) throws Exception {
		try {
			if (null == param) {
				return;
			}
			log.info("接收到资源退订消息：{}", JSONObject.toJSONString(param));
			String category = param.getCategory();
			resourceOperationService.process(ResourceCategory.valueOf(category),
					JSONObject.parseObject(JSONObject.toJSONString(param)), OrderMessage.Operation.Cancelation);
		} catch (Exception e) {
			log.error("Get error message:{}", e.getMessage());
		}
	}

}
