package com.bocloud.cmp.listener;


import java.nio.charset.Charset;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.bocloud.cmp.service.RosOperationService;

import lombok.extern.slf4j.Slf4j;

/**
 * 资源编排回滚监听器
 *
 */
@Component("rosCallbackListener")
@Slf4j
public class RosCallbackListener {

	@Autowired
	private RosOperationService rosOperationService;

	@RabbitListener(queues = "ros.callback.queue", admin = "rabbitAdmin", autoStartup = "true")
	public void process(Message message) {
		try {

			String content = new String(message.getBody(), Charset.forName("UTF-8"));
			processDebugLog("get bocloud message:{}", content);
			if (StringUtils.isBlank(content)) {
				// 消息没有内容，直接确认收到 淹没处理
				return;
			}
			log.info("收到资源编排退订消息{}", content);
			rosOperationService.callbackProcess(content);
			return ;
		} catch (Exception e) {
			log.error("Get error message:{}", e.getMessage());
		}
	}
	
	private void processDebugLog(String format, Object... objects) {
		if (log.isDebugEnabled()) {
			log.debug(format, objects);
		}
	}

}
