package com.bocloud.cmp.repository;

import com.bocloud.cmp.entity.Dictionary;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 字典操作数据层接口实现类
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年5月15日
 */
@Component("dictionaryRepository")
public class DictionaryRepository extends BasicGenericDao<Dictionary, Long> {

    public DictionaryRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<Dictionary> dictionaries(Long pid) {
        if (pid != null && pid != 0L) {
            String sql = "select * from dictionary where pid = :pid order by weight,id";
            Map<String, Object> params = MapTools.simpleMap("pid", pid);
            return this.list(Dictionary.class, sql, params);
        } else {
            String sql = "select * from dictionary where pid = 0 order by weight,id";
            return this.list(Dictionary.class, sql, new HashMap<>(8));
        }
    }

    public List<Dictionary> list() {
        return this.list(Dictionary.class, "SELECT * FROM dictionary order by weight,id", new HashMap<>(8));
    }

    public Dictionary query(Long id) {
        String sql = "select * from dictionary where id = :id order by weight,id";
        Map<String, Object> param = MapTools.simpleMap(Common.ID, id);
        List<Dictionary> list = this.list(Dictionary.class, sql, param);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    public Dictionary query(String value, String pvalue) throws Exception {
        String sql = "select * from dictionary where value = :value and pvalue = :pvalue order by weight,id";
        Map<String, Object> param = MapTools.simpleMap("value", value);
        param.put("pvalue", pvalue);
        List<Dictionary> list = this.list(Dictionary.class, sql, param);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }
    
    public Dictionary queryFistDic(String value) {
        String sql = "select * from dictionary where value = :value and pid = 0 order by weight,id";
        Map<String, Object> param = MapTools.simpleMap("value", value);
        List<Dictionary> list = this.list(Dictionary.class, sql, param);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    public Dictionary queryDic(String value) {
        String sql = "select * from dictionary where value = :value order by weight,id";
        Map<String, Object> param = MapTools.simpleMap("value", value);
        List<Dictionary> list = this.list(Dictionary.class, sql, param);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    public boolean existValue(Long pid, String value) {
        String sql = "select * from dictionary where pid = :pid and value = :value order by weight,id";
        Map<String, Object> params = MapTools.simpleMap("value", value);
        params.put("pid", pid);
        List<Dictionary> list = this.list(Dictionary.class, sql, params);
        return !list.isEmpty();
    }

    public boolean batchRemove(List<Long> ids) {
        String sql = "delete from dictionary where id in (:ids)";
        Map<String, Object> paramsMap = MapTools.simpleMap("ids", ids);
        return this.execute(sql, paramsMap) > 0;
    }

	public String getNameByValue(String value) throws Exception {
        String sql = "select * from dictionary where value = :value order by weight,id";
        Map<String, Object> param = MapTools.simpleMap("value", value);
        List<Dictionary> list = this.list(Dictionary.class, sql, param);
        return ListTool.isEmpty(list) ? null : list.get(0).getName();
    }

}
