package com.bocloud.cmp.repository;

import com.bocloud.cmp.entity.KubePod;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author weiwei
 */
@Component("kubePodRepository")
public class KubePodRepository extends JdbcGenericDao<KubePod, Long> {

	public KubePodRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
		super(jdbcTemplate, npJdbcTemplate, service);
	}

	public List<KubePod> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		return this.list(page, rows, params, sorter, null);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.id,a.name from kube_pod a where a.is_deleted = 0";
		sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		List<KubePod> contracts = this.list(KubePod.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<SimpleBean>();
		for (KubePod contract : contracts) {
			beans.add(new SimpleBean(contract.getId(), contract.getName()));
		}
		return beans;
	}

	public int count(List<Param> params) throws Exception {
		return this.count(params, null);
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update kube_pod set is_deleted = true , gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean batchRemove(List<Long> ids, Long userId) throws Exception {
		String sql = "update kube_pod set is_deleted = true , gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and id in (:ids) ";
		Map<String, Object> params = MapTools.simpleMap("ids", ids);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public KubePod query(Long id) throws Exception {
		String sql = "select a.* from kube_pod a where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> param = MapTools.simpleMap(Common.ID, id);
		List<KubePod> list = this.list(KubePod.class, sql, param);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public List<KubePod> list(List<Long> ids) throws Exception {
		String sql = "select a.* from kube_pod a where a.is_deleted = 0 and a.id in (:ids)";
		Map<String, Object> params = MapTools.simpleMap("ids", ids);
		return this.list(KubePod.class, sql, params);
	}

	public boolean removeByVendor(Long vendorId, Long userId) throws Exception {
		String sql = "update kube_pod set is_deleted = true , gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public List<KubePod> listByVendor(Long vendorId) throws Exception {
		String sql = "select a.* from kube_pod a where a.is_deleted = 0 and a.vendor_id = :vendorId";
		Map<String, Object> param = MapTools.simpleMap(Common.VENDORID, vendorId);
		List<KubePod> list = this.list(KubePod.class, sql, param);
		return list;
	}

	public List<KubePod> listByParams(List<Param> params) throws Exception {
		String sql = "select a.* from kube_pod a where a.is_deleted = 0";
		sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, null), "a");
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		return this.list(KubePod.class, sql, paramMap);
	}

	public int count(List<Param> params, List<Long> projectIds) {
		String sql = "select count(1) from kube_pod a where a.is_deleted = 0";
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		if (!ListTool.isEmpty(projectIds)) {
			sql += " and a.project_id in (:projectIds) ";
			paramMap.put("projectIds", projectIds);
		}
		sql = getQueryBuilder().buildRaw(sql, params, null, "a");
		return this.countQuery(sql, paramMap).intValue();
	}

	public List<KubePod> list(int page, int rows, List<Param> params, Map<String, String> sorter, List<Long> projectIds) {
		String sql = "select a.* , t.name tenant_name , project.visibility project_visibility from kube_pod a left join tenant t on a.tenant_id = t.id "
				+ " left join project project on a.project_id = project.id " + " WHERE a.is_deleted = 0 ";
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		if (!ListTool.isEmpty(projectIds)) {
			sql += " and a.project_id in (:projectIds) ";
			paramMap.put("projectIds", projectIds);
		}
		sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		return this.list(KubePod.class, sql, paramMap);
	}

}
