package com.bocloud.cmp.repository;

import com.bocloud.cmp.entity.ServerConfig;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 主机配置数据访问
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年5月15日
 *
 */
@Repository("serverConfigRepository")
public class ServerConfigRepository extends BasicGenericDao<ServerConfig, Long> {

	public ServerConfigRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
		super(jdbcTemplate, npJdbcTemplate, service);
	}

	public ServerConfig query(Long id) {
		String sql = "select * from server_config where id = :id";
		Map<String, Object> param = MapTools.simpleMap("id", id);
		List<ServerConfig> list = this.list(ServerConfig.class, sql, param);
		return ListTool.isEmpty(list) ? null : list.get(0);
	}

	public ServerConfig getBySid(Long serverId) {
		String sql = "select * from server_config where server_id = :serverId";
		Map<String, Object> param = MapTools.simpleMap("serverId", serverId);
		List<ServerConfig> list = this.list(ServerConfig.class, sql, param);
		return ListTool.isEmpty(list) ? null : list.get(0);
	}

	public Boolean removeBySid(Long serverId) {
		String sql = "delete from server_config where server_id = :serverId";
		Map<String, Object> param = MapTools.simpleMap("serverId", serverId);
		return this.execute(sql, param) > 0;
	}

	public ServerConfig getConfig(String ipmiIp) {
		String sql = "select a.* from server_config a left join server b on b.id = a.server_id and b.is_deleted = 0 where a.ipmi_address like '%"
				+ ipmiIp + "%' or a.ipmi_ips like '%" + ipmiIp + "%'";
		List<ServerConfig> list = this.list(ServerConfig.class, sql, null);
		return ListTool.isEmpty(list) ? null : list.get(0);
	}

	public List<ServerConfig> listByHostIds(List<Long> hostIds) {
		String sql = "select * from server_config where server_id in (:hostIds)";
		Map<String, Object> param = MapTools.simpleMap("hostIds", hostIds);
		return this.list(ServerConfig.class, sql, param);
	}

	public  List<ServerConfig> listUsageForVdc(Long vendorId){
		String sql = "   select a.vdc_id " +
				" ,sum(cpu_used) as cpu_used,sum(mem_used) as mem_used,sum(disk_used) as disk_used" +
				" ,sum(cpu_hz) as cpu_hz,sum(memory_capacity) as memory_capacity,sum(disk_capacity) as disk_capacity " +
				" from server a left join server_config b on a.id = b.server_id " +
				" where a.is_deleted = 0 and a.vendor_id = :vendorId group by a.vdc_id  ";
		Map<String,Object> param = MapTools.simpleMap("vendorId",vendorId);
		return this.list(ServerConfig.class, sql, param);
	}
	public List<ServerConfig> getByServerIds(List<Long> serverIds) {
		String sql = "select * from server_config where server_id in (:serverIds)";
		Map<String, Object> param = MapTools.simpleMap("serverIds", serverIds);
		return this.list(ServerConfig.class, sql, param);
	}
}
