package com.bocloud.cmp.repository;

import com.bocloud.cmp.entity.VendorCategory;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 基础组件 平台类型repository
 * 
 * @author hp
 *
 */
@Repository("vendorCategoryRepository")
public class VendorCategoryRepository extends BasicGenericDao<VendorCategory, Long> {

	public VendorCategoryRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
		super(jdbcTemplate, namedParameterJdbcTemplate, service);
	}

	public List<VendorCategory> list(int page, int rows, List<Param> params, Map<String, String> sorter)
			throws Exception {
		String sql = "select * from vendor_category where is_deleted = 0";
		sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "");
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		return this.list(VendorCategory.class, sql, paramMap);
	}

	public VendorCategory query(Long id) throws Exception {
		String sql = "select * from vendor_category a where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		List<VendorCategory> list = this.list(VendorCategory.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from vendor_category a where a.is_deleted = 0";
		sql = getQueryBuilder().buildRaw(sql, params, null, "a");
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		return this.countQuery(sql, paramMap).intValue();
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update vendor_category set is_deleted = true , gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean isNameRepeat(String name) throws Exception {
		String sql = "select id,name from vendor_category where name = :name and is_deleted = 0";
		Map<String, Object> paramMap = MapTools.simpleMap("name", name);
		List<VendorCategory> list = this.list(VendorCategory.class, sql, paramMap);
		if (list.isEmpty()) {
			return false;
		}
		return true;
	}

}
