package com.bocloud.cmp.scheduler;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.bocloud.cmp.service.SchedulerTaskService;
import com.gitee.starblues.integration.user.PluginUser;
import com.megatron.framework.core.CurrentService;

/**
 * cmp定时任务 插件中实现SchedulerTaskService接口即可
 * 
 * @author hp
 *
 */
@Component
public class CmpTaskScheduler implements InitializingBean {

	@Autowired
	private CurrentService service;
	@Autowired
	private PluginUser pluginUser;

	/**
	 * vc定时同步任务
	 */
	@Scheduled(cron = "${vendor.taskmsg.scheduler:0 0/5 * * * ?}")
	protected void updateTaskMsg() {
		if (!service.isLeader()) {
			return;
		}
		SchedulerTaskService schedulerTeskService = (SchedulerTaskService) pluginUser.getBean("vmware",
				"schedulerTaskService");
		if (null == schedulerTeskService) {
			return;
		}
		schedulerTeskService.updateTaskMsg();
	}

	/**
	 * openstack定时同步模板事件
	 */
	@Scheduled(cron = "${autoscaling.event.schedule:0 0/5 * * * ?}")
	protected void syncStackEvent() {
		if (!service.isLeader()) {
			return;
		}
		SchedulerTaskService schedulerTeskService = (SchedulerTaskService) pluginUser.getBean("openstack",
				"schedulerTaskService");
		if (null == schedulerTeskService) {
			return;
		}
		schedulerTeskService.syncStackEvent();
	}

	/**
	 * 同步openstack 云堆栈虚机
	 */
	@Scheduled(cron = "${autoscaling.resource.schedule:0 0 0/1 * * ?}")
	protected void syncStackServer() {
		if (!service.isLeader()) {
			return;
		}
		SchedulerTaskService schedulerTeskService = (SchedulerTaskService) pluginUser.getBean("openstack",
				"schedulerTaskService");
		if (null == schedulerTeskService) {
			return;
		}
		schedulerTeskService.syncStackServer();
	}

	@Override
	public void afterPropertiesSet() throws Exception {
	}

}
