package com.bocloud.cmp.service;

import com.bocloud.cmp.domain.CloudVendorBean;
import com.bocloud.cmp.entity.CloudVendor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ResultTools;

import java.util.List;

/**
 * 云供应商接口
 *
 * @author DZG
 * @since V1.0 2016年9月1日
 */
public interface CloudVendorService {

	/**
	 * 创建云供应商
	 *
	 * @param vendor
	 * @param context
	 * @return
	 */
	default GeneralResult<CloudVendor> create(CloudVendor vendor, RequestContext context) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 更新云供应商
	 *
	 * @param bean
	 * @param userId
	 * @return
	 */
	default GeneralResult<CloudVendor> modify(CloudVendorBean bean, RequestContext context) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 删除云供应商
	 *
	 * @param id
	 * @param context
	 * @return
	 */
	default GeneralResult remove(Long id, RequestContext context) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 冻结云供应商
	 *
	 * @param id
	 * @param userId
	 * @return
	 */
	default GeneralResult lock(Long id, Long userId) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 解冻云供应商
	 *
	 * @param id
	 * @param userId
	 * @return
	 */
	default GeneralResult active(Long id, Long userId) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 查看云供应商详情
	 *
	 * @param id
	 * @return
	 */
	default GeneralResult<CloudVendor> detail(Long id) {
		return ResultTools.noImplementedResult();
	}

	/**
	 * 根据云供应商姓名查询云供应商
	 *
	 * @param name
	 * @return
	 */
	default GeneralResult checkName(String name) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 查询云供应商列表
	 *
	 * @param pager
	 * @param context
	 * @return
	 */
	default GeneralResult<GridBean<CloudVendor>> list(Pager pager, RequestContext context) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 根据供应商类型查询列表
	 *
	 * @param type
	 * @return
	 */
	default List<CloudVendor> list(String type) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 查询所有云供应商
	 *
	 * @return
	 */
	default List<CloudVendor> listAll() {
		throw new UnsupportedOperationException();
	}

	/**
	 * 查询租户下已授权的云平台
	 *
	 * @param types
	 * @param tenantId
	 * @return
	 */
	default GeneralResult listAuthProvider(List<String> types, Long tenantId) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 查看vcenter平台全部集群的内存/cpu/存储的总使用量
	 *
	 * @param id 平台唯一标识
	 * @author sxp
	 * @since Apr 23, 2018
	 */
	default GeneralResult<CloudVendor> queryCapacity(Long id) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 根据平台类型查询所有平台
	 *
	 * @param type
	 * @return
	 */
	default GeneralResult getByType(String type) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 资源池查询可用平台
	 *
	 * @param params
	 * @return
	 */
	default GeneralResult listByPool(String params) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 批量开启告警
	 *
	 * @param ids
	 * @param userId
	 * @return
	 */
	default GeneralResult batchOpenAlarm(List<Long> ids, Long userId) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 批量关闭告警
	 *
	 * @param ids
	 * @param userId
	 * @return
	 */
	default GeneralResult batchCloseAlarm(List<Long> ids, Long userId) {
		throw new UnsupportedOperationException();
	}

	default GeneralResult batchOperationAlarm(List<Long> ids, String action, Long userId) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 根据平台类型查询
	 *
	 * @param types
	 * @return
	 */
	default GeneralResult listByTypes(List<String> types) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 云平台操作
	 *
	 * @param id
	 * @param action
	 * @param context
	 * @return
	 */
	default GeneralResult action(Long id, String action, RequestContext context) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 更新云平台密码
	 *
	 * @param id
	 * @param params
	 * @param context
	 * @return
	 */
	default GeneralResult updatePassword(Long id, CloudVendor cloudVendor, RequestContext context) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 通过condition方式获取云平台列表
	 *
	 * @param condition
	 * @param context
	 * @return
	 */
	default GeneralResult condition(String condition, RequestContext context) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 清除provider中缓存原生平台的session
	 * 
	 * @param id      平台id
	 * @param context 用户信息
	 */
	default GeneralResult clearSession(Long id, RequestContext context) {
		throw new UnsupportedOperationException();
	}

	default GeneralResult updateExcessRatio(Long id, CloudVendor cloudVendor, RequestContext context) {
		throw new UnsupportedOperationException();
	}

	default GeneralResult summary(Long id, RequestContext context) {
		throw new UnsupportedOperationException();
	}

	default GeneralResult getQuoteInfo(Long vendorId) {
		throw new UnsupportedOperationException();
	}

	/**
	 * 插件云平台基础信息验证
	 * 
	 * @param vendor
	 * @return
	 */
	default GeneralResult<CloudVendor> auth(CloudVendor vendor) {
		return ResultTools.noImplementedResult();
	}

	/**
	 * 查询云平台连接状态方法，该方法主服务定时任务会使用，如果插件中不实现该方法，在插件内部自己写定时任务，矫正平台状态也可以。
	 * 
	 * @param vendor
	 * @return
	 */
	default GeneralResult monitor(CloudVendor vendor) {
		return ResultTools.noImplementedResult();
	}

	/**
	 * 云平台统计信息
	 *
	 * @param vendor
	 * @return
	 */
	default GeneralResult summary(CloudVendor vendor) {
		return ResultTools.noImplementedResult();
	}

	default GeneralResult snapshotConfig(Long vendorId, CloudVendorBean bean, RequestContext context) {
		return ResultTools.noImplementedResult();
	}
}
