package com.bocloud.cmp.service;

import java.util.List;

import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.DataStore;
import com.bocloud.cmp.entity.Server;
import com.bocloud.cmp.model.DataStoreBean;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;

/**
 * 数据存储服务接口
 *
 * @author weiwei
 * @version 1.0
 * @since 2016.12.29
 */
public interface DataStoreService {

    /**
     * 列表
     *
     * @param pager
     * @param context
     * @return
     */
    GeneralResult<GridBean<DataStore>> list(Pager pager, RequestContext context);

    /**
     * 添加数据存储
     *
     * @param dataStore
     * @param context
     * @return
     */
    GeneralResult<DataStore> create(DataStore dataStore, RequestContext context);

    /**
     * 更新
     *
     * @param bean   基本信息
     * @param userId 操作用户ID
     * @return 操作结果
     */
    GeneralResult<DataStore> modify(DataStoreBean bean, Long userId);

    /**
     * 移除
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    GeneralResult remove(Long id, RequestContext context);

    /**
     * 详细信息
     *
     * @param id ID
     * @return 详细信息
     */
    GeneralResult<DataStore> detail(Long id);

    /**
     * 该存储下的云主机
     *
     * @param id
     * @return
     */
    GeneralResult servers(Long id);

    /**
     * 云主机列表展示
     *
     * @param pager
     * @param id
     * @return
     */
    GeneralResult<GridBean<CloudServer>> servers(Pager pager, Long id);

    /**
     * 该存储下的宿主机
     *
     * @param id
     * @return
     */
    GeneralResult<List<Server>> hosts(Long id);

    /**
     * 宿主机列表展示
     *
     * @param pager
     * @param id
     * @return
     */
    GeneralResult<GridBean<Server>> hosts(Pager pager, Long id);

    /**
     * 上传文件至数据存储
     *
     * @param id
     * @param filePath
     * @param context
     * @return
     */
    GeneralResult uploadFile(Long id, String filePath, RequestContext context);

    /**
     * 卸载主机的数据存储
     *
     * @param id
     * @param hostId
     * @param userId
     * @return
     */
    GeneralResult unmount(Long id, Long hostId, Long userId);

    /**
     * 查询宿主机绑定的存储
     *
     * @param hostId
     * @return
     */
    GeneralResult listByHost(Long hostId);

    /**
     * 查询数据存储下关联的宿主机
     *
     * @param storeId
     * @return
     */
    GeneralResult listByStore(Long storeId);

    /**
     * 批量修改标签
     *
     * @param params
     * @param context
     * @return
     */
    GeneralResult batchModifyLabel(String params, RequestContext context);

    GeneralResult condition(String condition, RequestContext context);

}
