package com.bocloud.cmp.service;

import java.util.List;
import java.util.Map;

import com.bocloud.cmp.entity.Rack;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;

/**
 * 机柜抽象Service接口
 * 
 * @version 1.0
 * @since 2016.02.27
 */
public interface RackService {

	/**
	 * 添加机柜
	 * 
	 * @param rack the rack
	 * @return the result
	 */
	public GeneralResult<Rack> create(Rack rack, Long userId);

	/**
	 * 更新机柜
	 * 
	 * @param bean   the bean
	 * @param userId the user id
	 * @return the result
	 */
	public GeneralResult<String> modify(Rack rack, Long userId);

	/**
	 * 逻辑删除机柜
	 * 
	 * @param id the id
	 * @param id the user id
	 * @return the result
	 */
	public GeneralResult<String> remove(Long id, Long userId);

	/**
	 * 查询机柜详细
	 * 
	 * @param id the id
	 * @return the result
	 */
	public GeneralResult<Rack> basic(Long id);

	/**
	 * 查询机柜列表
	 * 
	 * @param page   the page
	 * @param rows   the rows
	 * @param params the params
	 * @param sorter the sorter
	 * @param simple the simple
	 * @return the result
	 */
	public GeneralResult<GridBean<Rack>> list(Pager pager, RequestContext context);

	/**
	 * 得到当前机柜下的主机列表
	 * 
	 * @param page   the page
	 * @param rows   the rows
	 * @param params the params
	 * @param sorter the sorter
	 * @return the result
	 */
	public GeneralResult listServers(int page, int rows, List<Param> params, Map<String, String> sorter);

	/**
	 * 根据机柜查询机柜下机器的ip们
	 * 
	 * @since 2016年10月31日
	 */
	public List<String> listServersRackIps(Long id);

	/**
	 * 根据机房id查询机房下的IP们
	 * 
	 * @since 2016年10月31日
	 */
	public List<String> listServersRoomIps(Long id);

	/**
	 * 根据数据中心id查询数据中心下的IP们
	 * 
	 * @since 2016年10月31日
	 */
	public List<String> listServersDCIps(Long id);

	/**
	 * 得到当前机柜的基本信息和统计信息
	 * 
	 * @param id the id
	 * @return the result
	 */
	public GeneralResult detail(Long id);

	/**
	 * 加入机房
	 * 
	 * @param id     the rack id
	 * @param roomId the room id
	 * @param userId the user id
	 * @return the result
	 */
	public GeneralResult inRoom(Long id, Long roomId, Long userId);

	/**
	 * 移出机房
	 * 
	 * @param id     the rack id
	 * @param userId the user id
	 * @return the result
	 */
	public GeneralResult outRoom(Long id, Long userId);

	/**
	 * 查询空闲机柜列表(没有被分配机房)
	 * 
	 * @return the result
	 */
	public GeneralResult freeRacks();

	/**
	 * 查询空闲机柜列表(没有被分配分区和其他机房)
	 * 
	 * @return the result
	 */
	public GeneralResult freeRacks(Long roomId);

	/**
	 * 检查卡槽数量修改是否合理
	 * 
	 * @param id         the id
	 * @param slotAmount the slot amout
	 * @return the result
	 */
	public GeneralResult checkRange(Long id, Integer slotAmount);

	/**
	 * 检查卡槽位置是否被使用
	 * 
	 * @param id        the id
	 * @param slotStart the slot start
	 * @param slotNum   the slot num
	 * @return the result
	 */
	public GeneralResult checkSlot(Long id, Integer slotStart, Integer slotNum, Long serverId);

	/**
	 * 机柜下没有分配的服务器
	 * 
	 * @param id the id
	 * @return the result
	 */
	public GeneralResult freeNodes(Long id);

	/**
	 * 机柜下没有分配的服务器
	 * 
	 * @param id the id
	 * @return the result
	 */
	public GeneralResult freeManagers(Long id);

	/**
	 * 机柜的卡槽使用情况
	 * 
	 * @param id the id
	 * @return the result
	 */
	public GeneralResult slots(Long id);

	/**
	 * 机柜下的资源
	 * 
	 * @param id
	 * @return
	 */
	public GeneralResult resources(Long id);

	/**
	 * 条件查询详情
	 * 
	 * @param id
	 * @return
	 */
	GeneralResult details(Long id, String action);

	/**
	 * 机柜操作类
	 * 
	 * @param id
	 * @param context
	 * @param params
	 * @param action
	 * @return
	 */
	public GeneralResult<String> operations(RequestContext context, String params, String action);

	public GeneralResult<List<Rack>> condition(String condition, RequestContext context);
}
