package com.bocloud.cmp.service;

import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.entity.PoolGroup;
import com.bocloud.cmp.entity.PoolGroupLabel;
import com.bocloud.cmp.entity.PoolGroupRelation;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;

/**
 * 资源池service
 *
 * @author zhangdi
 * @version 1.0
 * @since 2018年4月23日
 */
public interface ResourcePoolService {

	/**
	 * 查询列表
	 * 
	 * @param page
	 * @param rows
	 * @param params
	 * @param sorter
	 * @param simple
	 * @param condition
	 * @param context
	 * @return
	 */
	GeneralResult<GridBean<PoolGroup>> list(Pager pager, RequestContext context);

	/**
	 * 查询资源池分组树
	 *
	 * @param parentId
	 * @return
	 */
	GeneralResult list(Long parentId, String category);

	/**
	 * 创建资源池分组
	 *
	 * @param group
	 * @return
	 */
	GeneralResult<PoolGroup> create(PoolGroup group, RequestContext context);

	/**
	 * 更新资源池分组
	 *
	 * @param bean
	 * @param userId
	 * @return
	 */
	GeneralResult modify(PoolGroup bean, RequestContext context);

	/**
	 * 删除资源池分组
	 *
	 * @param id
	 * @param userId
	 * @return
	 */
	GeneralResult remove(Long id, RequestContext context);

	/**
	 * 查看资源池分组详情
	 *
	 * @param id
	 * @return
	 */
	GeneralResult<PoolGroup> detail(Long id);

	/**
	 * 判断是否存在
	 *
	 * @param name
	 * @param category
	 * @return
	 */
	GeneralResult exists(String name, String category);

	/**
	 * 查询可绑定的资源池列表
	 *
	 * @param category
	 * @return
	 */
	GeneralResult listByCategory(String category);

	/**
	 * 资源池 目标关联
	 *
	 * @param params
	 * @return
	 */
	GeneralResult createReleation(PoolGroup poolGroupParam);

	/**
	 * 接触资源池 目标关联关系
	 *
	 * @param params
	 * @return
	 */
	GeneralResult removeReleation(PoolGroup poolGroupParam);

	/**
	 * 资源池创建标签
	 *
	 * @param params
	 * @return
	 */
	GeneralResult createLabel(PoolGroup poolGroupParam);

	/**
	 * 资源池删除标签
	 *
	 * @param params
	 * @return
	 */
	GeneralResult removeLabel(PoolGroup poolGroupParam);

	/**
	 * 查询业务绑定的资源池列表
	 *
	 * @param object
	 * @return
	 */
	GeneralResult listByBusinessId(JSONObject object);

	/**
	 * 资源池绑定的标签
	 *
	 * @param params
	 * @return
	 */
	GeneralResult<List<PoolGroupLabel>> listPoolGroupLabel(String params);

	/**
	 * 根据业务 平台查询资源池列表
	 * 
	 * @param object
	 * @param context
	 * @return
	 */
	GeneralResult listByBidVid(JSONObject object, RequestContext context);

	/**
	 * 查询资源池下绑定资源数据
	 *
	 * @param object
	 * @return
	 */
	GeneralResult listRelationByGroupId(JSONObject object);

	/**
	 * 资源池列表查询
	 *
	 * @param params
	 * @param user
	 * @return
	 */
	GeneralResult groups(String params, RequestContext context);

	/**
	 * 绑定/解绑策略
	 * 
	 * @param id
	 * @param policyId
	 * @param priorityType
	 * @param context
	 * @return
	 */
	GeneralResult patch(Long id, Long policyId, String priorityType, RequestContext context);

	GeneralResult condition(Pager pager, String condition, RequestContext context);

	/**
	 * 查询资源池容量信息
	 *
	 * @param id
	 * @return
	 */
	GeneralResult capacity(Long id,RequestContext context);

	/**
	 * 资源池状态调整
	 *
	 * @param id
	 * @param status
	 * @param context
	 * @return
	 */
	GeneralResult updateStatus(List<Long> id, String status, RequestContext context);

	/**
	 * 资源启用、禁用
	 * 
	 * @param poolGroupRelation
	 * @return
	 */
	GeneralResult enabled(PoolGroupRelation poolGroupRelation);

	List<PoolGroup> listTenantPoolGroups(Long tenantId, String vendorType);

	List<Long> listTenantVendorIds(Long tenantId, String vendorType);

	List<String> listTenantRegions(Long tenantId, String vendorType);

    GeneralResult modifyRegionRelation(List<Long> relationIds, Boolean isUsed, RequestContext context);

    /**
     * 根据云平台创建资源池
     * @param vendor
     * @param operator
     */
	void createVendorPoolGroup(CloudVendor vendor, Long operator);

}
