package com.bocloud.cmp.service;

import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.AlterationParam;
import com.bocloud.cmp.domain.ApplicationParam;
import com.bocloud.cmp.domain.CancelationParam;
import com.bocloud.cmp.domain.ExtensionParam;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.Snapshot;
import com.bocloud.cmp.entity.Volume;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;

/**
 * 块设备抽象Service接口
 *
 * @author dongkai
 */
public interface VolumeService {

	/**
	 * 列表展示
	 *
	 * @param page
	 * @param rows
	 * @param params
	 * @param sorter
	 * @param simple
	 * @param context
	 * @param condition
	 * @return
	 */
	public GeneralResult<GridBean<Volume>> list(Pager pager, RequestContext context);

	/**
	 * 详情
	 *
	 * @param id
	 * @return
	 */
	public GeneralResult<Volume> detail(Long id);

	/**
	 * 挂载机器
	 *
	 * @param id
	 * @return
	 */
	public GeneralResult<List<CloudServer>> servers(Long id);

	/**
	 * 查询未添加进映射的卷
	 *
	 * @param page
	 * @param rows
	 * @param paramMap
	 * @param sorter
	 * @return
	 */
	GeneralResult listVolumeNotInView(Integer page, Integer rows, Long accessPathId, Long viewUuid,
			List<Param> paramMap, Map<String, String> sorter);

	/**
	 * 查询已添加进映射的卷
	 *
	 * @param page
	 * @param rows
	 * @param paramMap
	 * @param sorter
	 * @return
	 */
	GeneralResult listVolumeInView(Integer page, Integer rows, Long accessPathId, Long viewUuid, List<Param> paramList,
			Map<String, String> sorter);


	/**
	 * 查询与客户端组映射的卷
	 *
	 * @param page
	 * @param rows
	 * @param paramMap
	 * @param sorter
	 * @return
	 */
	public GeneralResult<GridBean> listByHost(Integer page, Integer rows, Long accessPathId, Map<String, Object> paramMap,
			Map<String, String> sorter);

	public GeneralResult<GridBean> listInSyncedSet(Integer page, Integer rows, Map<String, Object> paramMap,
			Map<String, String> sorter);

	public GeneralResult<GridBean> listNotInSyncedSet(Integer page, Integer rows, Map<String, Object> paramMap,
			Map<String, String> sorter);

	public GeneralResult<List<Volume>> listByStatus(String params, RequestContext context);

	public GeneralResult<List<Volume>> listByVendor(Long vendorId);

	public GeneralResult<List<Volume>> listByImage(Long imageId);

	public GeneralResult<String> batchRemove(List<Long> ids, RequestContext context);

	public GeneralResult<String> uploadToImage(String params, RequestContext context);

	public GeneralResult<Integer> count(Long userId, Long projectId, Long tenantId);

	public GeneralResult attach(Long id, Long serverId, RequestContext context, String mountpoint);

	public GeneralResult<String> onboard(String params, RequestContext context);

	public GeneralResult<String> unmanage(RequestContext context, Long vendorId, Long storageId, List<Long> volumeIds);

	public GeneralResult<List<Volume>> listByStorage(Long storageId);

	public GeneralResult<List<Volume>> listByTemplate(Long templateId);

	/**
	 * 申请云硬盘
	 *
	 * @param param
	 * @return
	 */
	public GeneralResult apply(ApplicationParam param);

	/**
	 * 云硬盘变更
	 *
	 * @param param
	 * @return
	 */
	GeneralResult alteration(AlterationParam param);

	/**
	 * 云硬盘延期
	 *
	 * @param param
	 * @return
	 */
	GeneralResult extension(ExtensionParam param);

	/**
	 * 云硬盘退订
	 *
	 * @param param
	 * @return
	 */
	GeneralResult cancelation(CancelationParam param);

	/**
	 * 查询Powervc云主机创建时可添加的卷
	 *
	 * @param vendorId
	 *
	 */
	public GeneralResult<GridBean> listForPowervcVMCreate(int page, int rows, String keyword, long vendorId);

	/**
	 * 查询Powervc已创建的云主机可附加的卷
	 *
	 * @param vendorId
	 * @param vmId
	 */
	public GeneralResult<GridBean> listForPowervcVMAttach(int page, int rows, String keyword, long vendorId, long vmId);

	/**
	 * 查询Powervc创建映像时可添加的卷
	 *
	 * @param vendorId
	 */
	public GeneralResult<GridBean> listForPowervcImageCreate(int page, int rows, String keyword, long vendorId);

	public GeneralResult<List<Volume>> listByCondition(JSONObject object, RequestContext context);

	public GeneralResult<List<Volume>> listByVmId(JSONObject object);

	public GeneralResult<List<Volume>> listDataDisks(JSONObject object);

	public GeneralResult createOsVolume(JSONObject param, RequestContext context);

	public GeneralResult<Object> listBackupResources(String params, RequestContext context);

	public GeneralResult<GridBean<Snapshot>> listSnapshotById(Long id, Pager pager);
}
