package com.bocloud.cmp.service.impl;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Random;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.IPAddress;
import com.bocloud.cmp.domain.NotifyMessage;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.Ip;
import com.bocloud.cmp.entity.Ip.IpStatus;
import com.bocloud.cmp.entity.IpPool;
import com.bocloud.cmp.entity.Server;
import com.bocloud.cmp.model.IpPatchParams;
import com.bocloud.cmp.model.IpScanParam;
import com.bocloud.cmp.repository.CloudServerRepository;
import com.bocloud.cmp.repository.IpPoolRepository;
import com.bocloud.cmp.repository.IpRepository;
import com.bocloud.cmp.repository.ServerRepository;
import com.bocloud.cmp.service.IpService;
import com.bocloud.cmp.util.ResourceEventPublisher;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.OperateResult.OperateCategory;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @author wangyu
 * @version 5.1.0
 * @description IP业务接口实现
 * @date: 2020-11-30
 */
@Service("ipService")
@Slf4j
public class IpServiceImpl implements IpService {

	@Autowired
	private LockFactory lockFactory;
	@Autowired
	private IpRepository ipRepository;
	@Autowired
	private IpPoolRepository ipPoolRepository;
	@Autowired
	private CloudServerRepository cloudServerRepository;
	@Autowired
	private ServerRepository serverRepository;
	@Autowired
	private ResourceEventPublisher resourceEventPublisher;
	@Autowired
	private AmqpTemplate amqpTemplate;

	@Override
	public GeneralResult<GridBean<Ip>> list(Pager pager) {
		try {
			List<Object> pools = null;

			List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
			Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
			sorter.put("gmtCreate", Common.ONE);
			Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
			Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);

			GridBean gridBean = null;
			List<Ip> list = ipRepository.list(page, rows, params, sorter);
			for (Ip ip : list) {
				if (null != ip.getTarget() && null != ip.getCatalog()) {
					if ("VM".equalsIgnoreCase(ip.getCatalog())) {
						CloudServer server = cloudServerRepository.query(ip.getTarget());
						if (null != server) {
							ip.setTargetName(server.getName());
						}
					} else if ("PM".equalsIgnoreCase(ip.getCatalog())) {
						Server host = serverRepository.query(ip.getTarget());
						if (null != host) {
							ip.setTargetName(host.getName());
						}
					}
				}
			}
			int total = ipRepository.count(params);
			gridBean = GridHelper.getBean(page, rows, total, list);
			return new GeneralResult<>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("list ip failure:", e);
			return new GeneralResult<>(false, "查询失败", null, null);
		}

	}

	@Override
	public GeneralResult condition(String condition, RequestContext context) {
		try {
			List<Object> pools = null;
			JSONObject object = JSONObject.parseObject(condition);
			if ("achieveIps".equalsIgnoreCase(object.getString("condition"))) {
				GeneralResult<List<Ip>> result = this.achieveIps(object.getLong("count"), "");
				return new GeneralResult<>(result.isSuccess(), result.getData(), result.getMessage());
			} else if ("alarmIps".equalsIgnoreCase(object.getString("condition"))) {
				return this.alarmIps(object.getLong("vendorId"));
			} else if ("preAllocation".equalsIgnoreCase(object.getString("condition"))) {
				return this.preAllocation(object.getLong("poolId"), object.getInteger("count"), context);
			} else if ("checkIp".equalsIgnoreCase(object.getString("condition"))) {
				List<String> ips = JSONArray.parseArray(object.getString("ips"), String.class);
				return this.checkIp(object.getLong("poolId"), ips, context);
			}
//			if ("ipCount".equalsIgnoreCase(object.getString("condition"))) {
//				String poolName = null;
//				Long vendorId = null;
//				Iterator<Param> iterator = params.iterator();
//				while (iterator.hasNext()) {
//					Param param = iterator.next();
//					Map<String, Object> map = param.getParam();
//					for (String key : map.keySet()) {
//						if ("poolName".equals(key)) {
//							poolName = String.valueOf(map.get(key));
//							iterator.remove();
//							break;
//						}
//						if ("vendorId".equals(key)) {
//							vendorId = Long.valueOf((int) map.get(key));
//							break;
//						}
//					}
//				}
//				// 根据条件查询ip池
//				ArrayList<Param> paramsData = new ArrayList<>();
//				if (null != vendorId && 0L != vendorId) {
//					paramsData.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
//				}
//				if (null != poolName) {
//					paramsData.add(new Param(MapTools.simpleMap("name", poolName), Sign.LK));
//				}
//				List<IpPool> ipPools = ipPoolRepository.list(1, Integer.MAX_VALUE, paramsData, null);
//				pools = Lists.newArrayList();
//				for (IpPool ipPool : ipPools) {
//					pools.add(ipPool.getId());
//				}
//				if (!pools.isEmpty()) {
//					params.add(new Param(MapTools.simpleMap("pool_id", InBuilder.build(pools)), Sign.IN));
//				}
//			}
//			GridBean gridBean = null;
//			if (null == sorter) {
//				sorter = Maps.newHashMap();
//			}
//			List<Ip> list = ipRepository.list(page, rows, params, sorter);
//			for (Ip ip : list) {
//				if (null != ip.getTarget() && null != ip.getCatalog()) {
//					if ("VM".equalsIgnoreCase(ip.getCatalog())) {
//						CloudServer server = cloudServerRepository.query(ip.getTarget());
//						if (null != server) {
//							ip.setTargetName(server.getName());
//						}
//					} else if ("PM".equalsIgnoreCase(ip.getCatalog())) {
//						Server host = serverRepository.query(ip.getTarget());
//						if (null != host) {
//							ip.setTargetName(host.getName());
//						}
//					}
//				}
//			}
//			int total = ipRepository.count(params);
//			gridBean = GridHelper.getBean(page, rows, total, list);
//			return new GeneralResult<>(true, gridBean, "查询成功");
			return new GeneralResult<>(false, "参数格式错误");
		} catch (Exception e) {
			log.error("list ip failure:", e);
			return new GeneralResult<>(false, "查询失败", null, null);
		}

	}

	@Override
	public GeneralResult<Ip> detail(Long id) {
		try {
			Ip ip = this.ipRepository.query(id);
			if (null == ip) {
				return new GeneralResult<>(false, "IP不存在");
			}
			if (null != ip.getTarget() && null != ip.getCatalog()) {
				if ("VM".equalsIgnoreCase(ip.getCatalog())) {
					CloudServer server = cloudServerRepository.query(ip.getTarget());
					if (null != server) {
						ip.setTargetName(server.getName());
					}
				} else if ("PM".equalsIgnoreCase(ip.getCatalog())) {
					Server host = serverRepository.query(ip.getTarget());
					if (null != host) {
						ip.setTargetName(host.getName());
					}
				}
			}
			return new GeneralResult<>(true, ip, "查询IP详情成功");
		} catch (Exception e) {
			log.error("Get IP Exception：", e);
			return new GeneralResult<>(false, "查询IP详情失败");
		}
	}

	@Override
	public GeneralResult<List<Ip>> achieveIps(Long count, String version) {
		String path = Ip.class.getSimpleName() + "_vendor_assign";
		List<Ip> assignIps = new ArrayList<>();
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			if (StringUtils.isEmpty(version)) {
				version = "V4";
			}
			List<IpPool> pools = ipPoolRepository.listByIpVersion(version);
			Assert.notEmpty(pools, "当前所选云平台下未找到IP池数据");
			List<String> statusList = new ArrayList<>();
			statusList.add(IpStatus.free.name());
			// 查询所有未使用ip地址
			List<Ip> unusedIps = new ArrayList<>();
			for (IpPool pool : pools) {
				List<Ip> ips = ipRepository.listByStatus(statusList, pool.getId());
				if (!ListTool.isEmpty(ips)) {
					unusedIps.addAll(ips);
				}
			}
			if (unusedIps.size() < count) {
				return new GeneralResult<>(false, "可用ip不足");
			}
			for (Ip ip : unusedIps) {
				if (assignIps.size() == count) {
					break;
				}
				// 分配之前先ping
//				if (ping(ip.getIp(), 4, 2000)) {
//					ip.setStatus(IpStatus.locked);
//					ip.setTarget(null);
//					ip.setCatalog(null);
//					ipRepository.update(ip);
//					continue;
//				}
				Ip existIp = ipRepository.query(ip.getId());
				if (null == existIp || IpStatus.free != existIp.getStatus()) {
					continue;
				}
				existIp.setCatalog("VM");
				existIp.setStatus(IpStatus.preempt);
				ipRepository.update(existIp);
				assignIps.add(existIp);
			}
			if (assignIps.isEmpty() || assignIps.size() < count) {
				return new GeneralResult<>(false, "ip池下无IP可分配!");
			}
			log.info("分配的ip地址：{}", JSONObject.toJSON(assignIps));
			return new GeneralResult<>(true, assignIps, "success");
		} catch (Exception e) {
			for (Ip assignIp : assignIps) {
				ipRepository.makeFreeById(assignIp.getId());
			}
			log.error("Achieve Ips failed:", e);
			return new GeneralResult<>(false, "获取IP失败：" + e.getMessage());
		}
	}

	@Override
	public GeneralResult<List<Ip>> assginIps(Long poolId, Long count, String version) {
		String path = Ip.class.getSimpleName() + "_vendor_assign" + poolId;
		List<Ip> assignIps = new ArrayList<>();
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			if (StringUtils.isEmpty(version)) {
				version = "V4";
			}
			IpPool ipPool = ipPoolRepository.query(poolId);
			Assert.notNull(ipPool, "当前所选云平台下未找到IP池数据");
			List<String> statusList = new ArrayList<>();
			statusList.add(IpStatus.free.name());
			// 查询所有未使用ip地址
			List<Ip> unusedIps = new ArrayList<>();
			List<Ip> ips = ipRepository.listByStatus(statusList, ipPool.getId());
			if (!ListTool.isEmpty(ips)) {
				unusedIps.addAll(ips);
			}
			if (unusedIps.size() < count) {
				return new GeneralResult<>(false, "可用ip不足");
			}
			for (Ip ip : unusedIps) {
				if (assignIps.size() == count) {
					break;
				}
				Ip existIp = ipRepository.query(ip.getId());
				if (null == existIp || IpStatus.free != existIp.getStatus()) {
					continue;
				}
				existIp.setCatalog("VM");
				existIp.setStatus(IpStatus.preempt);
				ipRepository.update(existIp);
				assignIps.add(existIp);
			}
			if (assignIps.isEmpty() || assignIps.size() < count) {
				return new GeneralResult<>(false, "ip池下无IP可分配!");
			}
			log.info("分配的ip地址：{}", JSONObject.toJSON(assignIps));
			return new GeneralResult<>(true, assignIps, "success");
		} catch (Exception e) {
			for (Ip assignIp : assignIps) {
				ipRepository.makeFreeById(assignIp.getId());
			}
			log.error("Achieve Ips failed:", e);
			return new GeneralResult<>(false, "获取IP失败：" + e.getMessage());
		}
	}

	@Override
	public GeneralResult processRubbishIp(Long vendorId) {
		return new GeneralResult<>(false, "暂未开发");
	}

	/**
	 * 从网络资源池下的IP池中分配IP地址
	 *
	 * @param vendorId
	 * @param poolGroupId
	 * @param count
	 * @return
	 */
	@Override
	public GeneralResult<List<Ip>> achieveIps(Long vendorId, Long poolGroupId, Integer count) {
		return new GeneralResult<>(false, "暂未开发");
	}

	@Override
	public GeneralResult<List<String>> alarmIps(Long vendorId) {
		return new GeneralResult<>(false, "暂未开发");
	}

	@Override
	public GeneralResult<List<Ip>> achieveNsxIps(Long routerPortId, Integer count, String version) {
		return new GeneralResult<>(false, "暂未开发");
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public GeneralResult deleteBySubnetId(String ip, Long subnetId) {
		return new GeneralResult<>(false, "暂未开发");
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public GeneralResult batchSave(JSONObject object, RequestContext context) {
		return new GeneralResult<>(false, "暂未开发");
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public GeneralResult remove(Long id) {
		String path = Ip.class.getSimpleName() + "_" + id;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Boolean flag = ipRepository.delete(id);
			if (!flag) {
				return new GeneralResult<>(false, "删除失败");
			}
			return new GeneralResult<>(true, "删除成功");
		} catch (Exception e) {
			log.error("Get error message：{}", e.getMessage());
			return new GeneralResult<>(false, "删除失败");
		}
	}

	@Override
	public GeneralResult<List<Ip>> achieveDhcpIps(Long poolId, Long count, String version) {
		String path = Ip.class.getSimpleName() + "_vendor_assign";
		List<Ip> assignIps = new ArrayList<>();
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			if (StringUtils.isEmpty(version)) {
				version = "V4";
			}
			List<String> statusList = new ArrayList<>();
			statusList.add(IpStatus.free.name());
			// 查询所有未使用ip地址
			List<Ip> unusedIps = new ArrayList<>();
			List<Ip> ips = ipRepository.listByStatus(statusList, poolId);
			if (!ListTool.isEmpty(ips)) {
				unusedIps = new ArrayList<>(ips);
			}
			if (unusedIps.size() < count) {
				return new GeneralResult<>(false, "可用ip不足");
			}
			// dhcp方式从ip池中随机查找未使用的ip进行分配
			while (assignIps.size() < count) {
				int index = new Random().nextInt(unusedIps.size());
				Ip ip = unusedIps.get(index);
				// 分配之前先ping
				if (ping(ip.getIp(), 4, 2000)) {
					ip.setStatus(IpStatus.lock);
					ip.setTarget(null);
					ip.setCatalog(null);
					ipRepository.update(ip);
					unusedIps.remove(index);
					continue;
				}
				Ip existIp = ipRepository.query(ip.getId());
				if (null == existIp || IpStatus.free != existIp.getStatus()) {
					unusedIps.remove(index);
					continue;
				}
				existIp.setCatalog("VM");
				existIp.setStatus(IpStatus.preempt);
				ipRepository.update(existIp);
				assignIps.add(existIp);
				unusedIps.remove(index);
			}
			if (assignIps.isEmpty() || assignIps.size() < count) {
				return new GeneralResult<>(false, "ip池下无IP可分配!");
			}
			log.info("分配的ip地址：{}", JSONObject.toJSON(assignIps));
			return new GeneralResult<>(true, assignIps, "success");
		} catch (Exception e) {
			for (Ip assignIp : assignIps) {
				ipRepository.makeFreeById(assignIp.getId());
			}
			log.error("Achieve Ips failed:", e);
			return new GeneralResult<>(false, "获取IP失败：" + e.getMessage());
		}
	}

	@Override
	public GeneralResult scan(IpScanParam param, RequestContext context) {
		log.info("接收到扫描ip池请求，请求参数：{}", JSONObject.toJSONString(param));
		Long vendorId = param.getVendorId();
		Long poolId = param.getPoolId();
		List<Long> poolIds = param.getPoolIds();
		String path;
		if (null != vendorId && 0L != vendorId) {
			path = Ip.class.getSimpleName() + "_vendor_" + vendorId;
		} else if (null != poolId && 0L != poolId) {
			path = Ip.class.getSimpleName() + "_pool_" + poolId;
		} else {
			path = Ip.class.getSimpleName() + "_pools";
		}
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			List<IpPool> pools = new ArrayList<>();
			if (null != vendorId && 0L != vendorId) {
				pools = ipPoolRepository.listByIpVersion("V4");
			} else if (null != poolId && 0L != poolId) {
				IpPool pool = ipPoolRepository.query(poolId);
				Assert.notNull(pool, "IP池不存在");
				pools.add(pool);
			} else {
				pools = ipPoolRepository.pools(poolIds);
			}
			final List<IpPool> finalPools = pools;
			new Thread(() -> {
				log.info("ip扫描任务线程开始！！");
				List<Ip> reachableIps = new ArrayList<>();
				boolean success = true;
				String message = "IP扫描完成，本次扫描未锁定任何IP地址";
				log.info("本次需要被扫描的ip池：【{}】", JSONObject.toJSONString(finalPools));
				if (!ListTool.isEmpty(finalPools)){
					for (IpPool pool : finalPools) {
						List<String> statusList = new ArrayList<>();
						statusList.add(IpStatus.free.name());
						List<Ip> ips = ipRepository.listByStatus(statusList, pool.getId());
						if (!ListTool.isEmpty(ips)){
							log.info("本次需要被扫描的ip池：【{}】", JSONObject.toJSONString(ips.size()));
							if (null == pool.getVendorId()) {
								for (Ip ip : ips) {
									// 如果当前IP池没有关联云平台，则直接修改IP为未使用
									ip.setStatus(IpStatus.free);
									ip.setTarget(null);
									ip.setCatalog(null);
									ipRepository.update(ip);
									continue;
								}
								message = "IP池【" + pool.getName() + "】暂未绑定资源池，无需扫描！";
							} else {
								List<CloudServer> cloudServers = cloudServerRepository.listByVendorId(pool.getVendorId());
								for (Ip ip : ips) {
									log.info("开始处理ip：{}", ip.getIp());
									ip = ipRepository.query(ip.getId());
									if (null == ip || IpStatus.used == ip.getStatus()) {
										continue;
									}
									// IP池扫描，若云主机名称包含此IP，将此IP标记为已使用，主要针对一些因vmtool未开启未获取到IP的机器及关机状态机器
//							CloudServer cloudServer = cloudServerRepository.checkCloudServerName(ip.getIp(),
//									pool.getVendorId());
									// 处理虚机IP 不同平台获取ip有区别
									Boolean flag = true;
									for (CloudServer cloudServer : cloudServers) {
										List<String> vmIps = getIps(cloudServer.getName());
										if (!StringUtils.isEmpty(cloudServer.getName())
												&& vmIps.contains(ip.getIp())) {
											ip.setStatus(IpStatus.used);
											ip.setTarget(cloudServer.getId());
											ip.setVendorId(cloudServer.getVendorId());
											ip.setCatalog("VM");
											ipRepository.update(ip);
											reachableIps.add(ip);
										}
										String privateIps = cloudServer.getPrivateIps();
										if ("VMWARE".equalsIgnoreCase(cloudServer.getVendorType())
												|| "CNWARE".equalsIgnoreCase(cloudServer.getVendorType())
												|| "ZSTACK".equalsIgnoreCase(cloudServer.getVendorType())
												|| "CAS".equalsIgnoreCase(cloudServer.getVendorType())
												|| "USPHERE".equalsIgnoreCase(cloudServer.getVendorType())) {
											List<JSONObject> objects = JSONObject.parseArray(privateIps, JSONObject.class);
											if (!ListTool.isEmpty(objects)) {
												for (JSONObject object : objects) {
													String vmIp = object.getString("address");
													if (!StringUtils.isEmpty(vmIp) && vmIp.equalsIgnoreCase(ip.getIp())) {
														ip.setStatus(IpStatus.used);
														ip.setTarget(cloudServer.getId());
														ip.setVendorId(cloudServer.getVendorId());
														ip.setCatalog("VM");
														ipRepository.update(ip);
														reachableIps.add(ip);
														flag = false;
													}
												}
											}
										}
										if ("SMARTX".equalsIgnoreCase(cloudServer.getVendorType())) {
											List<JSONObject> objects = JSONObject.parseArray(privateIps, JSONObject.class);
											if (!ListTool.isEmpty(objects)) {
												for (JSONObject object : objects) {
													String smartxIp = object.getString("ip");
													if (!StringUtils.isEmpty(smartxIp)
															&& smartxIp.equalsIgnoreCase(ip.getIp())) {
														ip.setStatus(IpStatus.used);
														ip.setTarget(cloudServer.getId());
														ip.setVendorId(cloudServer.getVendorId());
														ip.setCatalog("VM");
														ipRepository.update(ip);
														reachableIps.add(ip);
														flag = false;
													}
												}
											}
										}
									}
									if (flag) {
										if (ping(ip.getIp(), 4, 2000)) {
											ip.setStatus(IpStatus.lock);
											ip.setTarget(null);
											ip.setCatalog(null);
											ipRepository.update(ip);
											reachableIps.add(ip);
										}
									}
									log.info("结束处理ip：{}", ip.getIp());
								}
							}
						}else {
							log.error("此ip池【{}】暂无可扫描ip，请检查！！！", pool.getName());
						}
					}
				}else {
					log.error("暂无ip池，请检查！！！");
				}

				if (!ListTool.isEmpty(reachableIps)) {
					String ipMsg = "";
					for (Ip ip : reachableIps) {
						ipMsg += ip.getIp() + ",";
					}
					if (ipMsg.lastIndexOf(",") > 0) {
						ipMsg = ipMsg.substring(0, ipMsg.lastIndexOf(","));
					}
					message = "IP扫描完成，本次扫描锁定了以下IP地址：【" + ipMsg + "】";
				}
				resourceEventPublisher.send(new OperateResult(success, message, "resource.ip", null,
						OperateCategory.ResourceEvent, context));
				// 发送站内信
				NotifyMessage notifyMessage = new NotifyMessage(context, "IP扫描任务", message, true, false);
				amqpTemplate.convertAndSend("event.notice.queue", notifyMessage);
			}).start();
			return new GeneralResult<>(true, "IP地址扫描任务已下发，请耐心等待扫描结果...");
		} catch (Exception e) {
			log.error("Get error message:{}", e);
			return new GeneralResult<>(false, e.getMessage());
		}
	}
	
	public static List<String> getIps(String ipString) {
		List<String> ips = new ArrayList<String>();
		try {
			if (!StringUtils.isEmpty(ipString)) {
				String regEx = "((2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(2[0-4]\\d|25[0-5]|[01]?\\d\\d?)";
				Pattern p = Pattern.compile(regEx);
				Matcher m = p.matcher(ipString);
				while (m.find()) {
					String result = m.group();
					ips.add(result);
				}
			}
		} catch (Exception e) {
			log.info("根据名称查询虚机IP异常：" + e);
		}
		return ips;
	}

	public static boolean ping(String ipAddress, int pingTimes, int timeOut) {
		log.info("开始ping{}", ipAddress);
		BufferedReader in = null;
		Runtime r = Runtime.getRuntime(); // 将要执行的ping命令,此命令是windows格式的命令
		String osName = System.getProperty("os.name");
		String pingCommand = "ping " + ipAddress + " -c " + pingTimes;
		if (osName.contains("Windows") || osName.contains("windows")) {
			pingCommand = "ping " + ipAddress + " -n " + pingTimes + " -w " + timeOut;
		}
		try {
			Process p = r.exec(pingCommand);
			if (p == null) {
				return false;
			}
			in = new BufferedReader(new InputStreamReader(p.getInputStream())); // 逐行检查输出,计算类似出现=23ms TTL=62字样的次数
			int connectedCount = 0;
			String line = null;
			while ((line = in.readLine()) != null) {
				connectedCount += getCheckResult(line);
			}
			// 如果出现类似=23ms TTL=62这样的字样,出现的次数=测试次数则返回真
			log.info("结束ping{},ping次数{}", ipAddress, pingTimes);
			return connectedCount == pingTimes;
		} catch (Exception ex) {
			return false;
		} finally {
			try {
				if (null != in) {
					in.close();
				}
			} catch (IOException e) {
				log.error("异常：" + e);
			}
		}
	}

	// 若line含有=18ms TTL=16字样,说明已经ping通,返回1,否則返回0.
	private static int getCheckResult(String line) {
		if (line.contains("ttl") || line.contains("TTL")) {
			return 1;
		}
		return 0;
	}

	@Override
	public GeneralResult<List<Ip>> preAllocation(Long poolId, Integer count, RequestContext context) {
		String path = Ip.class.getSimpleName() + "_pool_" + poolId;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			IpPool pool = ipPoolRepository.query(poolId);
			Assert.notNull(pool, "IP池不存在");
			// Ip池已使用IP地址
			List<Ip> ips = ipRepository.listByStatus(Arrays.asList("used", "lock", "preempt"), poolId);
			Map<String, Ip> map = Maps.newHashMap();
			for (Ip ip : ips) {
				map.put(ip.getIp(), ip);
			}
			// 已锁定IP地址
			List<Ip> lockIps = ipRepository.listLockedIps();
			for (Ip ip : lockIps) {
				map.put(ip.getIp(), ip);
			}
			List<Ip> reachableIps = new ArrayList<>();
			int index = 0;
			List<Ip> poolIds = ipRepository.listByPool(poolId);
			for (Ip ip : poolIds) {
				String ipKey = ip.getIp();
				if (!map.containsKey(ipKey)) {
					reachableIps.add(ip);
					index++;
				}
				if (index >= count) {
					break;
				}
			}
			if (reachableIps.isEmpty()) {
				return new GeneralResult<>(false, "该IP池下无IP可分配!");
			}
			if (reachableIps.size() < count) {
				return new GeneralResult<>(false, "此IP池下可分配IP不足!");
			}
			return new GeneralResult<>(true, reachableIps, "分配成功");
		} catch (Exception e) {
			log.error("Pre allocation Ip failed:{}", e.getMessage());
			return new GeneralResult<>(false, "分配IP失败：" + e.getMessage());
		}
	}

	@Override
	public GeneralResult checkIp(Long poolId, List<String> ips, RequestContext context) {
		try {
			if (ListTool.isEmpty(ips)) {
				return new GeneralResult<>(false, "无IP可校验");
			}
			IpPool ipPool = ipPoolRepository.query(poolId);
			if(null == ipPool) {
				return new GeneralResult<>(false, "IP池信息异常！");
			}
			IPAddress ipAddress = IPAddress.get("v4");
			for (String ip : ips) {
				boolean flag = ipAddress.isInPools(ip, JSON.parseArray(ipPool.getPools(), String.class));
				if(!flag) {
					return new GeneralResult<>(false, "IP地址【" + ip + "】不在IP地址池范围内，请重新选择");
				}
				Ip database = ipRepository.checkIp(poolId, ip);
				if (null != database) {
					return new GeneralResult<>(false, "IP地址【" + ip + "】已被使用，请重新设置");
				}
			}
			return new GeneralResult<>(true, "IP校验通过");
		} catch (Exception e) {
			log.error("Get error message:{}", e.getMessage());
			return new GeneralResult<>(false, e.getMessage());
		}
	}

	@Override
	public GeneralResult<List<Ip>> achievePoolIps(Long poolId, Long count, String version) {
		IpPool pool = null;
		try {
			pool = ipPoolRepository.query(poolId);
			Assert.notNull(pool, "IP池不存在");
		} catch (Exception e) {
			log.error("Get error mesage：{}", e.getMessage());
			return new GeneralResult<>(false, e.getMessage());
		}
		String path = Ip.class.getSimpleName() + "_vendor_assign";
		List<Ip> assignIps = new ArrayList<>();
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(600, TimeUnit.SECONDS), "请求超时");
			List<String> statusList = new ArrayList<>();
			// 查询所有未使用ip地址
			statusList.add(IpStatus.free.name());
			List<Ip> unusedIps = ipRepository.listByStatus(statusList, poolId);
			for (Ip ip : unusedIps) {
				if (assignIps.size() == count) {
					break;
				}
				if (ping(ip.getIp(), 4, 2000)) {
					ip.setStatus(IpStatus.lock);
					ip.setTarget(null);
					ip.setCatalog(null);
					ipRepository.update(ip);
					continue;
				}
				Ip existIp = ipRepository.query(ip.getId());
				if (null == existIp || IpStatus.free != existIp.getStatus()) {
					continue;
				}
				existIp.setCatalog("VM");
				existIp.setStatus(IpStatus.preempt);
				ipRepository.update(existIp);
				assignIps.add(existIp);
			}
			if (assignIps.isEmpty() || assignIps.size() < count) {
				return new GeneralResult<>(false, "ip池下无IP可分配!");
			}
			log.info("分配的ip地址：{}", JSONObject.toJSON(assignIps));
			return new GeneralResult<>(true, assignIps, "success");
		} catch (Exception e) {
			for (Ip assignIp : assignIps) {
				assignIp.setStatus(IpStatus.free);
				ipRepository.update(assignIp);
			}
			log.error("Achieve Ips failed:", e);
			return new GeneralResult<>(false, "获取IP失败：" + e.getMessage());
		}
	}

	@Override
	public GeneralResult<List<Ip>> achieveNsxSegmentIps(Long segmentId, Integer count, String version) {
		return new GeneralResult<>(false, "暂未开发");
	}

	@Override
	public GeneralResult action(Long id, IpPatchParams.Action status, String remark, RequestContext context) {
		String path = Ip.class.getSimpleName() + "_" + id;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			Ip ip = ipRepository.query(id);
			Assert.notNull(ip, "ip不存在");
			switch (status) {
			case lock:
				ip.setStatus(IpStatus.lock);
				ip.setCatalog(null);
				ip.setRemark(remark);
				ipRepository.update(ip);
				return new GeneralResult<>(true, "编辑成功");
			case free:
				ipRepository.makeFreeById(ip.getId());
				return new GeneralResult<>(true, "释放成功");
			default:
				return new GeneralResult<>(false, "暂不支持");
			}
		} catch (Exception e) {
			log.error("Get error message：{}", e.getMessage());
			return new GeneralResult<>(false, "操作失败：" + e.getMessage());
		}
	}

	@Override
	public GeneralResult batchUpdate(List<Long> ids, IpPatchParams.Action action, String remark,
			RequestContext context) {
		new Thread(() -> {
			boolean success = true;

			ArrayList<Ip> ips = new ArrayList<>();
			for (Long id : ids) {
				String path = Ip.class.getSimpleName() + "_" + id;
				try (AutoCloseLock lock = lockFactory.getACLock(path)) {
					Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
					Ip ip = ipRepository.query(id);
					Assert.notNull(ip, "ip不存在");
					IpPool pool = ipPoolRepository.query(ip.getPoolId());
					Assert.notNull(pool, "ip池信息异常");
					switch (action) {
					case lock:
						ip.setStatus(IpStatus.lock);
						ip.setCatalog(null);
						ip.setRemark(remark);
						ips.add(ip);
						ipRepository.update(ip);
						resourceEventPublisher.send(new OperateResult(success, "IP【" + ip.getIp() + "】锁定成功",
								"resource.ip", null, OperateCategory.ResourceEvent, context));
						continue;
					case free:
						// IP池扫描，若云主机名称包含此IP，将此IP标记为已使用，主要针对一些因vmtool未开启未获取到IP的机器及关机状态机器
						CloudServer cloudServer = cloudServerRepository.checkCloudServerName(ip.getIp(),
								pool.getVendorId());
						// 判单虚机privateIps失败包含ip
						if (null == cloudServer) {
							cloudServer = cloudServerRepository.checkCloudServerPrivateIps(ip.getIp(),
									pool.getVendorId());
						}
						if (null != cloudServer) {
							resourceEventPublisher
									.send(new OperateResult(false, "当前IP【" + ip.getIp() + "】已被云主机使用，无法手动释放",
											"resource.ip", null, OperateCategory.ResourceEvent, context));
							return;
						}
						if (ping(ip.getIp(), 4, 1000)) {
							resourceEventPublisher.send(new OperateResult(false, "当前IP【" + ip.getIp() + "】正在被使用，不允许释放",
									"resource.ip", null, OperateCategory.ResourceEvent, context));
							return;
						}
						
						ip.setCatalog(null);
						ip.setTarget(null);
						ip.setRemark(remark);
						ip.setStatus(IpStatus.free);
						ips.add(ip);
						ipRepository.update(ip);
						resourceEventPublisher.send(new OperateResult(success, "IP【" + ip.getIp() + "】释放成功",
								"resource.ip", null, OperateCategory.ResourceEvent, context));
						continue;
					default:
						resourceEventPublisher.send(new OperateResult(false, "无效的操作", "resource.ip", null,
								OperateCategory.ResourceEvent, context));
						return;
					}
				} catch (Exception e) {
					log.error("Get error message：{}", e.getMessage());
					String operation = null;
					if (IpPatchParams.Action.lock == action) {
						operation = "锁定";
					}
					if (IpPatchParams.Action.free == action) {
						operation = "释放";
					}
					resourceEventPublisher.send(new OperateResult(false, "IP" + operation + "失败：" + e.getMessage(),
							"resource.ip", null, OperateCategory.ResourceEvent, context));
					return;
				}
			}
		}).start();
		return new GeneralResult<>(true, "任务已下发，正在执行...");
	}
}
