package com.bocloud.cmp.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.cmp.entity.Business;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.entity.LoadBalancer;
import com.bocloud.cmp.entity.OSTenant;
import com.bocloud.cmp.entity.Project;
import com.bocloud.cmp.entity.Tenant;
import com.bocloud.cmp.entity.User;
import com.bocloud.cmp.internal.SmsBusinessService;
import com.bocloud.cmp.internal.SmsProjectService;
import com.bocloud.cmp.internal.SmsTenantService;
import com.bocloud.cmp.internal.SmsUserService;
import com.bocloud.cmp.repository.CloudServerRepository;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.repository.ElasticIpRepository;
import com.bocloud.cmp.repository.FloatingIpRepository;
import com.bocloud.cmp.repository.IpPoolRepository;
import com.bocloud.cmp.repository.IpRepository;
import com.bocloud.cmp.repository.LbListenerRepository;
import com.bocloud.cmp.repository.LoadBalancerRepository;
import com.bocloud.cmp.repository.NetworkCardRepository;
import com.bocloud.cmp.repository.OSTenantRepository;
import com.bocloud.cmp.repository.VpcRepository;
import com.bocloud.cmp.service.ElasticIpService;
import com.bocloud.cmp.service.LoadBalancerService;
import com.bocloud.cmp.util.ResourceEventPublisher;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.OperateResult.OperateCategory;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 负载均衡器service实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年9月4日
 */
@Service("loadBalancerService")
@Slf4j
public class LoadBalancerServiceImpl implements LoadBalancerService {

    private static OperateCategory operateCategory = OperateResult.OperateCategory.ResourceEvent;
    @Autowired
    private LoadBalancerRepository loadBalancerRepository;
    @Autowired
    private CloudVendorRepository cloudVendorRepository;
    @Autowired
    private LbListenerRepository lbListenerRepository;
    @Autowired
    private IpRepository ipRepository;
    @Autowired
    private IpPoolRepository ipPoolRepository;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private SmsProjectService smsProjectService;
    @Autowired
    private FloatingIpRepository floatingIpRepository;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private OSTenantRepository osTenantRepository;
    @Autowired
    private ElasticIpRepository elasticIpRepository;
    @Autowired
    private VpcRepository vpcRepository;
    @Autowired
    private ElasticIpService elasticIpService;
    @Autowired
    private CloudServerRepository cloudServerRepository;
    @Autowired
    private SmsUserService smsUserService;
    @Autowired
    private SmsBusinessService smsBusinessService;
    @Autowired
    private SmsTenantService smsTenantService;
    @Autowired
    private NetworkCardRepository networkCardRepository;

    @Override
    public GeneralResult<GridBean<LoadBalancer>> list(Pager pager, RequestContext context) {
        GridBean gridBean = null;
        List<LoadBalancer> list = Lists.newArrayList();
        int total = 0;
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
            sorter.put("gmtCreate", Common.ONE);
            sorter.put("id", Common.ONE);
            Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
            Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
            Long tenantId = context.getTenant();
            Long projectId = 0L;
            String vendorType = null;
            Long vendorId = null;
            for (Param param : params) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("vendorType")) {
                    vendorType = String.valueOf(paramMap.get("vendorType"));
                    paramMap.remove("vendorType");
                }
                if (paramMap.containsKey("vendorId")) {
                    vendorId = Long.valueOf(String.valueOf(paramMap.get("vendorId")));
                }
            }
            CloudVendor vendor = new CloudVendor();
            if (null != vendorId) {
                vendor = cloudVendorRepository.query(vendorId);
                Assert.notNull(vendor, "不存在此平台");
            }
            if (context.getCatalog().equals(Catalog.Tenant)) {
                // 租户能看到 1.属于当前租户的资源 2.租户下所有项目的资源
                List<Project> tenantProjects = smsProjectService.listTenantProjects(context.getTenant());
                List<Long> projectIds = tenantProjects.stream().map(Project::getId).collect(Collectors.toList());
                total = this.loadBalancerRepository.count(null, context.getTenant(), projectIds, params, vendorType);
                list = this.loadBalancerRepository.list(page, rows, null, context.getTenant(), projectIds, params,
                        sorter, vendorType);
                list = convertor(list);
                gridBean = new GridBean<>(page, rows, total, list);
            } else if (context.getCatalog().equals(Catalog.User)) {
                // 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
                List<Long> projectIds = new ArrayList<>();
                List<Long> userManagerProjectsIds =
                        userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                List<Long> visibleProjectsIds =
                        visibleProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!visibleProjectsIds.isEmpty()) {
                    projectIds.addAll(visibleProjectsIds);
                }
                total = this.loadBalancerRepository.count(context.getTarget(), null, projectIds, params, vendorType);
                list = this.loadBalancerRepository.list(page, rows, context.getTarget(), null, projectIds, params,
                        sorter, vendorType);
                list = convertor(list);
                gridBean = new GridBean<>(page, rows, total, list);
            } else {
                total = loadBalancerRepository.count(params);
                if (simple) {
                    if (0L == context.getTenant()) {
                        params.add(new Param(MapTools.simpleMap(Common.TENANTID, 0), Sign.EQ));
                    }
                    List<SimpleBean> beans = loadBalancerRepository.list(params, sorter);
                    gridBean = new GridBean(1, 1, total, beans);
                } else {
                    if (!"VOLCENGINE".equals(vendor.getType())) {
                        list = this.loadBalancerRepository.list(page, rows, params, sorter);
                    } else {
                        list = loadBalancerRepository.listHaveEip(page, rows, params, sorter);
                    }
                    for (LoadBalancer balancer : list) {
                        params.clear();
                        params.add(new Param(MapTools.simpleMap("lbId", balancer.getId()), Sign.EQ));
                        int listenerNum = lbListenerRepository.count(params);
                        balancer.setListenerNum(listenerNum);
                        if (balancer.getAvailabilityZones() != null) {
                            balancer
                                    .setAvailabilityZones(JSONArray.parseArray(balancer.getAvailabilityZones().toString()));
                        }
                    }
                    gridBean = GridHelper.getBean(page, rows, total, list);
                }
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("list load balancer failure:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private List<LoadBalancer> convertor(List<LoadBalancer> list) {
        try {
            if (!ListTool.isEmpty(list)) {
                List<Long> tenantIds = new ArrayList<>();
                List<Long> userIds = new ArrayList<>();
                List<Long> businessIds = new ArrayList<>();
                List<Long> projectIds = new ArrayList<>();
                for (LoadBalancer boadBalancer : list) {
                    if (boadBalancer.getTenantId() != null) {
                        tenantIds.add(boadBalancer.getTenantId());
                    }
                    if (boadBalancer.getBusinessId() != null) {
                        businessIds.add(boadBalancer.getBusinessId());
                    }
                    if (boadBalancer.getProjectId() != null) {
                        projectIds.add(boadBalancer.getProjectId());
                    }
                    if (boadBalancer.getCreatorId() != null) {
                        userIds.add(boadBalancer.getCreatorId());
                    }
                    if (boadBalancer.getOwnerId() != null) {
                        userIds.add(boadBalancer.getOwnerId());
                    }
                    if (boadBalancer.getMenderId() != null) {
                        userIds.add(boadBalancer.getMenderId());
                    }
                }
                Map<Long, String> userMap = smsUserService.listUsersByIds(userIds).stream()
                        .collect(Collectors.toMap(User::getId, User::getName));
                Map<Long, String> projectMap = smsProjectService.listProjectsByIds(projectIds).stream()
                        .collect(Collectors.toMap(Project::getId, Project::getName));
                Map<Long, String> businessMap = smsBusinessService.listBusinesssByIds(businessIds).stream()
                        .collect(Collectors.toMap(Business::getId, Business::getName));
                Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(tenantIds).stream()
                        .collect(Collectors.toMap(Tenant::getId, Tenant::getName));
                for (LoadBalancer boadBalancer : list) {
                    // 设置云主机归属信息
                    if (boadBalancer.getTenantId() != null) {
                        boadBalancer.setTenantName(tenantMap.get(boadBalancer.getTenantId()));
                    }
                    if (boadBalancer.getBusinessId() != null) {
                        boadBalancer.setBusinessName(businessMap.get(boadBalancer.getBusinessId()));
                    }
                    if (boadBalancer.getProjectId() != null) {
                        boadBalancer.setProjectName(projectMap.get(boadBalancer.getProjectId()));
                    }
                    if (boadBalancer.getCreatorId() != null) {
                        boadBalancer.setCreatorName(userMap.get(boadBalancer.getCreatorId()));
                    }
                    if (boadBalancer.getOwnerId() != null) {
                        boadBalancer.setOwnerName(userMap.get(boadBalancer.getOwnerId()));
                    }
                    if (boadBalancer.getMenderId() != null) {
                        boadBalancer.setMenderName(userMap.get(boadBalancer.getMenderId()));
                    }
                }
            }
        } catch (Exception e) {
            log.error("获取基本信息异常：" + e.getMessage());
        }
        return list;

    }

    @Override
    public GeneralResult detail(Long id) {
        try {
            Long vendorId = loadBalancerRepository.getVendorId(id);
            if (null == vendorId) {
                return new GeneralResult<>(false, "负载均衡器不存在");
            }
            CloudVendor vendor = cloudVendorRepository.query(vendorId);
            Assert.notNull(vendor, "不存在此平台");
            LoadBalancer balancer = null;
            if (!"VOLCENGINE".equals(vendor.getType())) {
                balancer = loadBalancerRepository.query(id);
            } else {
                balancer = loadBalancerRepository.queryHaveEip(id);
            }
            if (null == balancer) {
                return new GeneralResult<>(false, "负载均衡器不存在");
            }
            if (!StringUtils.isEmpty(balancer.getTenantUuid())) {
                OSTenant osTenant =
                        osTenantRepository.getByTenantUuid(balancer.getTenantUuid(), balancer.getVendorId());
                if (null != osTenant) {
                    balancer.setOsTenantName(osTenant.getName());
                }
            }
            List<LoadBalancer> balancers = new ArrayList<>();
            balancers.add(balancer);
            balancer = convertor(balancers).get(0);
            return new GeneralResult<>(true, balancer, "查询成功");
        } catch (Exception e) {
            log.error("query load balancer fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }


    @Override
    public GeneralResult labels(Long id, String labels, RequestContext context) {
        String path = LoadBalancer.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            LoadBalancer loadBalancer = loadBalancerRepository.query(id);
            loadBalancer.setLabels(labels);
            loadBalancer.setMenderId(context.getTarget());
            loadBalancer.setGmtModify(new Date());
            boolean result = loadBalancerRepository.update(loadBalancer);
            if (result) {
                return new GeneralResult(true, "负载均衡器标签成功");
            }
            return new GeneralResult(false, "负载均衡器标签失败");
        } catch (Exception e) {
            log.error("负载均衡器标签失败 {}", e);
            return new GeneralResult(false, "负载均衡器标签失败");
        }
    }

    @Override
    public GeneralResult listAvailableLbs(Long vendorId, String regionId, RequestContext context) {
        try {
            List<LoadBalancer> listAvailableLbs = new ArrayList<>();
            List<LoadBalancer> availableLbs = new ArrayList<>();

            if (context.getCatalog().equals(Catalog.Tenant)) {
                // 租户能看到 1.属于当前租户的资源 2.租户下所有项目的资源
                List<Project> tenantProjects = smsProjectService.listTenantProjects(context.getTenant());
                List<Long> projectIds = tenantProjects.stream().map(Project::getId).collect(Collectors.toList());
                availableLbs =
                        loadBalancerRepository.listAvailableLbs(vendorId, regionId, null, context.getTenant(), projectIds);
            } else if (context.getCatalog().equals(Catalog.User)) {
                // 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
                List<Long> projectIds = new ArrayList<>();
                List<Long> userManagerProjectsIds =
                        userManagerProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                List<Long> visibleProjectsIds =
                        visibleProjects.stream().map(Project::getId).collect(Collectors.toList());
                if (!visibleProjectsIds.isEmpty()) {
                    projectIds.addAll(visibleProjectsIds);
                }
                availableLbs =
                        loadBalancerRepository.listAvailableLbs(vendorId, regionId, context.getTarget(), null, projectIds);
            } else {
                availableLbs = loadBalancerRepository.listAvailableLbs(vendorId, regionId);
            }
            for (LoadBalancer loadBalancer : availableLbs) {
                if (!StringUtils.isEmpty(loadBalancer.getVpcUuid())) {
                    listAvailableLbs.add(loadBalancer);
                }
            }
            return new GeneralResult(true, listAvailableLbs, "查询成功");
        } catch (Exception e) {
            log.error("Get error message:{}", e);
            return new GeneralResult(false, "查询失败");
        }
    }


}
