package com.bocloud.cmp.service.impl;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.entity.Server;
import com.bocloud.cmp.entity.ServerConfig;
import com.bocloud.cmp.model.ChargeResultTrend;
import com.bocloud.cmp.model.ChargeTrendData;
import com.bocloud.cmp.repository.CloudServerRepository;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.repository.ServerConfigRepository;
import com.bocloud.cmp.repository.ServerRepository;
import com.bocloud.cmp.service.ResourceScreenService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.ListTool;

import com.megatron.common.utils.MapTools;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class ResourceScreenServiceImpl implements ResourceScreenService {
    @Autowired
    private CloudVendorRepository cloudVendorRepository;
    @Autowired
    private CloudServerRepository cloudServerRepository;
    @Autowired
    private ServerRepository serverRepository;
    @Autowired
    private ServerConfigRepository serverConfigRepository;

    public static final String END_TIME = " 23:59:59";

    @Override
    public GeneralResult getBasicinfo(String code, RequestContext context) {
        List<Param> params = new ArrayList<>();
        //根据地市code去查询云平台
        params.add(new Param(MapTools.simpleMap("code", code), Sign.EQ));
        List<CloudVendor> list = cloudVendorRepository.list(params);
        if(ListTool.isEmpty(list)){
            return new GeneralResult(false,"未配置云厂商");
        }
        JSONObject jsonObject = new JSONObject();
        int unitCloudNum = 0;
        //省中心编码就是郑州市编码410100
        if(code.equalsIgnoreCase("410100")){
            List<Long> vendorIds;
            List<CloudServer> cloudServers;
            //计算专有域云主机个数
            vendorIds = this.getCloudVendorData(list, "EXCLUSIVE");
            cloudServers = cloudServerRepository.listByVenderIds(vendorIds);
            jsonObject.put("exclusiveCloudNum",cloudServers.size());//专有域云主机
            if(!ListTool.isEmpty(cloudServers)){
                List<Long> departIds = cloudServers.stream().map(CloudServer::getDepartId).collect(Collectors.toList());
                unitCloudNum += departIds.size();
            }
            //计算公有域云主机个数
            vendorIds = this.getCloudVendorData(list, "PUBLIC");
            cloudServers = cloudServerRepository.listByVenderIds(vendorIds);
            jsonObject.put("publicCloudNum",cloudServers.size());//公有域云主机
            if(!ListTool.isEmpty(cloudServers)){
                List<Long> departIds = cloudServers.stream().map(CloudServer::getDepartId).collect(Collectors.toList());
                unitCloudNum += departIds.size();
            }
            jsonObject.put("unitCloudNum",unitCloudNum); //上云单位
            //todo 地市云接入这块目前没有办法拿到数据，我们现在的云管是统计的省云平台，不同地市下面的云平台信息还没办法对接，所以这个值暂时是空的，也就是0
            jsonObject.put("cityCloudNum",0);//地市云接入
        }else{
            List<Long> vendorIds = list.stream().map(CloudVendor::getId).collect(Collectors.toList());
            jsonObject.put("cloudVendorNum",vendorIds.size());//政务云平台
            List<CloudServer> cloudServers = cloudServerRepository.listByVenderIds(vendorIds);
            if(!ListTool.isEmpty(cloudServers)){
                List<Long> departIds = cloudServers.stream().map(CloudServer::getDepartId).collect(Collectors.toList());
                unitCloudNum += departIds.size();
            }
            jsonObject.put("unitCloudNum",unitCloudNum);//上云单位  根据我们纳管的云主机统计，看这些云主机的所属单位都有哪些
            jsonObject.put("deliverCloudNum",cloudServers.size());//已交付云主机 所有云主机
        }
        return new GeneralResult(true,jsonObject,"查询成功");
    }

    @Override
    public GeneralResult getOverview(String type, String code, RequestContext context) {
        List<Param> params = new ArrayList<>();
        if(StringUtils.hasText(code) && !"ALL".equalsIgnoreCase(type)){
            params.add(new Param(MapTools.simpleMap("code", code), Sign.EQ));
        }
        List<CloudVendor> list = cloudVendorRepository.list(params);
        if(ListTool.isEmpty(list)){
            return new GeneralResult(false,"未配置云厂商");
        }
        Map<String,Object> map = new HashMap<>();
        Map<String,Object> cpuMap = new HashMap<>();
        Map<String,Object> memMap = new HashMap<>();
        Map<String,Object> diskMap = new HashMap<>();
        // cpu、内存、磁盘总量和使用量
        int cpuTotal = 0;
        int cpuUsed = 0;
        Double memTotal = 0D;
        Double memUsed = 0D;
        double diskTotal = 0;
        double diskUsed = 0;
        int cloudVendorNum = 0;
        List<Long> vendorIds;
        if(StringUtils.hasText(type)){
            vendorIds = this.getCloudVendorData(list, type);
        }else{
            vendorIds = list.stream().map(CloudVendor::getId).collect(Collectors.toList());
        }
        cloudVendorNum = vendorIds.size();
        // 获取宿主机各状态数量
        List<Server> hostSystems = this.serverRepository.listByVendorIds(vendorIds);
        for (Server host : hostSystems) {
            ServerConfig config = serverConfigRepository.getBySid(host.getId());
            if (null != config) {
                cpuTotal += config.getCpuHz();
                cpuUsed += config.getCpuUsed() == null ? 0 : config.getCpuUsed();
                memTotal += new BigDecimal(config.getMemoryCapacity()).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                memUsed += new BigDecimal(config.getMemUsed() == null ? 0 : config.getMemUsed()).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                diskTotal += config.getDiskCapacity() == null ? 0 : config.getDiskCapacity().doubleValue();
                diskUsed += config.getDiskUsed() == null ? 0 : config.getDiskUsed().doubleValue();
            }
        }
        Double cpuRate = 0D;
        if (cpuTotal == 0) {
            cpuRate = 0D;
        } else {
            cpuRate = new BigDecimal(cpuUsed).divide(new BigDecimal(cpuTotal), 2, RoundingMode.HALF_UP).doubleValue();
        }
        Double memRate = 0D;
        if (memTotal == 0) {
            memRate = 0D;
        } else {
            memRate = new BigDecimal(memUsed).divide(new BigDecimal(memTotal), 2, RoundingMode.HALF_UP).doubleValue();
        }
        Double diskRate = 0D;
        if (diskTotal == 0) {
            diskRate = 0D;
        } else {
            diskRate = new BigDecimal(diskUsed).divide(new BigDecimal(diskTotal), 2, RoundingMode.HALF_UP).doubleValue();
        }
        cpuMap.put("used",cpuUsed);
        cpuMap.put("total",cpuTotal);
        cpuMap.put("rate",cpuRate);

        memMap.put("used",memUsed);
        memMap.put("total",memTotal);
        memMap.put("rate",memRate);

        diskMap.put("used",diskUsed);
        diskMap.put("total",diskTotal);
        diskMap.put("rate",diskRate);

        map.put("cloudVendorNum",cloudVendorNum);//政务云平台(个)
        map.put("cpuTotal",cpuTotal);//VCPU总量(核)
        map.put("memTotal",memTotal);//内存总量(TB)
        map.put("diskTotal",diskTotal);//存储总量(PB)
        map.put("cpu",cpuMap);//cpu: {used: 150, total: 2000, rate: 0.5}
        map.put("mem",memMap);//mem: {used: 150, total: 2000, rate: 0.5}
        map.put("storage",diskMap);//storage: {used: 150, total: 2000, rate: 0.5}
        return new GeneralResult(true,map,"查询成功");
    }

    @Override
    public GeneralResult getCloudHostDeliveryQuantity(String code, RequestContext context) {
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("code", code), Sign.EQ));
        List<CloudVendor> list = cloudVendorRepository.list(params);
        if(ListTool.isEmpty(list)){
            return new GeneralResult(false,"未配置云厂商");
        }
        JSONArray jsonArray = new JSONArray();
        for (CloudVendor cloudVendor : list) {
            JSONObject object = new JSONObject();
            List<CloudServer> cloudServers = cloudServerRepository.listByVendorId(cloudVendor.getId());
            object.put("name",cloudVendor.getName());
            object.put("value",cloudServers.size());
            jsonArray.add(object);
        }
        //[{name: '阿里飞天(专有域)', value: 700}]
        return new GeneralResult(true,jsonArray,"查询成功");
    }

    @Override
    public GeneralResult getCloudHostDeliverystatus(String code, RequestContext context) {
        try {
            //获取3个月内的虚拟机数据
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("code", code), Sign.EQ));
            List<CloudVendor> list = cloudVendorRepository.list(params);
            if(ListTool.isEmpty(list)){
                return new GeneralResult(false,"未配置云厂商");
            }
            List<Long> vendorIds = list.stream().map(CloudVendor::getId).collect(Collectors.toList());
            List<CloudServer> cloudServers = cloudServerRepository.listByVenderIds(vendorIds);
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Date endDate = new Date();
            //3个月之前的日期开始到今天
            Date beginDate = DateUtils.addMonths(endDate, -3);
            List<String> keys = new ArrayList<>();
            JSONObject jsonObject = new JSONObject();
            if(code.equalsIgnoreCase("省中心")){
                for (Long vendorId : vendorIds) {
                    jsonObject.put(String.valueOf(vendorId), new ArrayList<Object>());
                }
            }
            List<Object> cloudServerData = new ArrayList<>();
            List<ChargeTrendData> values = new ArrayList<>();
            while (beginDate.before(endDate)) {
                String key = format.format(beginDate);
                keys.add(key);
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                Date end = simpleDateFormat.parse(String.format("%s%s", key, END_TIME));
                List<CloudServer> targetServers = new ArrayList<>();
                for (CloudServer server : cloudServers) {
                    Date date = server.getGmtCreate();
                    if (date != null && date.before(end)) {
                        targetServers.add(server);
                    }
                }
                if(code.equalsIgnoreCase("省中心")){
                    for (Map.Entry<String, Object> stringObjectEntry : jsonObject.entrySet()) {
                        String vendorId = stringObjectEntry.getKey();
                        List<Object> valueList = (List<Object>) stringObjectEntry.getValue();
                        if(targetServers.stream().anyMatch(cloudServer -> cloudServer.getVendorId().equals(Long.valueOf(vendorId)))){
                            valueList.add(targetServers.stream().filter(cloudServer -> cloudServer.getVendorId().equals(Long.valueOf(vendorId))).count());
                        }else{
                            valueList.add(0);
                        }
                    }
                }else{
                    cloudServerData.add(targetServers.size());
                }
                //日期加1天
                beginDate = DateUtils.addDays(beginDate, 1);
            }
            if(code.equalsIgnoreCase("省中心")){
                for (String vendorId : jsonObject.keySet()) {
                    values.add(new ChargeTrendData(list.stream().filter(cloudVendor -> cloudVendor.getId().equals(Long.valueOf(vendorId))).findFirst().get().getName(), (List<Object>) jsonObject.get(vendorId)));
                }
            }else{
                values.add(new ChargeTrendData("云主机(台)", cloudServerData));
            }
            return new GeneralResult<>(true, new ChargeResultTrend(keys, values),  "获取资源使用趋势成功");
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public GeneralResult getCloudUsage(String code, RequestContext context) {
        try {
            //获取3个月内的虚拟机数据
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("code", code), Sign.EQ));
            List<CloudVendor> list = cloudVendorRepository.list(params);
            if(ListTool.isEmpty(list)){
                return new GeneralResult(false,"未配置云厂商");
            }
            List<Long> vendorIds = list.stream().map(CloudVendor::getId).collect(Collectors.toList());
            List<CloudServer> cloudServers = cloudServerRepository.listByVenderIds(vendorIds);
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Date endDate = new Date();
            //3个月之前的日期开始到今天
            Date beginDate = DateUtils.addMonths(endDate, -3);
            List<String> keys = new ArrayList<>();
            List<Object> cpuData = new ArrayList<>();
            List<Object> memData = new ArrayList<>();
            List<Object> diskData = new ArrayList<>();
            List<ChargeTrendData> values = new ArrayList<>();
            while (beginDate.before(endDate)) {
                String key = format.format(beginDate);
                keys.add(key);
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                Date end = simpleDateFormat.parse(String.format("%s%s", key, END_TIME));
                List<CloudServer> targetServers = new ArrayList<>();
                for (CloudServer server : cloudServers) {
                    Date date = server.getGmtCreate();
                    if (date != null && date.before(end)) {
                        targetServers.add(server);
                    }
                }
                Integer cpuNum = targetServers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0)).reduce(Integer::sum).orElse(0);
                BigDecimal memNum = targetServers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.valueOf(0))).reduce(BigDecimal::add).orElse(BigDecimal.valueOf(0));
                Integer diskNum = targetServers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getDisk()).orElse(0)).reduce(Integer::sum).orElse(0);
                cpuData.add(cpuNum);
                memData.add(memNum);
                diskData.add(diskNum);
                //日期加1天
                beginDate = DateUtils.addDays(beginDate, 1);
            }
            values.add(new ChargeTrendData("CPU(C)", cpuData));
            values.add(new ChargeTrendData("内存(GB)", memData));
            values.add(new ChargeTrendData("硬盘(GB)", diskData));
            return new GeneralResult<>(true, new ChargeResultTrend(keys, values),  "获取资源使用趋势成功");
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public GeneralResult getDistributionData(String code, RequestContext context) {
        List<Param> params = new ArrayList<>();
        //根据地市code去查询云平台
        params.add(new Param(MapTools.simpleMap("code", code), Sign.EQ));
        List<CloudVendor> list = cloudVendorRepository.list(params);
        if(ListTool.isEmpty(list)){
            return new GeneralResult(false,"未配置云厂商");
        }
        List<JSONObject> cloudServerList = this.getCloudServerList(list);
        List<JSONObject> collect = cloudServerList.stream()
                .sorted(((o1, o2) -> Integer.valueOf(o2.get("云主机数量(台)").toString()).compareTo(Integer.valueOf(o1.get("云主机数量(台)").toString()))))
                .limit(5)
                .collect(Collectors.toList());
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        List<Object> cloudServerData = new ArrayList<>();
        for (JSONObject jsonObject : collect) {
            keys.add(jsonObject.get("单位名称").toString());
            cloudServerData.add(Integer.valueOf(jsonObject.get("云主机数量(台)").toString()));
        }
        values.add(new ChargeTrendData("云主机资源分布TOP5", cloudServerData));
        return new GeneralResult<>(true, new ChargeResultTrend(keys, values),  "获取资源使用趋势成功");
    }

    @Override
    public GeneralResult getUnitStatistics(String code, RequestContext context) {
        List<Param> params = new ArrayList<>();
        //根据地市code去查询云平台
        params.add(new Param(MapTools.simpleMap("code", code), Sign.EQ));
        List<CloudVendor> list = cloudVendorRepository.list(params);
        if(ListTool.isEmpty(list)){
            return new GeneralResult(false,"未配置云厂商");
        }
        List<Long> vendorIds = list.stream().map(CloudVendor::getId).collect(Collectors.toList());
        List<CloudServer> cloudServers = cloudServerRepository.listByVenderIds(vendorIds);
        List<JSONObject> jsonObjects = new ArrayList<>();
        //根据departId分组
        Map<Long, List<CloudServer>> cloudServerMap = cloudServers.stream().collect(Collectors.groupingBy(cloudServer -> cloudServer.getDepartId()));
        for (Long departId : cloudServerMap.keySet()) {
            List<CloudServer> servers = cloudServerMap.get(departId);
            Integer cpuNum = servers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0)).reduce(Integer::sum).orElse(0);
            BigDecimal memNum = servers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.valueOf(0))).reduce(BigDecimal::add).orElse(BigDecimal.valueOf(0));
            Integer diskNum = servers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getDisk()).orElse(0)).reduce(Integer::sum).orElse(0);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("单位名称", servers.get(0).getDepartName());
            jsonObject.put("云主机数量(台)", servers.size());
            jsonObject.put("VCPU(核)", cpuNum);
            jsonObject.put("内存(GB)", memNum);
            jsonObject.put("存储(TB)", diskNum);
            jsonObjects.add(jsonObject);
        }
        return new GeneralResult<>(true, jsonObjects, "获取单位统计成功");
    }

    @Override
    public GeneralResult getCity() {
        Map<String, String> map = new HashMap<>();
        map.put("410000", "河南省");
        map.put("410100", "郑州市");
        map.put("410200", "开封市");
        map.put("410300", "洛阳市");
        map.put("410400", "平顶山市");
        map.put("410500", "安阳市");
        map.put("410600", "鹤壁市");
        map.put("410700", "新乡市");
        map.put("410800", "焦作市");
        map.put("410900", "濮阳市");
        map.put("411000", "许昌市");
        map.put("411100", "漯河市");
        map.put("411200", "三门峡市");
        map.put("411300", "南阳市");
        map.put("411400", "商丘市");
        map.put("411500", "信阳市");
        map.put("411600", "周口市");
        map.put("411700", "驻马店市");
        JSONArray jsonArray = new JSONArray();
        for (String key : map.keySet()) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("name", map.get(key));
            jsonObject.put("code", key);
            jsonArray.add(jsonObject);
        }
        return new GeneralResult<>(true, jsonArray, "获取城市成功");
    }

    private List<JSONObject> getCloudServerList(List<CloudVendor> list){
        List<Long> vendorIds = list.stream().map(CloudVendor::getId).collect(Collectors.toList());
        List<CloudServer> cloudServers = cloudServerRepository.listByVenderIds(vendorIds);
        List<JSONObject> jsonObjects = new ArrayList<>();
        //根据departId分组
        Map<Long, List<CloudServer>> cloudServerMap = cloudServers.stream().collect(Collectors.groupingBy(cloudServer -> cloudServer.getDepartId()));
        for (Long departId : cloudServerMap.keySet()) {
            List<CloudServer> servers = cloudServerMap.get(departId);
            Integer cpuNum = servers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0)).reduce(Integer::sum).orElse(0);
            BigDecimal memNum = servers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.valueOf(0))).reduce(BigDecimal::add).orElse(BigDecimal.valueOf(0));
            Integer diskNum = servers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getDisk()).orElse(0)).reduce(Integer::sum).orElse(0);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("unitName", servers.get(0).getDepartName());
            jsonObject.put("cloudSeverNum", servers.size());
            jsonObject.put("cpuNum", cpuNum);
            jsonObject.put("memNum", memNum);
            jsonObject.put("storageNum", diskNum);
            jsonObjects.add(jsonObject);
        }
        return jsonObjects;
    }

    private List<Long> getCloudVendorData(List<CloudVendor> list, String regionType){
        //计算专有域云主机个数
        if(regionType.equalsIgnoreCase("ALL")){
            return list.stream().map(CloudVendor::getId).collect(Collectors.toList());
        }
        List<CloudVendor> cloudVendors = list.stream().filter(item -> item.getRegionType().equalsIgnoreCase(regionType)).collect(Collectors.toList());
        List<Long> vendorIds = cloudVendors.stream().map(CloudVendor::getId).collect(Collectors.toList());
        return vendorIds;
    }
}
