package com.bocloud.cmp.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cmp.config.DefaultSchedulingConfigurer;
import com.bocloud.cmp.constant.ScheduledTasksConstant;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.entity.SnapshotTask;
import com.bocloud.cmp.repository.CloudServerRepository;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.repository.SnapshotTaskRepository;
import com.bocloud.cmp.service.SnapshotTaskService;
import com.gitee.starblues.integration.user.PluginUser;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;

import lombok.extern.slf4j.Slf4j;

/**
 * 定时任务调整
 *
 * @author ymh
 * @date 2021/05/31
 */
@Service
@Slf4j
public class SnapshotTaskServiceImpl implements SnapshotTaskService {

	@Autowired
	private DefaultSchedulingConfigurer defaultSchedulingConfigurer;
	@Autowired
	private SnapshotTaskRepository snapshotTaskRepository;
	@Autowired
	private CloudServerRepository cloudServerRepository;
	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	@Autowired
	private PluginUser pluginUser;

	@Override
	public GeneralResult create(String taskUuid, RequestContext context) {
		try {
			SnapshotTask snapshotTask = snapshotTaskRepository.queryByTaskId(taskUuid);
			if (null == snapshotTask) {
				log.info("定时快照任务不存在,任务启动失败....");
				return new GeneralResult<>(false, "定时快照任务不存在");
			}
			CloudVendor cloudVendor = cloudVendorRepository.query(snapshotTask.getVendorId());
			if (null == cloudVendor) {
				log.info("定时快照任务所属云平台不存在");
				return new GeneralResult<>(false, "定时快照任务所属云平台不存在");
			}
			SnapshotTaskService snapshotTaskService = (SnapshotTaskService) pluginUser.getBean(cloudVendor.getType().toLowerCase(),
					"snapshotTaskService");
			if (null == snapshotTaskService) {
				log.info(cloudVendor.getType() + "插件暂未实现该接口，定时任务ID 【" + taskUuid + "】启动失败");
				return new GeneralResult<>(false, cloudVendor.getType() + "插件暂未实现该接口，定时任务ID 【" + taskUuid + "】启动失败");
			}
			return snapshotTaskService.create(taskUuid, context);
		} catch (Exception e) {
			log.error("定时快照任务不存在,任务启动失败.... ： {}", e.getMessage());
			return new GeneralResult<>(false, "添加定时快照任务失败 ： {}", e.getMessage());
		}
	}

	@Override
	public GeneralResult remove(String taskUuid) {
		try {
			log.info("taskUuid :" + taskUuid);
			SnapshotTask snapshotTask = snapshotTaskRepository.queryByTaskId(taskUuid);
			Assert.notNull(snapshotTask, "定时快照任务不存在");
			// 等待任务调度初始化完成
			while (!defaultSchedulingConfigurer.inited()) {
				Thread.sleep(5000);
			}
			String taskId = ScheduledTasksConstant.SNAPSHOT + taskUuid;
			defaultSchedulingConfigurer.cancelTriggerTask(taskId);
			log.info(snapshotTask.getServerId() + "云主机取消定时快照任务成功！");
			return new GeneralResult<>(true, "云主机取消定时快照任务成功！");
		} catch (Exception e) {
			log.error("云主机取消定时快照任务失败 ： " + e.getMessage());
			return new GeneralResult<>(false, "云主机取消定时快照任务失败 ： " + e.getMessage());
		}
	}

	@Override
	public GeneralResult removeByVmId(Long serverId) {
		try {
			log.info("serverId :" + serverId);
			CloudServer cloudServer = cloudServerRepository.query(serverId);
			Assert.notNull(cloudServer, "云主机不存在");
			List<SnapshotTask> snapshotTasks = snapshotTaskRepository.listByServerId(serverId);
			if (ListTool.isEmpty(snapshotTasks)) {
				return new GeneralResult<>(true, "定时快照任务删除成功");
			}
			for (SnapshotTask snapshotTask : snapshotTasks) {
				GeneralResult remove = this.remove(snapshotTask.getTaskId());
				if (remove.isFailed()) {
					log.error("云主机{}取消定时快照任务失败：" + remove.getMessage(), cloudServer.getName());
					continue;
				}
				snapshotTaskRepository.remove(snapshotTask.getId());
			}
			return new GeneralResult<>(true, "云主机取消定时快照任务成功！");
		} catch (Exception e) {
			log.error("云主机取消定时快照任务失败 ： " + e.getMessage());
			return new GeneralResult<>(false, "云主机取消定时快照任务失败 ： " + e.getMessage());
		}
	}

	@Override
	public GeneralResult modify(String taskUuid, RequestContext context) {
		try {
			log.info("taskUuid :" + taskUuid);
			SnapshotTask snapshotTask = snapshotTaskRepository.queryByTaskId(taskUuid);
			Assert.notNull(snapshotTask, "定时快照任务不存在");
			CloudVendor cloudVendor = cloudVendorRepository.query(snapshotTask.getVendorId());
			if (null == cloudVendor) {
				log.info("定时快照任务所属云平台不存在");
				return new GeneralResult<>(false, "定时快照任务所属云平台不存在");
			}
			SnapshotTaskService snapshotTaskService = (SnapshotTaskService) pluginUser.getBean(cloudVendor.getType().toLowerCase(),
					"snapshotTaskService");
			if (null == snapshotTaskService) {
				log.info(cloudVendor.getType() + "插件不存在，定时任务ID 【" + taskUuid + "】启动失败");
				return new GeneralResult<>(false, cloudVendor.getType() + "插件不存在，定时任务ID 【" + taskUuid + "】修改失败");
			}
			return snapshotTaskService.create(taskUuid, context);
		} catch (Exception e) {
			log.error("重置定时快照任务失败 ： " + e.getMessage());
			return new GeneralResult<>(false, "重置定时快照任务失败 ： " + e.getMessage());
		}
	}

	public GeneralResult removeByVendor(Long vendorId) {
		try {
			List<SnapshotTask> snapshotTasks = snapshotTaskRepository.listByVendorId(vendorId);
			if (ListTool.isEmpty(snapshotTasks)) {
				return new GeneralResult<>(true, "定时快照任务删除成功");
			}
			for (SnapshotTask snapshotTask : snapshotTasks) {
				GeneralResult remove = this.remove(snapshotTask.getTaskId());
				if (remove.isFailed()) {
					continue;
				}
				snapshotTaskRepository.remove(snapshotTask.getId());
			}
			return new GeneralResult<>(true, "云主机取消定时快照任务成功！");
		} catch (Exception e) {
			log.error("云主机取消定时快照任务失败 ： " + e.getMessage());
			return new GeneralResult<>(false, "云主机取消定时快照任务失败 ： " + e.getMessage());
		}
	}

}
