package com.bocloud.cmp.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.entity.Tag;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.cmp.model.VmTagParam;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.repository.TagRepository;
import com.bocloud.cmp.service.TagService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.framework.lock.LockFactory;
import com.google.common.collect.Maps;

import lombok.extern.slf4j.Slf4j;

@Service("tagService")
@Slf4j
public class TagServiceImpl implements TagService {
	
	@Autowired
	private TagRepository tagRepository;

	@Override
	public GeneralResult<GridBean<Tag>> list(Pager pager) {
		try {
			int page = pager.getPage();
			int rows = pager.getRows();
			List<Param> params = pager.getParams();
			Map<String, String> sorter = pager.getSorter();

			params = Optional.ofNullable(params).orElse(new ArrayList<>());
			sorter = Optional.ofNullable(sorter).orElse(Maps.newHashMap());
			sorter.put("gmtCreate", Common.ONE);
			int total = tagRepository.count(params);
			List<Tag> list = tagRepository.list(page, rows, params, sorter);
			GridBean gridBean = GridHelper.getBean(page, rows, total, list);
			return new GeneralResult<>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("list tag failure:", e);
			return new GeneralResult<>(false, "查询失败", null, null);
		}
	}

	@Override
	public GeneralResult create(VmTagParam vmTagParam) {
		return null;
	}

	private List<TagModel> getTagModels(List<Tag> tags) {
		List<TagModel> tagModels = new ArrayList<>();
		for (Tag tag : tags) {
			tagModels.add(new TagModel(tag.getKey(), tag.getValue(), tag.getResourceId()));
		}
		return tagModels;
	}

	@Override
	public GeneralResult remove(Long id) {
		return null;
	}
}
