package com.bocloud.cmp.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cmp.entity.ResourceModule;
import com.bocloud.cmp.entity.VendorCategory;
import com.bocloud.cmp.repository.ResourceModuleRepository;
import com.bocloud.cmp.repository.VendorCategoryRepository;
import com.bocloud.cmp.service.VendorCategoryService;
import com.google.common.collect.Maps;
import com.megatron.common.enums.BaseStatus;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 平台类型 实现类
 */
@Service("vendorCategoryService")
@Slf4j
public class VendorCategoryServiceImpl implements VendorCategoryService {

	@Autowired
	private VendorCategoryRepository vendorCategoryRepository;
	@Autowired
	private LockFactory lockFactory;
	@Autowired
	private ResourceModuleRepository resourceModuleRepository;

	@Override
	public GeneralResult<VendorCategory> create(VendorCategory vendorCategory, RequestContext context) {
		try {
			boolean flag = vendorCategoryRepository.isNameRepeat(vendorCategory.getName());
			if (flag) {
				return new GeneralResult<>(false, "名称已存在");
			}
			vendorCategory.setCreatorId(context.getTarget());
			vendorCategory.setMenderId(context.getTarget());
			vendorCategory.setStatus(BaseStatus.NORMAL.name());
			vendorCategory.setParentId(0L);
			vendorCategoryRepository.save(vendorCategory);
			log.info("name : " + vendorCategory.getName());
			return new GeneralResult<>(true, vendorCategory, "添加成功");
		} catch (Exception e) {
			log.error("create vendor category fail:", e);
			return new GeneralResult<>(false, e.getMessage(), "", "");
		}
	}

	@Override
	public GeneralResult<VendorCategory> modify(VendorCategory vendorCategory, RequestContext context) {
		try {
			boolean flag = vendorCategoryRepository.isNameRepeat(vendorCategory.getName());
			if (flag) {
				return new GeneralResult<>(false, "名称已存在");
			}
			VendorCategory dbVednorCategory = vendorCategoryRepository.query(vendorCategory.getId());
			if (null == dbVednorCategory) {
				return new GeneralResult<>(false, "数据不存在");
			}
			dbVednorCategory.setName(vendorCategory.getName());
			dbVednorCategory.setRemark(vendorCategory.getRemark());
			vendorCategoryRepository.update(dbVednorCategory);
			return new GeneralResult<>(true, vendorCategory, "编辑成功");
		} catch (Exception e) {
			log.error("create vendor category fail:", e);
			return new GeneralResult<>(false, e.getMessage(), "", "");
		}
	}

	@Override
	public GeneralResult remove(Long id, RequestContext context) {
		String path = VendorCategory.class.getSimpleName() + "_" + id;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			VendorCategory vendorCategory = vendorCategoryRepository.query(id);
			if (null == vendorCategory) {
				return new GeneralResult(true, "删除成功");
			}
			List<ResourceModule> resourceModules = resourceModuleRepository.listByCategoryId(vendorCategory.getId());
			if (!ListTool.isEmpty(resourceModules)) {
				return new GeneralResult(false, "该类型下存在资源模型，不允许删除");
			}
			vendorCategoryRepository.remove(id, context.getTarget());
			return new GeneralResult<>(true, "删除成功");
		} catch (Exception e) {
			log.error("Remove data center fail:", e);
			return new GeneralResult<>(false, "删除失败", null, null);
		}
	}

	@Override
	public GeneralResult<GridBean<VendorCategory>> list(Pager pager, RequestContext context) {
		List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
		Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
		sorter.put("gmtCreate", Common.ONE);
		Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
		Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
		List<VendorCategory> list = null;
		int total = 0;
		GridBean gridBean = null;
		try {
			total = this.vendorCategoryRepository.count(params);
			list = this.vendorCategoryRepository.list(page, rows, params, sorter);
			gridBean = GridHelper.getBean(page, rows, total, list);
			return new GeneralResult<>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("Query vendor category list fail:", e);
			return new GeneralResult<>(false, "查询失败", null, null);
		}
	}

	@Override
	public GeneralResult<VendorCategory> detail(Long id) {
		// TODO Auto-generated method stub
		return null;
	}

}