package com.bocloud.cmp.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.AlterationParam;
import com.bocloud.cmp.domain.ApplicationParam;
import com.bocloud.cmp.domain.CancelationParam;
import com.bocloud.cmp.domain.ExtensionParam;
import com.bocloud.cmp.entity.Business;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.entity.DataStore;
import com.bocloud.cmp.entity.Project;
import com.bocloud.cmp.entity.Region;
import com.bocloud.cmp.entity.Snapshot;
import com.bocloud.cmp.entity.Tenant;
import com.bocloud.cmp.entity.User;
import com.bocloud.cmp.entity.Volume;
import com.bocloud.cmp.internal.SmsBusinessService;
import com.bocloud.cmp.internal.SmsProjectService;
import com.bocloud.cmp.internal.SmsTenantService;
import com.bocloud.cmp.internal.SmsUserService;
import com.bocloud.cmp.repository.CloudServerRepository;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.repository.DataStoreRepository;
import com.bocloud.cmp.repository.DictionaryRepository;
import com.bocloud.cmp.repository.RegionRepository;
import com.bocloud.cmp.repository.VolumeRepository;
import com.bocloud.cmp.service.VolumeService;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.common.utils.ResultTools;

import lombok.extern.slf4j.Slf4j;

/**
 * 块设备Service接口实现
 *
 * @author weiwei
 */
@Service("volumeService")
@Slf4j
public class VolumeServiceImpl implements VolumeService {

    @Autowired
    private VolumeRepository volumeRepository;
    @Autowired
    private CloudVendorRepository cloudVendorRepository;
    @Autowired
    private RegionRepository regionRepository;
    @Autowired
    private SmsUserService smsUserService;
    @Autowired
    private CloudServerRepository cloudServerRepository;
    @Autowired
    private SmsProjectService smsProjectService;
    @Autowired
    private SmsBusinessService smsBusinessService;
    @Autowired
    private SmsTenantService smsTenantService;
    @Autowired
    private DataStoreRepository dataStoreRepository;
    @Autowired
	private DictionaryRepository dictionaryRepository;

    @Override
    public GeneralResult<GridBean<Volume>> list(Pager pager, RequestContext context) {

        int page = pager.getPage();
        int rows = pager.getRows();
        List<Param> params = pager.getParams();
        Map<String, String> sorter = pager.getSorter();
        boolean simple = pager.getSimple();

        List<Volume> list = Lists.newArrayList();
        int total = 0;
        GridBean gridBean = null;
        try {
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            Long tenantId = context.getTenant();

            String vendorType = null;
            String labelContent = null;
            for (Param param : params) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("vendorType")) {
                    vendorType = String.valueOf(paramMap.get("vendorType"));
                    paramMap.remove("vendorType");
                }
                if (paramMap.containsKey("labelContent")) {
                    labelContent = paramMap.get("labelContent").toString();
                    paramMap.remove("labelContent");
                }
            }
            if (context.getCatalog().equals(Catalog.Tenant)) {
                // 租户能看到 1.属于当前租户的资源 2.租户下所有项目的资源
                List<Project> tenantProjects = smsProjectService.listTenantProjects(context.getTenant());
                List<Long> projectIds = tenantProjects.stream().map(Project::getId).collect(Collectors.toList());
                total = this.volumeRepository.count(null, context.getTenant(), projectIds, params, vendorType, labelContent);
                list = this.volumeRepository.list(page, rows, null, context.getTenant(), projectIds, params, sorter,
                        vendorType, labelContent);
                list = convertor(list);
                gridBean = new GridBean<>(page, rows, total, list);
            } else if (context.getCatalog().equals(Catalog.User)) {
                // 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
                List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
                List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
                List<Long> projectIds = new ArrayList<>();
                List<Long> userManagerProjectsIds = userManagerProjects.stream().map(Project::getId)
                        .collect(Collectors.toList());
                if (!userManagerProjectsIds.isEmpty()) {
                    projectIds.addAll(userManagerProjectsIds);
                }
                List<Long> visibleProjectsIds = visibleProjects.stream().map(Project::getId)
                        .collect(Collectors.toList());
                if (!visibleProjectsIds.isEmpty()) {
                    projectIds.addAll(visibleProjectsIds);
                }
                total = this.volumeRepository.count(context.getTarget(), null, projectIds, params, vendorType, labelContent);
                list = this.volumeRepository.list(page, rows, context.getTarget(), null, projectIds, params, sorter,
                        vendorType, labelContent);
                list = convertor(list);
                gridBean = new GridBean<>(page, rows, total, list);
            } else {
                total = volumeRepository.countByExistsLabel(params, labelContent);
                if (simple) {
                    if (0 == tenantId) {
                        params.add(new Param(MapTools.simpleMap(Common.TENANTID, 0), Sign.EQ));
                    }
                    List<SimpleBean> beans = volumeRepository.list(params, sorter, labelContent);
                    gridBean = new GridBean(1, 1, total, beans);
                } else {
                    list = volumeRepository.list(page, rows, params, sorter, labelContent);
                    for (Volume volume : list) {
                        if (null != volume.getDataStoreId()) {
                            Optional.ofNullable(dataStoreRepository.query(volume.getDataStoreId()))
                                    .map(DataStore::getStoreVal)
                                    .ifPresent(volume::setDataStoreUuid);
                        }
                    }
                    gridBean = GridHelper.getBean(page, rows, total, list);
                }
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("list volume failure:", e);
            return new GeneralResult<>(false, "查询失败", null, null);
        }
    }


    private List<Volume> convertor(List<Volume> list) {
        try {
            if (!ListTool.isEmpty(list)) {
                List<Long> tenantIds = new ArrayList<>();
                List<Long> userIds = new ArrayList<>();
                List<Long> businessIds = new ArrayList<>();
                List<Long> projectIds = new ArrayList<>();
                for (Volume volume : list) {
                    if (volume.getTenantId() != null) {
                        tenantIds.add(volume.getTenantId());
                    }
                    if (volume.getBusinessId() != null) {
                        businessIds.add(volume.getBusinessId());
                    }
                    if (volume.getProjectId() != null) {
                        projectIds.add(volume.getProjectId());
                    }
                    if (volume.getCreatorId() != null) {
                        userIds.add(volume.getCreatorId());
                    }
                    if (volume.getOwnerId() != null) {
                        userIds.add(volume.getOwnerId());
                    }
                    if (volume.getMenderId() != null) {
                        userIds.add(volume.getMenderId());
                    }
                }
                Map<Long, String> userMap = smsUserService.listUsersByIds(userIds).stream()
                        .collect(Collectors.toMap(User::getId, User::getName));
                Map<Long, String> projectMap = smsProjectService.listProjectsByIds(projectIds).stream()
                        .collect(Collectors.toMap(Project::getId, Project::getName));
                Map<Long, String> businessMap = smsBusinessService.listBusinesssByIds(businessIds).stream()
                        .collect(Collectors.toMap(Business::getId, Business::getName));
                Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(tenantIds).stream()
                        .collect(Collectors.toMap(Tenant::getId, Tenant::getName));
                for (Volume volume : list) {
                    // 设置云主机归属信息
                    if (volume.getTenantId() != null) {
                        volume.setTenantName(tenantMap.get(volume.getTenantId()));
                    }
                    if (volume.getBusinessId() != null) {
                        volume.setBusinessName(businessMap.get(volume.getBusinessId()));
                    }
                    if (volume.getProjectId() != null) {
                        volume.setProjectName(projectMap.get(volume.getProjectId()));
                    }
                    if (volume.getCreatorId() != null) {
                        volume.setCreatorName(userMap.get(volume.getCreatorId()));
                    }
                    if (volume.getOwnerId() != null) {
                        volume.setOwnerName(userMap.get(volume.getOwnerId()));
                    }
                    if (volume.getMenderId() != null) {
                        volume.setMenderName(userMap.get(volume.getMenderId()));
                    }
                }
            }
        } catch (Exception e) {
            log.error("获取云硬盘基本信息异常：" + e.getMessage());
        }
        return list;

    }




    @Override
    public GeneralResult<Volume> detail(Long id) {
        try {
            Volume volume = volumeRepository.query(id);
            if (null == volume) {
                log.warn("volume data not exist");
                return new GeneralResult<>(false, "数据不存在");
            }
            // 设置云主机归属
            if (null != volume.getOwnerId() && 0L != volume.getOwnerId()) {
                GeneralResult<User> ownerDetail = smsUserService.detail(volume.getOwnerId());
                User owner = ownerDetail.getData();
                if (owner != null) {
                    volume.setOwnerName(owner.getName());
                }
            }
            if (null != volume.getMenderId() && 0L != volume.getMenderId()) {
                GeneralResult<User> menderDetail = smsUserService.detail(volume.getMenderId());
                User mender = menderDetail.getData();
                if (mender != null) {
                    volume.setMenderName(mender.getName());
                }
            }
            if (null != volume.getCreatorId() && 0L != volume.getCreatorId()) {
                GeneralResult<User> creatorDetail = smsUserService.detail(volume.getCreatorId());
                User creator = creatorDetail.getData();
                if (creator != null) {
                    volume.setCreatorName(creator.getName());
                }
            }
            if (null != volume.getProjectId() && 0L != volume.getProjectId()) {
                GeneralResult<Project> projectResult = smsProjectService.detail(volume.getProjectId());
                if (projectResult.isFailed() || null == projectResult.getData()) {
                    return new GeneralResult(false, projectResult.getMessage());
                }
                Project project = projectResult.getData();
                if (null != project) {
                    volume.setProjectName(project.getName());
                }
            }
            if (null != volume.getBusinessId() && 0L != volume.getBusinessId()) {
                GeneralResult<Business> businessDetail = smsBusinessService.detail(volume.getBusinessId());
                Business business = businessDetail.getData();
                if (business != null) {
                    volume.setBusinessName(business.getName());
                }
            }

            // 云供应商
            CloudVendor vendor = cloudVendorRepository.query(volume.getVendorId());
            if (null == vendor) {
                log.error("get cloudVendor failure");
                return new GeneralResult<>(false, "平台查询失败");
            }
            Region region = regionRepository.getByRegionId(volume.getRegionId(), volume.getVendorId());
            if (null != region) {
                volume.setRegionName(region.getName());
            }
            
			String regionName = dictionaryRepository.getNameByValue(volume.getRegionId());
			if (!StringUtils.isEmpty(regionName)) {
				volume.setRegionName(regionName);
			}
			String azName = dictionaryRepository.getNameByValue(volume.getZone());
			if (!StringUtils.isEmpty(azName)) {
				volume.setZoneName(azName);
			}
            
            return new GeneralResult<>(true, volume, "查询详情成功");
        } catch (Exception e) {
            log.error("query volume detail fail:", e);
            return new GeneralResult<>(false, "查询详情失败");
        }
    }

    @Override
    public GeneralResult<List<CloudServer>> servers(Long id) {
        try {
            List<CloudServer> servers = cloudServerRepository.listByVolume(id);
            return new GeneralResult<>(true, servers, "查询成功");
        } catch (Exception e) {
            log.error("list servers fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<Volume>> listByStatus(String params, RequestContext context) {
        try {
            JSONObject jsonObject = JSONTools.isJSONObj(params);
            if (null == jsonObject) {
                return new GeneralResult<>(false, "参数异常");
            }
            // 云平台ID
            Long vendorId = jsonObject.getLong("vendorId");
            CloudVendor cloudVendor = cloudVendorRepository.query(vendorId);
            Assert.notNull(cloudVendor, "云平台不存在");
            // 状态
            String status = jsonObject.getString("status");
            List<String> statusList = JSONArray.parseArray(status, String.class);
            Long projectId = context.getProject();
            List<Volume> volumes = volumeRepository.list(vendorId, statusList, null, null, projectId,
                    jsonObject.getString("tenantUuid"), jsonObject.getString("regionId"));
            return new GeneralResult<>(true, volumes, "查询成功");
        } catch (Exception e) {
            log.error("list volume by status error：", e);
            return new GeneralResult<>(false, "查询失败 :" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<List<Volume>> listByVendor(Long vendorId) {
        try {
            List<Volume> volumes = volumeRepository.listByVendorId(vendorId);
            return new GeneralResult<>(true, volumes, "查询volume成功!");
        } catch (Exception e) {
            log.error("list volume by status error：", e);
            return new GeneralResult<>(false, "查询volume失败!");
        }
    }

    @Override
    public GeneralResult<List<Volume>> listByImage(Long imageId) {
        try {
            List<Volume> volumes = Lists.newArrayList();
            // List<ImageVolume> imageVolumes =
            // imageVolumeRepository.listByImage(imageId);
            // for (ImageVolume imageVolume : imageVolumes) {
            // Volume volume =
            // volumeRepository.query(imageVolume.getVolumeId());
            // if (volume != null) {
            // volumes.add(volume);
            // }
            // }
            return new GeneralResult<>(true, volumes, "查询volume成功!");
        } catch (Exception e) {
            log.error("list volume by image id error：", e);
            return new GeneralResult<>(false, "查询volume失败!");
        }
    }

    @Override
    public GeneralResult<String> batchRemove(List<Long> ids, RequestContext context) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult<String> uploadToImage(String params, RequestContext context) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult<Integer> count(Long userId, Long projectId, Long tenantId) {
        try {
            List<Param> params = Lists.newArrayList();
            Catalog catalog = Catalog.Tenant;
            Long target = tenantId;
            if (null != userId) {
                target = userId;
                catalog = Catalog.User;
            }
            List<Project> projects = smsProjectService.projectsByCondition(target, catalog);
            List<Long> projectIds = new ArrayList<>();
            // 此时查询的是全部用户全部项目的信息
            if (null != projectId && 0L != projectId) {
                projectIds.add(projectId);
            } else if (!ListTool.isEmpty(projects)) {
                projects.forEach(project -> projectIds.add(project.getId()));
            } else {
                // 当前用户不属于任何项目，则登录之后查看不到任何信息
                projectIds.add(-1L);
            }
            Integer total = this.volumeRepository.count(params, null, projectIds);
            return new GeneralResult<>(true, total, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult attach(Long id, Long serverId, RequestContext context, String mountpoint) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult<String> onboard(String params, RequestContext context) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult<String> unmanage(RequestContext context, Long vendorId, Long storageId, List<Long> volumeIds) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult<List<Volume>> listByStorage(Long storageId) {
        try {
            List<Volume> volumes = volumeRepository.listByStorageId(storageId);
            return new GeneralResult<>(true, volumes, "查询volume成功!");
        } catch (Exception e) {
            log.error("list volume by status error：", e);
            return new GeneralResult<>(false, "查询volume失败!");
        }
    }

    @Override
    public GeneralResult<List<Volume>> listByTemplate(Long templateId) {
        try {
            List<Volume> volumes = volumeRepository.listByTemplateId(templateId);
            return new GeneralResult<>(true, volumes, "查询volume成功!");
        } catch (Exception e) {
            log.error("list volume by status error：", e);
            return new GeneralResult<>(false, "查询volume失败!");
        }
    }


    @Override
    public GeneralResult<GridBean> listByHost(Integer page, Integer rows, Long hostId, Map<String, Object> paramMap,
                                              Map<String, String> sorter) {
        try {
            List<Volume> volumes = volumeRepository.listByHost(page, rows, hostId, paramMap, sorter);
            int total = volumeRepository.countByHost(hostId, paramMap);
            GridBean gridBean = new GridBean(page, rows, total, volumes);
            return new GeneralResult<>(true, gridBean, "查询详情成功");
        } catch (Exception e) {
            log.error("get volumes fail:", e);
            return new GeneralResult<>(false, "查询详情失败", "", "");
        }
    }

    @Override
    public GeneralResult<GridBean> listInSyncedSet(Integer page, Integer rows, Map<String, Object> paramMap,
                                                   Map<String, String> sorter) {
        try {
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            List<Volume> list = volumeRepository.listInSyncedSet(page, rows, paramMap, sorter);
            int total = volumeRepository.countListInSyncedSet(paramMap);
            GridBean gridBean = new GridBean(page, rows, total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<GridBean> listNotInSyncedSet(Integer page, Integer rows, Map<String, Object> paramMap,
                                                      Map<String, String> sorter) {
        try {
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            List<Volume> list = volumeRepository.listNotInSyncedSet(page, rows, paramMap, sorter);
            int total = volumeRepository.countListNotInSyncedSet(paramMap);
            GridBean gridBean = new GridBean(page, rows, total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<GridBean> listForPowervcVMCreate(int page, int rows, String keyword, long vendorId) {
        try {
            List<Volume> volumes = volumeRepository.listForPowervcVMCreate(page, rows, keyword, vendorId);
            int total = volumeRepository.countForPowervcVMCreate(keyword, vendorId);
            GridBean gridBean = new GridBean(page, rows, total, volumes);
            return new GeneralResult<>(true, gridBean, "查询volume成功!");
        } catch (Exception e) {
            log.error("list volume by status error：", e);
            return new GeneralResult<>(false, "查询volume失败!");
        }
    }

    @Override
    public GeneralResult<GridBean> listForPowervcVMAttach(int page, int rows, String keyword, long vendorId,
                                                          long vmId) {
        try {
            List<Volume> volumes = volumeRepository.listForPowervcVMAttach(page, rows, keyword, vendorId, vmId);
            int total = volumeRepository.countForPowervcVMAttach(keyword, vendorId, vmId);
            GridBean gridBean = new GridBean(page, rows, total, volumes);
            return new GeneralResult<>(true, gridBean, "查询volume成功!");
        } catch (Exception e) {
            log.error("list volume by status error：", e);
            return new GeneralResult<>(false, "查询volume失败!");
        }
    }

    @Override
    public GeneralResult<GridBean> listForPowervcImageCreate(int page, int rows, String keyword, long vendorId) {
        try {
            List<Volume> volumes = volumeRepository.listForPowervcImageCreate(page, rows, keyword, vendorId);
            int total = volumeRepository.countForPowervcImageCreate(keyword, vendorId);
            GridBean gridBean = new GridBean(page, rows, total, volumes);
            return new GeneralResult<>(true, gridBean, "查询volume成功!");
        } catch (Exception e) {
            log.error("list volume by status error：", e);
            return new GeneralResult<>(false, "查询volume失败!");
        }
    }

    @Override
    public GeneralResult<String> apply(ApplicationParam param) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult alteration(AlterationParam param) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult extension(ExtensionParam param) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult cancelation(CancelationParam param) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult<List<Volume>> listByCondition(JSONObject object, RequestContext context) {
        try {
            List<Volume> volumes = volumeRepository.listByCondition(object.getLong("vendorId"), object.getLong("vdcId"),
                    object.getLong("clusterId"));
            return new GeneralResult<>(true, volumes, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e.getMessage());
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<List<Volume>> listByVmId(JSONObject object) {
        try {
            List<Volume> volumes = volumeRepository.listByServerId(object.getLong("serverId"));
            return new GeneralResult<>(true, volumes, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e.getMessage());
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<List<Volume>> listDataDisks(JSONObject object) {
        try {
            List<Volume> volumes = volumeRepository.listByServerId(object.getLong("serverId"));
            List<Volume> filterVolumes = new ArrayList<>();
            for (Volume volume : volumes) {
                if (!"Hard disk 1".equalsIgnoreCase(volume.getName()) || !"硬盘 1".equalsIgnoreCase(volume.getName())) {
                    filterVolumes.add(volume);
                }
            }
            return new GeneralResult<>(true, filterVolumes, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e.getMessage());
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult createOsVolume(JSONObject param, RequestContext context) {
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult<Object> listBackupResources(String params, RequestContext context) {
        try {
            JSONObject object = JSONTools.isJSONObj(params);
            Long vendorId = object.getLong("vendorId");
            String regionId = object.getString("regionId");
            String type = object.getString("type");
            if (null == regionId) {
                return new GeneralResult<>(false, "参数格式错误");
            }
            CloudVendor cloudVendor = cloudVendorRepository.query(vendorId);
            if (null == cloudVendor) {
                return new GeneralResult<>(false, "平台认证失败！");
            }
            if ("disk".equalsIgnoreCase(type)) {
                return new GeneralResult<>(true, volumeRepository.listBackupResources(vendorId, regionId), "查询成功");
            } else {
                return new GeneralResult<>(true, cloudServerRepository.listBackupResources(vendorId, regionId), "查询成功");
            }
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<GridBean<Snapshot>> listSnapshotById(Long id, Pager pager) {
    	return ResultTools.noImplementedResult();
    }

	@Override
	public GeneralResult listVolumeNotInView(Integer page, Integer rows, Long accessPathId, Long viewUuid,
			List<Param> paramMap, Map<String, String> sorter) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public GeneralResult listVolumeInView(Integer page, Integer rows, Long accessPathId, Long viewUuid,
			List<Param> paramList, Map<String, String> sorter) {
		// TODO Auto-generated method stub
		return null;
	}


}
